@@ -23,13 +23,10 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "ekg2-config.h"
-#include "win32.h"
+#include "ekg2.h"
+
+#include <glib/gprintf.h>
 
-#ifndef __FreeBSD__
-#define _XOPEN_SOURCE 600
-#define __EXTENSIONS__
-#endif
 #include <sys/types.h>
 
 #ifndef NO_POSIX_SYSTEM
@@ -37,7 +34,6 @@
 #endif
 
 #include <sys/stat.h>
-#define __USE_BSD
 #include <sys/time.h>
 
 #ifndef NO_POSIX_SYSTEM
@@ -53,11 +49,6 @@
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#ifdef HAVE_GETOPT_LONG
-#  include <getopt.h>
-#else
-#  include "compat/getopt.h"
-#endif
 #include <limits.h>
 #include <locale.h>
 
@@ -81,36 +72,14 @@
 #include <time.h>
 #include <unistd.h>
 
-#include "audio.h"
-#include "bindings.h"
-#include "commands.h"
-#include "debug.h"
-#include "events.h"
-#include "configfile.h"
 #include "emoticons.h"
-#include "log.h"
-#include "metacontacts.h"
-#include "msgqueue.h"
-#include "plugins.h"
-#include "protocol.h"
-#ifndef HAVE_STRLCPY
-#  include "compat/strlcpy.h"
-#endif
-#include "sessions.h"
-#include "stuff.h"
-#include "themes.h"
-#include "userlist.h"
+#include "internal.h"
 #include "scripts.h"
-#include "vars.h"
-#include "windows.h"
-#include "xmalloc.h"
-
-#include "queries.h"
 
 char *config_dir;
 int mesg_startup;
-int ekg_watches_removed;
 static char argv0[PATH_MAX];
+static gchar last_err_message[128] = {0};
 
 pid_t speech_pid = 0;
 
@@ -173,177 +142,10 @@ static TIMER(ekg_autoaway_timer) {
  */
 
 void ekg_loop() {
-	struct timeval tv;
-	struct timeval stv;
-	fd_set rd, wd;
-	int ret, maxfd, status;
-	pid_t pid;
-
-	gettimeofday(&tv, NULL);
-
+	g_main_context_iteration(NULL, FALSE);
 	{
-		{		/* przejrzyj timery uytkownika, ui, skryptw */
-			struct timer *t;
-			
-			for (t = timers; t; t = t->next) {
-				if (tv.tv_sec > t->ends.tv_sec || (tv.tv_sec == t->ends.tv_sec && tv.tv_usec >= t->ends.tv_usec)) {
-					int ispersist = t->persist;
-					
-					if (ispersist) {
-						memcpy(&t->ends, &tv, sizeof(tv));
-						t->ends.tv_sec += (t->period / 1000);
-						t->ends.tv_usec += ((t->period % 1000) * 1000);
-						if (t->ends.tv_usec >= 1000000) {
-							t->ends.tv_usec -= 1000000;
-							t->ends.tv_sec++;
-						}
-					}
-
-					if ((t->function(0, t->data) == -1) || !ispersist)
-						t = timers_removei(t);
-				}
-			}
-		}
-
-		{		/* removed 'w->removed' watches, timeout checking moved below select() */
-			list_t l;
-
-			for (l = watches; l; l = l->next) {
-				watch_t *w = l->data;
-
-				if (w && w->removed == 1) {
-					w->removed = 0;
-					watch_free(w);
-				}
-			}
-		}
-
-		/* auto save */
-		if (config_auto_save && config_changed && (tv.tv_sec - last_save) > config_auto_save) {
-			debug("autosaving userlist and config after %d seconds\n", tv.tv_sec - last_save);
-			last_save = tv.tv_sec;
-
-			if (!config_write(NULL) && !session_write()) {
-				config_changed = 0;
-				ekg2_reason_changed = 0;
-				print("autosaved");
-			} else
-				print("error_saving");
-		}
-
-		/* przegldanie zdechych dzieciakw */
-#ifndef NO_POSIX_SYSTEM
-		while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
-			child_t *c;
-			debug("child process %d exited with status %d\n", pid, WEXITSTATUS(status));
-
-			for (c = children; c; c = c->next) {
-				if (pid != c->pid)
-					continue;
-
-				if (pid == speech_pid) {
-					speech_pid = 0;
-
-					if (!config_speech_app)
-						buffer_free(&buffer_speech);
-
-					if (buffer_speech.count && !WEXITSTATUS(status)) {
-						char *str = buffer_tail(&buffer_speech);
-						say_it(str);
-						xfree(str);
-					}
-				}
-
-				if (c->handler)
-					c->handler(c, c->pid, c->name, WEXITSTATUS(status), c->priv_data);
-
-				c = children_removei(c);
-			}
-		}
-#endif
-		/* zerknij na wszystkie niezbdne deskryptory */
-
-		FD_ZERO(&rd);
-		FD_ZERO(&wd);
-
-		{
-			list_t l;
-
-			for (maxfd = 0, l = watches; l; l = l->next) {
-				watch_t *w = l->data;
-				if (!w)
-					continue;
-
-				if (w->fd > maxfd)
-					maxfd = w->fd;
-				if ((w->type & WATCH_READ))
-					FD_SET(w->fd, &rd);
-				if ((w->type & WATCH_WRITE)) {
-					if (w->buf && !w->buf->len) continue; /* if we have WATCH_WRITE_LINE and there's nothink to send, ignore this */ 
-					FD_SET(w->fd, &wd); 
-				}
-			}
-		}
-
-		stv.tv_sec = 1;
-		stv.tv_usec = 0;
-		{
-			struct timer *t;
-
-			for (t = timers; t; t = t->next) {
-				int usec = 0;
-
-				/* zeby uniknac przekrecenia licznika mikrosekund przy
-				 * wiekszych czasach, pomijamy dlugie timery */
-				if (t->ends.tv_sec - tv.tv_sec > 1)
-					continue;
-
-				/* zobacz, ile zostalo do wywolania timera */
-				usec = (t->ends.tv_sec - tv.tv_sec) * 1000000 + (t->ends.tv_usec - tv.tv_usec);
-
-				/* jesli wiecej niz sekunda, to nie ma znacznia */
-				if (usec >= 1000000)
-					continue;
-				
-				/* jesli mniej niz aktualny timeout, zmniejsz */
-				if (stv.tv_sec * 1000000 + stv.tv_usec > usec) {
-					stv.tv_sec = 0;
-					stv.tv_usec = usec;
-				}
-			}
-		}
-
-		/* na wszelki wypadek sprawd wartoci */
-		if (stv.tv_sec != 1)
-			stv.tv_sec = 0;
-		if (stv.tv_usec < 0)
-			stv.tv_usec = 1;
-
-		/* sprawd, co si dzieje */
-		ret = select(maxfd + 1, &rd, &wd, NULL, &stv);
-
-		/* jeli wystpi bd, daj zna */
-		if (ret == -1) {
-			/* jaki plugin doda do watchw zy deskryptor. eby
-			 * ekg mogo dziaa dalej, sprawdmy ktry to i go
-			 * usumy z listy. */
-			if (errno == EBADF) {
-				list_t l;
-
-				for (l = watches; l; l = l->next) {
-					watch_t *w = l->data;
-					struct stat st;
-
-					if (w && fstat(w->fd, &st)) {
-						debug("select(): bad file descriptor: fd=%d, type=%d, plugin=%s\n", w->fd, w->type, (w->plugin) ? w->plugin->name : ("none"));
-						watch_free(w);
-					}
-				}
-			} else if (errno != EINTR)
-				debug("select() failed: %s\n", strerror(errno));
-			return;
-		}
 
+#ifdef WATCHES_FIXME
 		{		/* przejrzyj deskryptory */
 			list_t l;
 
@@ -384,28 +186,14 @@ void ekg_loop() {
 						if (!s->connected || !s->autoaway)
 							continue;
 
-						if (session_int_get(s, "auto_back") != 2)
-							continue;
-
-						command_exec(NULL, s, ("/_autoback"), 2);
+						if (session_int_get(s, "auto_back") == 2)
+							command_exec(NULL, s, ("/_autoback"), 2);
 					}
 				}
-				if (!w->buf) {
-					if (((w->type == WATCH_WRITE) && FD_ISSET(w->fd, &wd)) ||
-							((w->type == WATCH_READ) && FD_ISSET(w->fd, &rd)))
-						watch_handle(w);
-				} else {
-					if (FD_ISSET(w->fd, &rd) && w->type == WATCH_READ)		watch_handle_line(w);
-					else if (FD_ISSET(w->fd, &wd) && w->type == WATCH_WRITE)	watch_handle_write(w);
-				}
 			}
 		}
+#endif
 
-		if (ekg_watches_removed > 0) {
-			debug("ekg_loop() Removed %d watches this loop, let's cleanup calling: list_cleanup() ...\n", ekg_watches_removed);
-			list_cleanup(&watches);
-			ekg_watches_removed = 0;
-		}
 	}
 #undef tv
 
@@ -415,14 +203,14 @@ void ekg_loop() {
 static void handle_sigusr1()
 {
 	debug("sigusr1 received\n");
-	query_emit_id(NULL, EKG_SIGUSR1);
+	query_emit(NULL, "ekg-sigusr1");
 	signal(SIGUSR1, handle_sigusr1);
 }
 
 static void handle_sigusr2()
 {
 	debug("sigusr2 received\n");
-	query_emit_id(NULL, EKG_SIGUSR2);
+	query_emit(NULL, "ekg-sigusr2");
 	signal(SIGUSR2, handle_sigusr2);
 }
 
@@ -431,9 +219,64 @@ static void handle_sighup()
 	ekg_exit();
 }
 
+static void handle_sigabrt()
+{
+	GSList *pl;
+
+	signal(SIGABRT, SIG_DFL);
+
+	if (stderr_backup && stderr_backup != -1)
+		dup2(stderr_backup, 2);
+
+	/* wycz pluginy ui, eby odday terminal
+	 * destroy also log plugins to make sure that latest changes are written */
+	for (pl = plugins; pl; pl = pl->next) {
+		const plugin_t *p = pl->data;
+		if (p->pclass != PLUGIN_UI && p->pclass != PLUGIN_LOG)
+			continue;
+
+		p->destroy();
+	}
+
+	fprintf(stderr,
+"\r\n"
+"\r\n"
+"*** Abnormal program termination ***\r\n"
+"\r\n"
+"%s"
+"\r\n"
+"The program will attempt to write its settings, but it is not\r\n"
+"guaranteed to succeed. They will be saved as\r\n"
+"%s/crash-%d-config,\r\n"
+"%s/crash-%d-config-<plugin>\r\n"
+"and %s/crash-%d-userlist\r\n"
+"\r\n"
+"Last messages from the debugging window will be saved to a file called\r\n"
+"%s/crash-%d-debug.\r\n"
+"\r\n"
+"If a file called %s/core will be created, try running the following\r\n"
+"command:\r\n"
+"\r\n"
+"    gdb %s %s/core\r\n"
+"\n"
+"note the last few lines, and then note the output from the ,,bt'' command.\r\n"
+"This will help the program authors find the location of the problem\r\n"
+"and most likely will help avoid such crashes in the future.\r\n"
+"More details can be found in the documentation, in the file ,,gdb.txt''.\r\n"
+"\r\n",
+last_err_message, config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, argv0, config_dir);
+
+	config_write_crash();
+	userlist_write_crash();
+	debug_write_crash();
+	config_commit();
+
+	raise(SIGABRT);
+}
+
 static void handle_sigsegv()
 {
-	plugin_t *p;
+	GSList *pl;
 
 	signal(SIGSEGV, SIG_DFL);
 
@@ -442,7 +285,8 @@ static void handle_sigsegv()
 
 	/* wycz pluginy ui, eby odday terminal
 	 * destroy also log plugins to make sure that latest changes are written */
-	for (p = plugins; p; p = p->next) {
+	for (pl = plugins; pl; pl = pl->next) {
+		const plugin_t *p = pl->data;
 		if (p->pclass != PLUGIN_UI && p->pclass != PLUGIN_LOG)
 			continue;
 
@@ -456,27 +300,29 @@ static void handle_sigsegv()
 "\r\n"
 "The program will attempt to write its settings, but it is not\r\n"
 "guaranteed to succeed. They will be saved as\r\n"
-"%s/config.%d,  %s/config-<plugin>.%d\r\n"
-"and %s/userlist.%d\r\n"
+"%s/crash-%d-config,\r\n"
+"%s/crash-%d-config-<plugin>\r\n"
+"and %s/crash-%d-userlist\r\n"
 "\r\n"
 "Last messages from the debugging window will be saved to a file called\r\n"
-"%s/debug.%d.\r\n"
+"%s/crash-%d-debug.\r\n"
 "\r\n"
-"If a file called %s/core.%d will be created, try running the following\r\n"
+"If a file called %s/core will be created, try running the following\r\n"
 "command:\r\n"
 "\r\n"
-"    gdb %s %s/core.%d\r\n"
+"    gdb %s %s/core\r\n"
 "\n"
 "note the last few lines, and then note the output from the ,,bt'' command.\r\n"
 "This will help the program authors find the location of the problem\r\n"
 "and most likely will help avoid such crashes in the future.\r\n"
 "More details can be found in the documentation, in the file ,,gdb.txt''.\r\n"
 "\r\n",
-config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir,(int) getpid(), argv0, config_dir, (int) getpid());
+config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, argv0, config_dir);
 
 	config_write_crash();
 	userlist_write_crash();
 	debug_write_crash();
+	config_commit();
 
 	raise(SIGSEGV);			/* niech zrzuci core */
 }
@@ -494,20 +340,20 @@ config_dir, (int) getpid(), config_dir, (int) getpid(), config_dir, (int) getpid
  *
  * zwraca stworzon linie w zaalokowanym buforze lub NULL przy bdzie.
  */
-static char *prepare_batch_line(int argc, char *argv[], int n)
+static char *prepare_batch_line(char *argv[], int n)
 {
 	size_t len = 0;
 	char *buf;
 	int i;
 
-	for (i = n; i < argc; i++)
+	for (i = n; argv[i]; i++)
 		len += xstrlen(argv[i]) + 1;
 
 	buf = xmalloc(len);
 
-	for (i = n; i < argc; i++) {
+	for (i = n; argv[i]; i++) {
 		xstrcat(buf, argv[i]);
-		if (i < argc - 1)
+		if (argv[i+1])
 			xstrcat(buf, " ");
 	}
 
@@ -544,33 +390,33 @@ static WATCHER_LINE(handle_stderr)	/* stay */
  */
 
 void ekg_debug_handler(int level, const char *format, va_list ap) {
-	static string_t line = NULL;
-	char *tmp;
+	static GString *line = NULL;
+	char *tmp = NULL;
 
-	int is_UI = 0;
 	char *theme_format;
+	int is_UI = 0;
 
 	if (!config_debug)
 		return;
 
-	if (!(tmp = vsaprintf(format, ap)))
-		return;
-
 	if (line) {
-		string_append(line, tmp);
-		xfree(tmp);
+		g_string_append_vprintf(line, format, ap);
 
 		if (line->len == 0 || line->str[line->len - 1] != '\n')
 			return;
 
 		line->str[line->len - 1] = '\0';	/* remove '\n' */
-		tmp = string_free(line, 0);
+		tmp = g_string_free(line, FALSE);
 		line = NULL;
 	} else {
-		const size_t tmplen = xstrlen(tmp);
+		int tmplen = g_vasprintf(&tmp, format, ap);
+
+		if (tmplen < 0 || !tmp)	/* OutOfMemory? */
+			return;
+
 		if (tmplen == 0 || tmp[tmplen - 1] != '\n') {
-			line = string_init(tmp);
-			xfree(tmp);
+			line = g_string_new_len(tmp, tmplen);
+			g_free(tmp);
 			return;
 		}
 		tmp[tmplen - 1] = 0;			/* remove '\n' */
@@ -585,85 +431,114 @@ void ekg_debug_handler(int level, const char *format, va_list ap) {
 		case DEBUG_WHITE:		theme_format = "wdebug";	break;
 		case DEBUG_WARN:		theme_format = "warndebug";	break;
 		case DEBUG_OK:			theme_format = "okdebug";	break;
-		case DEBUG_WTF:		theme_format = "wtfdebug";	break;
 		default:			theme_format = "debug";		break;
 	}
 
+	ekg_fix_utf8(tmp); /* debug message can contain random data */
 	buffer_add(&buffer_debug, theme_format, tmp);
 
-	query_emit_id(NULL, UI_IS_INITIALIZED, &is_UI);
+	query_emit(NULL, "ui-is-initialized", &is_UI);
 
 	if (is_UI && window_debug) {
 		print_window_w(window_debug, EKG_WINACT_NONE, theme_format, tmp);
-
-		if (level == DEBUG_WTF) /* if real failure, warn also in current window (XXX: maybe always __status?) */
-			print("ekg_failure", tmp);
 	}
 #ifdef STDERR_DEBUG	/* STDERR debug */
 	else
 		fprintf(stderr, "%s\n", tmp);
 #endif
-
 	xfree(tmp);
 }
 
-struct option ekg_options[] = {
-	{ "user", required_argument, 0, 'u' },
-	{ "theme", required_argument, 0, 't' },
-	{ "no-auto", no_argument, 0, 'n' },
-	{ "no-mouse", no_argument, 0, 'm' },
-	{ "no-global-config", no_argument, 0, 'N' },
-	{ "frontend", required_argument, 0, 'F' },
+static void glib_debug_handler(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data) {
+	static int recurse = 0;
 
-	{ "away", optional_argument, 0, 'a' },
-	{ "back", optional_argument, 0, 'b' },
-	{ "invisible", optional_argument, 0, 'i' },
-	{ "dnd", optional_argument, 0, 'd' },
-	{ "free-for-chat", optional_argument, 0, 'f' },
-	{ "xa", optional_argument, 0, 'x' },
+	if (recurse)
+		return;
 
-	{ "unicode", no_argument, 0, 'U' }, 
+	if (!config_debug)
+		return;
 
-	{ "help", no_argument, 0, 'h' },
-	{ "version", no_argument, 0, 'v' },
-	{ 0, 0, 0, 0 }
+	recurse++;
+	debug("[%s] %s", log_domain, message);
+	recurse--;
+}
+
+static void glib_print_handler(const gchar *string) {
+	debug("%s", string);
+}
+
+static void glib_printerr_handler(const gchar *string) {
+	g_strlcpy(last_err_message, string, sizeof(last_err_message));
+
+	debug_error("%s", string);
+}
+
+static GOptionEntry ekg_options[] = {
+	{ "user", 'u', 0, G_OPTION_ARG_STRING, NULL,
+		"uses profile NAME", "NAME" },
+	{ "theme", 't', 0, G_OPTION_ARG_STRING, NULL,
+		"loads theme from FILE", "FILE" },
+	{ "no-auto", 'n', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, NULL,
+		"does not connect to server automatically", NULL },
+	{ "no-mouse", 'm', 0, G_OPTION_ARG_NONE, &no_mouse,
+		"does not load mouse support", NULL },
+	{ "no-global-config", 'N', 0, G_OPTION_ARG_NONE, NULL,
+		"ignores global configuration file", NULL },
+	{ "frontend", 'F', 0, G_OPTION_ARG_STRING, NULL,
+		"uses NAME frontend (default is ncurses)", "NAME" },
+
+	{ "away", 'a', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, NULL,
+		"changes status to ``away''", "[DESCRIPTION]" },
+	{ "back", 'b', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, NULL,
+		"changes status to ``available''", "[DESCRIPTION]" },
+	{ "invisible", 'i', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, NULL,
+		"changes status to ``invisible''", "[DESCRIPTION]" },
+	{ "dnd", 'd', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, NULL,
+		"changes status to ``do not disturb''", "[DESCRIPTION]" },
+	{ "free-for-chat", 'f', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, NULL,
+		"changes status to ``free for chat''", "[DESCRIPTION]" },
+	{ "xa", 'x', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, NULL,
+		"changes status to ``very busy''", "[DESCRIPTION]" },
+
+	{ "version", 'v', 0, G_OPTION_ARG_NONE, NULL,
+		"print program version and exit", NULL },
+
+	{ NULL }
 };
 
-#define EKG_USAGE N_( \
-"Usage: %s [OPTIONS] [COMMANDS]\n" \
-"  -u, --user=NAME	       uses profile NAME\n" \
-"  -t, --theme=FILE	       loads theme from FILE\n"\
-"  -n, --no-auto	       does not connect to server automatically\n" \
-"  -m, --no-mouse	       does not load mouse support\n" \
-"  -N, --no-global-config      ignores global configuration file\n" \
-"  -F, --frontend=NAME	       uses NAME frontend (default is ncurses)\n" \
-\
-"  -a, --away[=DESCRIPTION]    changes status to ``away''\n" \
-"  -b, --back[=DESCRIPTION]    changes status to ``available''\n" \
-"  -i, --invisible[=DESCR]     changes status to ``invisible''\n" \
-"  -d, --dnd[=DESCRIPTION]     changes status to ``do not disturb''\n" \
-"  -f, --free-for-chat[=DESCR] changes status to ``free for chat''\n" \
-"  -x, --xa[=DESCRIPTION]      changes status to ``very busy''\n" \
-\
-"  -h, --help		       displays this help message\n" \
-"  -v, --version	       displays program version and exits\n" \
-"\n" \
-"Options concerned with status depend on the protocol of particular session --\n" \
-"some sessions may not support ``do not disturb'' status, etc.\n" \
-"\n" )
-
-
 int main(int argc, char **argv)
 {
-	int auto_connect = 1, c = 0, no_global_config = 0, no_config = 0, new_status = 0;
+	gint auto_connect = 1, no_global_config = 0, no_config = 0, new_status = 0, print_version = 0;
 	char *tmp = NULL, *new_descr = NULL;
-	char *load_theme = NULL, *new_profile = NULL, *frontend = NULL;
+	gchar *load_theme = NULL, *new_profile = NULL, *frontend = NULL;
+	GOptionContext *opt;
+	GError *err = NULL;
 #ifndef NO_POSIX_SYSTEM
 	struct rlimit rlim;
 #else
 	WSADATA wsaData;
 #endif
 
+	gboolean set_status_callback(const gchar *optname, const gchar *optval,
+			gpointer data, GError **error) {
+
+		gchar c = optname[1] == '-' ? optname[2] : optname[1];
+		switch (c) {
+			case 'a': new_status = EKG_STATUS_AWAY; break;
+			case 'b': new_status = EKG_STATUS_AVAIL; break;
+			case 'd': new_status = EKG_STATUS_DND; break;
+			case 'f': new_status = EKG_STATUS_FFC; break;
+			case 'i': new_status = EKG_STATUS_INVISIBLE; break;
+			case 'x': new_status = EKG_STATUS_XA; break;
+		}
+		xfree(new_descr);
+		new_descr = xstrdup(optval);
+
+		return TRUE;
+	}
+
+	g_type_init();
+
 #ifndef NO_POSIX_SYSTEM
 	/* zostaw po sobie core */
 	rlim.rlim_cur = RLIM_INFINITY;
@@ -678,7 +553,6 @@ int main(int argc, char **argv)
 
 	ekg_started = time(NULL);
 
-	ekg2_dlinit();
 	setlocale(LC_ALL, "");
 	tzset();
 #ifdef ENABLE_NLS
@@ -687,7 +561,7 @@ int main(int argc, char **argv)
 #endif
 	srand(time(NULL));
 
-	strlcpy(argv0, argv[0], sizeof(argv0));
+	g_strlcpy(argv0, argv[0], sizeof(argv0));
 
 	home_dir = xstrdup(getenv("HOME"));
 
@@ -713,6 +587,7 @@ int main(int argc, char **argv)
 
 	command_init();
 #ifndef NO_POSIX_SYSTEM
+	signal(SIGABRT, handle_sigabrt);
 	signal(SIGSEGV, handle_sigsegv);
 	signal(SIGHUP, handle_sighup);
 	signal(SIGTERM, handle_sighup);
@@ -721,118 +596,44 @@ int main(int argc, char **argv)
 	signal(SIGALRM, SIG_IGN);
 	signal(SIGPIPE, SIG_IGN);
 #endif
-	while ((c = getopt_long(argc, argv, "b::a::i::d::f::x::u:F:t:nmNhvU", ekg_options, NULL)) != -1) 
-	{
-		switch (c) {
-			case 'a':
-				if (!optarg && argv[optind] && argv[optind][0] != '-')
-					optarg = argv[optind++];
 
-				new_status = EKG_STATUS_AWAY;
-				xfree(new_descr);
-				new_descr = xstrdup(optarg);
-				break;
+	ekg_options[0].arg_data = &new_profile;
+	ekg_options[1].arg_data = &load_theme;
+	ekg_options[2].arg_data = &auto_connect;
+	ekg_options[4].arg_data = &no_global_config;
+	ekg_options[5].arg_data = &frontend;
+	ekg_options[6].arg_data = &set_status_callback;
+	ekg_options[7].arg_data = &set_status_callback;
+	ekg_options[8].arg_data = &set_status_callback;
+	ekg_options[9].arg_data = &set_status_callback;
+	ekg_options[10].arg_data = &set_status_callback;
+	ekg_options[11].arg_data = &set_status_callback;
+	ekg_options[12].arg_data = &print_version;
 
-			case 'b':
-				if (!optarg && argv[optind] && argv[optind][0] != '-')
-					optarg = argv[optind++];
+	opt = g_option_context_new("[COMMAND...]");
+	g_option_context_set_description(opt, "Options concerned with status depend on the protocol of particular session -- \n" \
+"some sessions may not support ``do not disturb'' status, etc.\n");
+	g_option_context_add_main_entries(opt, ekg_options, "ekg2");
 
-				new_status = EKG_STATUS_AVAIL;
-				xfree(new_descr);
-				new_descr = xstrdup(optarg);
-				break;
+	if (!g_option_context_parse(opt, &argc, &argv, &err)) {
+		g_print("Option parsing failed: %s\n", err->message);
+		return 1;
+	}
+	
+	g_option_context_free(opt);
 
-			case 'i':
-				if (!optarg && argv[optind] && argv[optind][0] != '-')
-					optarg = argv[optind++];
-
-				new_status = EKG_STATUS_INVISIBLE;
-				xfree(new_descr);
-				new_descr = xstrdup(optarg);
-				break;
-
-			case 'd':
-				if (!optarg && argv[optind] && argv[optind][0] != '-')
-					optarg = argv[optind++];
-
-				new_status = EKG_STATUS_DND;
-				xfree(new_descr);
-				new_descr = xstrdup(optarg);
-				break;
-
-			case 'f':
-				if (!optarg && argv[optind] && argv[optind][0] != '-')
-					optarg = argv[optind++];
-
-				new_status = EKG_STATUS_FFC;
-				xfree(new_descr);
-				new_descr = xstrdup(optarg);
-				break;
-
-			case 'x':
-				if (!optarg && argv[optind] && argv[optind][0] != '-')
-					optarg = argv[optind++];
-
-				new_status = EKG_STATUS_XA;
-				xfree(new_descr);
-				new_descr = xstrdup(optarg);
-				break;
-
-
-			case 'u':
-				new_profile = optarg;
-				break;
-			case 'F':
-				frontend = optarg;
-				break;
-			case 't':
-				load_theme = optarg;
-				break;
-
-			case 'n':
-				auto_connect = 0;
-				break;
-
-			case 'm':
-				no_mouse = 1;
-				break;
-
-			case 'N':
-				no_global_config = 1;
-				break;
-
-
-			case 'h':
-				printf(_(EKG_USAGE), argv[0]);
-				return 0;
-
-			case 'U':
-#ifdef USE_UNICODE
-				config_use_unicode = 1;
-#else
-				fprintf(stderr, _("EKG2 compiled without unicode support. This just can't work!\n"));
-				return 1;
-#endif
-				break;
-
-			case 'v':
-				printf("ekg2-%s (compiled on %s)\n", VERSION, compile_time());
-				return 0;
-
-			case '?':
-				/* supported by getopt */
-				fprintf(stdout, _("To get more information, start program with --help.\n"));
-				return 1;
-
-			default:
-				break;
-		}
+	if (print_version) {
+		g_print("ekg2-%s (compiled on %s)\n", VERSION, compile_time());
+		return 0;
 	}
 
+	g_log_set_default_handler(glib_debug_handler, NULL);
+	g_set_print_handler(glib_print_handler);
+	g_set_printerr_handler(glib_printerr_handler);
 	in_autoexec = 1;
 
-	if (optind < argc) {
-		batch_line = prepare_batch_line(argc, argv, optind);
+	if (argv[1]) {
+		batch_line = prepare_batch_line(argv, 1);
 		batch_mode = 1;
 	}
 
@@ -849,9 +650,14 @@ int main(int argc, char **argv)
 	xfree(tmp);
 	tmp = NULL;
 
+	/* initialize dynamic module support */
+	ekg2_dlinit(argv[0]);
+
 	variable_init();
 	variable_set_default();
 
+	queries_init();
+
 	mesg_startup = mesg_set(MESG_CHECK);
 #ifdef DEFAULT_THEME 
 	if (theme_read(DEFAULT_THEME, 1) == -1) 
@@ -862,31 +668,39 @@ int main(int argc, char **argv)
 	window_status	= window_new(NULL, NULL, 1);			/* okno stanu */
 	window_current	= window_status;
 
+#if 0
 	if (!no_global_config)
 		config_read(SYSCONFDIR "/ekg2.conf");
+#endif
 
 	if (frontend) {
 		plugin_load(frontend, -254, 1);
 		config_changed = 1;
+		g_free(frontend);
 	}
 
 	config_read_plugins();
+#if 0
 	if (!no_global_config)
 		config_read(SYSCONFDIR "/ekg2-override.conf");
+#endif
 
 /*	  userlist_read(); */
 	emoticon_read();
 	msg_queue_read();
 
+	if (!frontend) {
 #ifdef HAVE_NCURSES
-	if (!have_plugin_of_class(PLUGIN_UI)) plugin_load(("ncurses"), -254, 1);
+		if (!have_plugin_of_class(PLUGIN_UI)) plugin_load(("ncurses"), -254, 1);
 #endif
 #ifdef HAVE_GTK
-	if (!have_plugin_of_class(PLUGIN_UI)) plugin_load(("gtk"), -254, 1);
+		if (!have_plugin_of_class(PLUGIN_UI)) plugin_load(("gtk"), -254, 1);
 #endif
-#ifdef HAVE_READLINE
-	if (!have_plugin_of_class(PLUGIN_UI)) plugin_load(("readline"), -254, 1);
+#ifdef HAVE_LIBREADLINE
+		if (!have_plugin_of_class(PLUGIN_UI)) plugin_load(("readline"), -254, 1);
 #endif
+	}
+
 	if (!have_plugin_of_class(PLUGIN_UI)) {
 		struct buffer *b;
 		for (b = buffer_debug.data; b; b = b->next)
@@ -916,8 +730,11 @@ int main(int argc, char **argv)
 		config_version = -1;
 
 	/* jeli ma by theme, niech bdzie theme */
-	if (load_theme)		theme_read(load_theme, 1);
-	else if (config_theme)	theme_read(config_theme, 1);
+	if (load_theme) {
+		theme_read(load_theme, 1);
+		g_free(load_theme);
+	} else if (config_theme)
+		theme_read(config_theme, 1);
 
 	in_autoexec = 0;
 
@@ -944,7 +761,6 @@ int main(int argc, char **argv)
 	protocol_init();
 	events_init();
 	metacontact_init();
-	audio_initialize();
 /*	scripts_init();		*/
 
 	/* it has to be done after plugins are loaded, either we wouldn't know if we are
@@ -986,9 +802,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (config_auto_save)
-		last_save = time(NULL);
-
 	if (no_config) {
 #ifdef HAVE_LIBGADU
 		if (plugin_find("gg"))
@@ -1005,11 +818,11 @@ int main(int argc, char **argv)
 	ekg2_reason_changed = 0;
 	/* jesli jest emit: ui-loop (plugin-side) to dajemy mu kontrole, jesli nie 
 	 * to wywolujemy normalnie sami ekg_loop() w petelce */
-	if (query_emit_id(NULL, UI_LOOP) != -1) {
+	if (query_emit(NULL, "ui-loop") != -1) {
+
 		/* kr imprez */
-		while (1) {
-			ekg_loop();
-		}
+		while (1)
+			g_main_context_iteration(NULL, TRUE);
 	}
 
 	ekg_exit();
@@ -1058,19 +871,6 @@ void ekg_exit()
 	}
 	send_nicks_count = 0;
 
-	{
-		child_t *c;
-
-		for (c = children; c; c = c->next) {
-#ifndef NO_POSIX_SYSTEM
-			kill(c->pid, SIGTERM);
-#else
-			/* TerminateProcess / TerminateThread */
-#endif
-		}
-		children_destroy();
-	}
-
 	{
 		list_t l;
 
@@ -1082,10 +882,12 @@ void ekg_exit()
 	}
 
 	{
-		plugin_t *p, *next;
+		GSList *pl;
 
-		for (p = plugins; p; p = next) {
-			next = p->next;
+		for (pl = plugins; pl;) {
+			const plugin_t *p = pl->data;
+
+			pl = pl->next;
 
 			if (p->pclass != PLUGIN_UI)
 				continue;
@@ -1106,13 +908,22 @@ void ekg_exit()
 			if (line[xstrlen(line) - 1] == '\n')
 				line[xstrlen(line) - 1] = 0;
 			if (!xstrcasecmp(line, "tak") || !xstrcasecmp(line, "yes") || !xstrcasecmp(line, "t") || !xstrcasecmp(line, "y")) {
-				if (config_write(NULL) || session_write() || metacontact_write() || script_variables_write())
+				config_write();
+				session_write();
+				metacontact_write();
+				script_variables_write();
+				if (!config_commit())
 					printf(_("Error while saving.\n"));
 			}
 		} else
 			printf("\n");
 	} else if (config_save_quit == 2) {
-		if (config_write(NULL) || session_write() || metacontact_write() || script_variables_write())
+		config_write();
+		session_write();
+		metacontact_write();
+		script_variables_write();
+
+		if (!config_commit())
 			printf(_("Error while saving.\n"));
 
 	} else if (config_keep_reason && ekg2_reason_changed && config_save_quit == 1) {
@@ -1124,20 +935,23 @@ void ekg_exit()
 			if (line[xstrlen(line) - 1] == '\n')
 				line[xstrlen(line) - 1] = 0;
 			if (!xstrcasecmp(line, "tak") || !xstrcasecmp(line, "yes") || !xstrcasecmp(line, "t") || !xstrcasecmp(line, "y")) {
-				if (session_write())
+				session_write();
+				if (!config_commit())
 					printf(_("Error while saving.\n"));
 			}
 		} else
 			printf("\n");
 
 	} else if (config_keep_reason && ekg2_reason_changed && config_save_quit == 2) {
-		if (session_write())
+		session_write();
+		if (!config_commit())
 			printf(_("Error while saving.\n"));
 	}
 	config_exit_exec = NULL; /* avoid freeing it */
 
 /* XXX, think about sequence of unloading. */
 
+	sources_destroy();
 	msgs_queue_destroy();
 	conferences_destroy();
 	newconferences_destroy();
@@ -1145,27 +959,24 @@ void ekg_exit()
 	sessions_free();
 
 	{
-		plugin_t *p;
+		GSList *pl;
 
-		for (p = plugins; p; ) {
-			plugin_t *next = p->next;
+		for (pl = plugins; pl; ) {
+			const plugin_t *p = pl->data;
 
+			pl = pl->next;
 			p->destroy();
 
 //			if (p->dl) ekg2_dlclose(p->dl);
-
-			p = next;
 		}
 	}
 
-	audio_deinitialize();
 	aliases_destroy();
 	theme_free();
 	variables_destroy();
 	script_variables_free(1);
 	emoticons_destroy();
 	commands_destroy();
-	timers_destroy();
 	binding_free();
 	lasts_destroy();
 
@@ -1174,36 +985,24 @@ void ekg_exit()
 
 	/* free internal read_file() buffer */
 	read_file(NULL, -1);
-	read_file_iso(NULL, -1);
+	read_file_utf(NULL, -1);
 
 /* windows: */
 	windows_destroy();
 	window_status = NULL; window_debug = NULL; window_current = NULL;	/* just in case */
 
-/* queries: */
+/* queries */
 	{
-		query_t **ll;
-
-		for (ll = queries; ll <= &queries[QUERY_EXTERNAL]; ll++) {
-			query_t *q;
-
-			for (q = *ll; q; ) {	/* free other queries... connected by protocol_init() for example */
-				query_t *next = q->next;
-
-				query_free(q);
-
-				q = next;
-			}
-
-			LIST_DESTROY2(*ll, NULL); /* XXX: really needed? */
+		query_t** kk;
+		for (kk = queries; kk < &queries[QUERIES_BUCKETS]; ++kk) {
+			queries_list_destroy(kk);
 		}
 	}
-	query_external_free();
+	registered_queries_free();
 
 	xfree(home_dir);
 
 	xfree(config_dir);
-	xfree(console_charset);
 
 	mesg_set(mesg_startup);
 #ifdef NO_POSIX_SYSTEM
