@@ -20,8 +20,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "ekg2-config.h"
-#include "win32.h"
+#include "ekg2.h"
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -34,24 +33,8 @@
 
 #include <string.h>
 
-#include "debug.h"
-#include "dynstuff.h"
-#include "dynstuff_inline.h"
-#include "xmalloc.h"
-
-#include "commands.h"
 #include "emoticons.h"
 #include "objects.h"
-#include "userlist.h"
-#include "windows.h"
-
-#include "log.h"
-#include "msgqueue.h"
-#include "protocol.h"
-#include "stuff.h"
-#include "themes.h"
-
-#include "queries.h"
 
 static int auto_find_limit = 100; /* counter of persons who we were looking for when autofind */
 dcc_t *dccs = NULL;
@@ -82,17 +65,17 @@ static QUERY(protocol_userlist_changed);
  */
 
 void protocol_init() {
-	query_connect_id(NULL, PROTOCOL_STATUS, protocol_status, NULL);
-	query_connect_id(NULL, PROTOCOL_MESSAGE, protocol_message, NULL);
-	query_connect_id(NULL, PROTOCOL_MESSAGE_ACK, protocol_message_ack, NULL);
-	query_connect_id(NULL, PROTOCOL_XSTATE, protocol_xstate, NULL);
+	query_connect(NULL, "protocol-status", protocol_status, NULL);
+	query_connect(NULL, "protocol-message", protocol_message, NULL);
+	query_connect(NULL, "protocol-message-ack", protocol_message_ack, NULL);
+	query_connect(NULL, "protocol-xstate", protocol_xstate, NULL);
 
-	query_connect_id(NULL, PROTOCOL_CONNECTED, protocol_connected, NULL);
-	query_connect_id(NULL, PROTOCOL_DISCONNECTED, protocol_disconnected, NULL);
+	query_connect(NULL, "protocol-connected", protocol_connected, NULL);
+	query_connect(NULL, "protocol-disconnected", protocol_disconnected, NULL);
 
-	query_connect_id(NULL, USERLIST_ADDED,		protocol_userlist_changed, NULL);
-	query_connect_id(NULL, USERLIST_REMOVED,	protocol_userlist_changed, NULL);
-	query_connect_id(NULL, USERLIST_RENAMED,	protocol_userlist_changed, NULL);
+	query_connect(NULL, "userlist-added",		protocol_userlist_changed, NULL);
+	query_connect(NULL, "userlist-removed",	protocol_userlist_changed, NULL);
+	query_connect(NULL, "userlist-renamed",	protocol_userlist_changed, NULL);
 }
 
 /*
@@ -118,7 +101,7 @@ static QUERY(protocol_userlist_changed) {
 		xfree(w->target);
 		w->target = xstrdup(*p2);
 
-		query_emit_id(NULL, UI_WINDOW_TARGET_CHANGED, &w);
+		query_emit(NULL, "ui-window-target-changed", &w);
 	}
 
 	return 0;
@@ -197,7 +180,7 @@ static QUERY(protocol_disconnected) {
 
 			s->last_conn = time(NULL);
 			s->connected = 0;
-			query_emit_id(NULL, SESSION_EVENT, &s, &one);	/* notify UI */
+			query_emit(NULL, "session-event", &s, &one);	/* notify UI */
 		} else
 			s->connecting = 0;
 		command_exec(NULL, s, "/session --unlock", 1);
@@ -253,7 +236,7 @@ static QUERY(protocol_disconnected) {
 int protocol_disconnected_emit(const session_t *s, const char *reason, int type) {
 	char *session   = xstrdup(s->uid);
 	char *reason_ro = xstrdup(reason);
-	int result      = query_emit_id(NULL, PROTOCOL_DISCONNECTED, &session, &reason_ro, &type);
+	int result      = query_emit(NULL, "protocol-disconnected", &session, &reason_ro, &type);
 
 	xfree(session);
 	xfree(reason_ro);
@@ -300,7 +283,7 @@ static QUERY(protocol_connected) {
 		s->connected = 1;
 		timer_remove_session(s, "reconnect");
 
-		query_emit_id(NULL, SESSION_EVENT, &s, &two);	/* Notify UI */
+		query_emit(NULL, "session-event", &s, &two);	/* Notify UI */
 	}
 
 	if (!msg_queue_flush(session))
@@ -311,7 +294,7 @@ static QUERY(protocol_connected) {
 
 int protocol_connected_emit(const session_t *s) {
 	char *session = xstrdup(s->uid);
-	int result    = query_emit_id(NULL, PROTOCOL_CONNECTED, &session);
+	int result    = query_emit(NULL, "protocol-connected", &session);
 
 	xfree(session);
 	return result;
@@ -425,7 +408,7 @@ static QUERY(protocol_status)
 
 	/* daj zna dwikiem... */
 	if (config_beep && config_beep_notify)
-		query_emit_id(NULL, UI_BEEP);
+		query_emit(NULL, "ui-beep");
 
 	/* ...i muzyczk */
 	if (config_sound_notify_file)
@@ -447,9 +430,9 @@ notify_plugins:
 		xfree(u->last_descr);
 		u->last_descr = xstrdup(de);
 		if (EKG_STATUS_IS_NA(status) && !ignore_events)
-			query_emit_id(NULL, EVENT_OFFLINE, __session, __uid);
+			query_emit(NULL, "event-offline", __session, __uid);
 	} else if (!EKG_STATUS_IS_NA(status) && !ignore_events)
-		query_emit_id(NULL, EVENT_ONLINE, __session, __uid);
+		query_emit(NULL, "event-online", __session, __uid);
 
 	if (!ignore_status) {
 		if (r) {
@@ -464,7 +447,7 @@ notify_plugins:
 	}
 
 	if (xstrcasecmp(de, descr) && !ignore_events)
-		query_emit_id(NULL, EVENT_DESCR, __session, __uid, __descr);
+		query_emit(NULL, "event-descr", __session, __uid, __descr);
 
 	if (!ignore_status && !ignore_status_descr) {
 		if (r) {
@@ -488,7 +471,7 @@ notify_plugins:
 			u->status_time = when ? when : time(NULL);
 	}
 	
-	query_emit_id(NULL, USERLIST_CHANGED, __session, __uid);
+	query_emit(NULL, "userlist-changed", __session, __uid);
 
 	/* Currently it behaves like event means grouped statuses,
 	 * i.e. EVENT_AVAIL is for avail&ffc
@@ -496,11 +479,11 @@ notify_plugins:
 	 *	... */
 	if (!ignore_events) {
 		if (EKG_STATUS_IS_AVAIL(status))
-			query_emit_id(NULL, EVENT_AVAIL, __session, __uid);
+			query_emit(NULL, "event-avail", __session, __uid);
 		else if (EKG_STATUS_IS_AWAY(status))
-			query_emit_id(NULL, EVENT_AWAY, __session, __uid);
+			query_emit(NULL, "event-away", __session, __uid);
 		else if (EKG_STATUS_IS_NA(status))
-			query_emit_id(NULL, EVENT_NA, __session, __uid);
+			query_emit(NULL, "event-na", __session, __uid);
 	}
 
 	return 0;
@@ -510,7 +493,7 @@ int protocol_status_emit(const session_t *s, const char *uid, int status, char *
 	char *session  = xstrdup(s->uid);
 	char *uid_ro   = xstrdup(uid);
 	char *descr_ro = xstrdup(descr);
-	int result     = query_emit_id(NULL, PROTOCOL_STATUS, &session, &uid_ro, &status, &descr_ro, &when);
+	int result     = query_emit(NULL, "protocol-status", &session, &uid_ro, &status, &descr_ro, &when);
 
 	xfree(session);
 	xfree(uid_ro);
@@ -525,7 +508,7 @@ int protocol_status_emit(const session_t *s, const char *uid, int status, char *
  *
  * zwraca target
  */
-char *message_print(const char *session, const char *sender, const char **rcpts, const char *__text, const uint32_t *format, time_t sent, int mclass, const char *seq, int dobeep, int secure)
+char *message_print(const char *session, const char *sender, const char **rcpts, const char *__text, const guint32 *format, time_t sent, int mclass, const char *seq, int dobeep, int secure)
 {
 	char *class_str, timestamp[100], *text = xstrdup(__text);
 	char *securestr = NULL;
@@ -592,7 +575,7 @@ char *message_print(const char *session, const char *sender, const char **rcpts,
 
 		for (i = 0; text[i]; i++) {
 
-			uint32_t f = format[i];
+			guint32 f = format[i];
 
 			if (i == 0 || f != format[i - 1]) {
 				char attr = 'n';
@@ -659,7 +642,7 @@ char *message_print(const char *session, const char *sender, const char **rcpts,
 
 	/* if there is a lot of recipients, conference should be made */
 	{
-		int recipients_count = array_count((char **) rcpts);
+		int recipients_count = g_strv_length((char **) rcpts);
 
 		if ((mclass < EKG_MSGCLASS_SENT) && recipients_count > 0) {
 			c = conference_find_by_uids(s, sender, rcpts, recipients_count, 0);
@@ -699,7 +682,7 @@ char *message_print(const char *session, const char *sender, const char **rcpts,
 	if (mclass == EKG_MSGCLASS_CHAT) {
 
 		if (config_beep && config_beep_chat && dobeep)
-			query_emit_id(NULL, UI_BEEP);
+			query_emit(NULL, "ui-beep");
 	
 		if (config_sound_chat_file && dobeep)
 			play_sound(config_sound_chat_file);
@@ -707,7 +690,7 @@ char *message_print(const char *session, const char *sender, const char **rcpts,
 	} else if (mclass == EKG_MSGCLASS_MESSAGE) {
 
 		if (config_beep && config_beep_msg && dobeep)
-			query_emit_id(NULL, UI_BEEP);
+			query_emit(NULL, "ui-beep");
 		if (config_sound_msg_file && dobeep)
 			play_sound(config_sound_msg_file);
 
@@ -763,7 +746,7 @@ static QUERY(protocol_message)
 	char *uid	= *(va_arg(ap, char**));
 	char **rcpts	= *(va_arg(ap, char***));
 	char **ptext	= (va_arg(ap, char**));
-	uint32_t *format= *(va_arg(ap, uint32_t**));
+	guint32 *format= *(va_arg(ap, guint32**));
 	time_t sent	= *(va_arg(ap, time_t*));
 	int mclass	= *(va_arg(ap, int*));
 	char *seq	= *(va_arg(ap, char**));
@@ -799,7 +782,7 @@ static QUERY(protocol_message)
 		}
 
 		if (oldstate != userlist->blink)
-			query_emit_id(NULL, USERLIST_CHANGED, &session, &uid);
+			query_emit(NULL, "userlist-changed", &session, &uid);
 	}
 	
 	if (mclass & EKG_NO_THEMEBIT) {
@@ -815,7 +798,7 @@ static QUERY(protocol_message)
 		char *___message = xstrdup(*ptext);
 		int ___decrypted = 0;
 
-		query_emit_id(NULL, MESSAGE_DECRYPT, &___session, &___sender, &___message, &___decrypted, NULL);
+		query_emit(NULL, "message-decrypt", &___session, &___sender, &___message, &___decrypted, NULL);
 
 		if (___decrypted) {
 			xfree(*ptext);
@@ -829,10 +812,10 @@ static QUERY(protocol_message)
 		xfree(___message);
 	}
 
-	if (our_msg)	query_emit_id(NULL, PROTOCOL_MESSAGE_SENT, &session, &(rcpts[0]), ptext);
-	else		query_emit_id(NULL, PROTOCOL_MESSAGE_RECEIVED, &session, &uid, &rcpts, ptext, &format, &sent, &mclass, &seq, &secure);
+	if (our_msg)	query_emit(NULL, "protocol-message-sent", &session, &(rcpts[0]), ptext);
+	else		query_emit(NULL, "protocol-message-received", &session, &uid, &rcpts, ptext, &format, &sent, &mclass, &seq, &secure);
 
-	query_emit_id(NULL, PROTOCOL_MESSAGE_POST, &session, &uid, &rcpts, ptext, &format, &sent, &mclass, &seq, &secure);
+	query_emit(NULL, "protocol-message-post", &session, &uid, &rcpts, ptext, &format, &sent, &mclass, &seq, &secure);
 
 	/* show it ! */
 	if (!(our_msg && !config_display_sent)) {
@@ -875,13 +858,13 @@ static QUERY(protocol_message)
 	return 0;
 }
 
-int protocol_message_emit(const session_t *s, const char *uid, char **rcpts, const char *text, const uint32_t *format, time_t sent, int mclass, const char *seq, int dobeep, int secure) {
+int protocol_message_emit(const session_t *s, const char *uid, char **rcpts, const char *text, const guint32 *format, time_t sent, int mclass, const char *seq, int dobeep, int secure) {
 	char *session = xstrdup(s->uid);
 	char *uid_ro  = xstrdup(uid);
 	char *text_ro = xstrdup(text);
 	char *seq_ro  = xstrdup(seq);
 	/* XXX, rcpts_ro, format_ro */
-	int result    = query_emit_id(NULL, PROTOCOL_MESSAGE, &session, &uid_ro, &rcpts, &text_ro, &format, &sent, &mclass, &seq_ro, &dobeep, &secure);
+	int result    = query_emit(NULL, "protocol-message", &session, &uid_ro, &rcpts, &text_ro, &format, &sent, &mclass, &seq_ro, &dobeep, &secure);
 
 	xfree(session);
 	xfree(uid_ro);
@@ -944,7 +927,7 @@ int protocol_message_ack_emit(const session_t *s, const char *rcpt, const char *
 	char *session = xstrdup(s->uid);
 	char *rcpt_ro = xstrdup(rcpt);
 	char *seq_ro  = xstrdup(seq);
-	int result    = query_emit_id(NULL, PROTOCOL_MESSAGE_ACK, &session, &rcpt_ro, &seq_ro, &status);
+	int result    = query_emit(NULL, "protocol-message-ack", &session, &rcpt_ro, &seq_ro, &status);
 
 	xfree(session);
 	xfree(rcpt_ro);
@@ -975,7 +958,7 @@ static QUERY(protocol_xstate)
 		else
 			goto xs_userlist;
 
-		query_emit_id(NULL, UI_WINDOW_ACT_CHANGED, &w);		/* XXX, UI_WINDOW_TYPING_CHANGED? :> */
+		query_emit(NULL, "ui-window-act-changed", &w);		/* XXX, UI_WINDOW_TYPING_CHANGED? :> */
 	}
 
 xs_userlist:
@@ -987,7 +970,7 @@ xs_userlist:
 		else
 			return 0;
 
-		query_emit_id(NULL, USERLIST_CHANGED, __session, __uid);
+		query_emit(NULL, "userlist-changed", __session, __uid);
 	}
 
 	return 0;
@@ -996,7 +979,7 @@ xs_userlist:
 int protocol_xstate_emit(const session_t *s, const char *uid, int state, int offstate) {
 	char *session = xstrdup(s->uid);
 	char *uid_ro  = xstrdup(uid);
-	int result    = query_emit_id(NULL, PROTOCOL_XSTATE, &session, &uid_ro, &state, &offstate);
+	int result    = query_emit(NULL, "protocol-xstate", &session, &uid_ro, &state, &offstate);
 
 	xfree(session);
 	xfree(uid_ro);
