@@ -22,11 +22,8 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "ekg2-config.h"
-#include "win32.h"
+#include "ekg2.h"
 
-#define _XOPEN_SOURCE 600
-#define __EXTENSIONS__
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -34,7 +31,6 @@
 #include <sys/socket.h>
 #endif
 
-#define __USE_BSD
 #include <sys/time.h>
 
 #ifndef NO_POSIX_SYSTEM
@@ -60,56 +56,12 @@
 #include <time.h>
 #include <unistd.h>
 
-#ifdef HAVE_ICONV
-#	include <iconv.h>
-#endif
-
-#ifndef HAVE_STRLCPY
-#  include "compat/strlcpy.h"
-#endif
-#ifndef HAVE_STRLCAT
-#  include "compat/strlcat.h"
-#endif
-
-#include "debug.h"
-#include "commands.h"
-#include "dynstuff.h"
-#include "protocol.h"
-#include "stuff.h"
-#include "themes.h"
-#include "userlist.h"
-#include "vars.h"
-#include "windows.h"
-#include "xmalloc.h"
-#include "plugins.h"
-#include "sessions.h"
-#include "recode.h"
-
-#include "dynstuff_inline.h"
-#include "queries.h"
-
-child_t *children = NULL;
-
-static LIST_FREE_ITEM(child_free_item, child_t *) { xfree(data->name); }
-
-DYNSTUFF_LIST_DECLARE(children, child_t, child_free_item,
-	static __DYNSTUFF_LIST_ADD,		/* children_add() */
-	__DYNSTUFF_LIST_REMOVE_ITER,		/* children_removei() */
-	__DYNSTUFF_LIST_DESTROY)		/* children_destroy() */
-
 alias_t *aliases = NULL;
 list_t autofinds = NULL;
 
-struct timer *timers = NULL;
-static LIST_FREE_ITEM(timer_free_item, struct timer *) { data->function(1, data->data); xfree(data->name); }
-
-
-DYNSTUFF_LIST_DECLARE2(timers, struct timer, timer_free_item,
-	static __DYNSTUFF_LIST_ADD,		/* timers_add() */
-	__DYNSTUFF_LIST_REMOVE_SAFE,		/* timers_remove() */
-	__DYNSTUFF_LIST_REMOVE_ITER,		/* timers_removei() */
-	__DYNSTUFF_LIST_DESTROY)		/* timers_destroy() */
-
+/***************
+ * conferences
+ ***************/
 struct conference *conferences = NULL;
 newconference_t *newconferences = NULL;
 
@@ -123,13 +75,11 @@ char *config_subject_reply_prefix;
 int in_autoexec = 0;
 int config_auto_save = 0;
 int config_auto_user_add = 0;
-time_t last_save = 0;
 int config_display_color = 1;
 int config_beep = 1;
 int config_beep_msg = 1;
 int config_beep_chat = 1;
 int config_beep_notify = 1;
-char *config_console_charset;
 char *config_dcc_dir;
 int config_display_blinking = 1;
 int config_events_delay = 3;
@@ -141,8 +91,6 @@ char *config_sound_notify_file = NULL;
 char *config_sound_sysmsg_file = NULL;
 char *config_sound_mail_file = NULL;
 char *config_sound_app = NULL;
-int config_use_unicode;
-int config_use_iso;
 int config_changed = 0;
 int config_display_ack = 12;
 int config_completion_notify = 1;
@@ -182,9 +130,6 @@ int config_windows_save = 0;
 char *config_windows_layout = NULL;
 char *config_profile = NULL;
 int config_debug = 1;
-int config_lastlog_noitems = 0;
-int config_lastlog_case = 0;
-int config_lastlog_display_all = 0;
 int config_version = 0;
 char *config_exit_exec = NULL;
 int config_session_locks = 0;
@@ -195,6 +140,8 @@ char *last_search_last_name = NULL;
 char *last_search_nickname = NULL;
 char *last_search_uid = 0;
 
+char *formated_config_timestamp = NULL;
+
 int ekg2_reason_changed = 0;
 
 /*
@@ -278,7 +225,7 @@ DYNSTUFF_LIST_DECLARE(aliases, alias_t, list_alias_free,
 int alias_add(const char *string, int quiet, int append)
 {
 	char *cmd, *aname, *tmp;
-	command_t *c;
+	GSList *cl;
 	alias_t *a;
 	char **params = NULL;
 	char *array;
@@ -298,7 +245,8 @@ int alias_add(const char *string, int quiet, int append)
 				list_add(&a->commands, xstrdup(cmd));
 				
 				/* przy wielu komendach trudno dopenia, bo wg. ktrej? */
-				for (c = commands; c; c = c->next) {
+				for (cl = commands; cl; cl = cl->next) {
+					command_t *c = cl->data;
 					if (!xstrcasecmp(c->name, a->name)) {
 						xfree(c->params);
 						c->params = array_make(("?"), (" "), 0, 1, 1);
@@ -319,7 +267,8 @@ int alias_add(const char *string, int quiet, int append)
 		*tmp = 0;
 
 	for (i=0; i<2; i++) {
-		for (c = commands; c && !params; c = c->next) {
+		for (cl = commands; cl && !params; cl = cl->next) {
+			command_t *c = cl->data;
 			const char *cname = c->name;
 			if (i) {
 				if ((tmp = xstrchr(cname, ':')))
@@ -348,7 +297,7 @@ int alias_add(const char *string, int quiet, int append)
 	list_add(&(a->commands), xstrdup(cmd));
 	aliases_add(a);
 
-	array = (params) ? array_join(params, (" ")) : xstrdup(("?"));
+	array = (params) ? g_strjoinv(" ", params) : xstrdup(("?"));
 	command_add(NULL, a->name, array, cmd_alias_exec, COMMAND_ISALIAS, NULL);
 	xfree(array);
 	
@@ -586,17 +535,38 @@ void changed_mesg(const char *var)
 	else
 		mesg_set(config_mesg);
 }
-	
+
+static TIMER(auto_save_timer) {
+	if (type)
+		return 0;
+
+	if (!config_changed)
+		return 0;
+
+	debug("autosaving userlist and config.\n");
+
+	config_write();
+	session_write();
+
+	if (config_commit()) {
+		config_changed = 0;
+		ekg2_reason_changed = 0;
+		print("autosaved");
+	} else
+		print("error_saving");
+
+	return 0;
+}
+
 /*
  * changed_auto_save()
  *
  * wywoywane po zmianie wartoci zmiennej ,,auto_save''.
  */
-void changed_auto_save(const char *var)
-{
-	/* oszukujemy, ale takie zachowanie wydaje si by
-	 * bardziej ,,naturalne'' */
-	last_save = time(NULL);
+void changed_auto_save(const char *var) {
+	timer_remove(NULL, "auto_save");
+	if (config_auto_save > 0)
+		timer_add(NULL, "auto_save", config_auto_save, 1, auto_save_timer, NULL);
 }
 
 /*
@@ -641,6 +611,16 @@ void changed_theme(const char *var)
 	}
 }
 
+/*
+ * changed_theme()
+ *
+ * funkcja wywoywana przy zmianie wartoci zmiennej ,,config_timestamp''.
+ */
+void changed_config_timestamp(const char *var) {
+	xfree(formated_config_timestamp);
+	formated_config_timestamp = (config_timestamp && *config_timestamp) ? format_string(config_timestamp) : NULL;
+}
+
 /**
  * compile_time()
  *
@@ -761,7 +741,7 @@ struct conference *conference_add(session_t *session, const char *name, const ch
 		return NULL;
 
 	if (nicklist[0] == ',' || nicklist[xstrlen(nicklist) - 1] == ',') {
-		printq("invalid_params", ("chat"));
+		printq("invalid_params", ("chat"), nicklist);
 		return NULL;
 	}
 
@@ -808,19 +788,19 @@ struct conference *conference_add(session_t *session, const char *name, const ch
 			if (!nig) {
 				printq("group_empty", gname);
 				printq("conferences_not_added", name);
-				array_free(nicks);
+				g_strfreev(nicks);
 				return NULL;
 			}
 		}
 	}
 
-	count = array_count(nicks);
+	count = g_strv_length(nicks);
 
 	for (cf = conferences; cf; cf = cf->next) {
 		if (!xstrcasecmp(name, cf->name)) {
 			printq("conferences_exist", name);
 
-			array_free(nicks);
+			g_strfreev(nicks);
 
 			return NULL;
 		}
@@ -842,7 +822,7 @@ struct conference *conference_add(session_t *session, const char *name, const ch
 	}
 
 
-	array_free(nicks);
+	g_strfreev(nicks);
 
 	if (i != count) {
 		printq("conferences_not_added", name);
@@ -856,7 +836,7 @@ struct conference *conference_add(session_t *session, const char *name, const ch
 
 	tabnick_add(name);
 
-	cf = xmemdup(&c, sizeof(c));
+	cf = g_memdup(&c, sizeof(c));
 	conferences_add(cf);
 	return cf;
 }
@@ -1001,7 +981,7 @@ struct conference *conference_find_by_uids(session_t *s, const char *from, const
 			int comma = 0;
 
 			if (xstrcasecmp(from, s->uid) && !conference_participant(c, from)) {
-				list_add(&c->recipients, xmemdup(&from, sizeof(from)));
+				list_add(&c->recipients, g_memdup(&from, sizeof(from)));
 
 				comma++;
 				string_append(new, format_user(s, from));
@@ -1009,7 +989,7 @@ struct conference *conference_find_by_uids(session_t *s, const char *from, const
 
 			for (i = 0; i < count; i++) {
 				if (xstrcasecmp(recipients[i], s->uid) && !conference_participant(c, recipients[i])) {
-					list_add(&c->recipients, xmemdup(&recipients[i], sizeof(recipients[0])));
+					list_add(&c->recipients, g_memdup(&recipients[i], sizeof(recipients[0])));
 			
 					if (comma++)
 						string_append(new, ", ");
@@ -1088,63 +1068,57 @@ int conference_rename(const char *oldname, const char *newname, int quiet)
 	
 	printq("conferences_rename", oldname, newname);
 
-	query_emit_id(NULL, CONFERENCE_RENAMED, &oldname, &newname);	/* XXX READ-ONLY QUERY */
+	query_emit(NULL, "conference-renamed", &oldname, &newname);	/* XXX READ-ONLY QUERY */
 
 	return 0;
 }
 
-/*
- * help_path()
+/**
+ * help_open()
  *
- * zwraca plik z pomoc we waciwym jzyku lub null jeli nie ma takiego pliku
+ * Open the help file in best language available.
  *
+ * @param name - help file basename.
+ * @param plugin - plugin name or NULL if core help is requested.
+ *
+ * @return Open GDataInputStream with utf8 encoding or NULL if no file was
+ * found. It should be unreferenced with g_object_unref(). 
  */
-FILE *help_path(char *name, char *plugin) {
-	char lang[3];
-	char *tmp;
-	FILE *fp;
+GDataInputStream *help_open(const gchar *name, const gchar *plugin) {
+	const gchar* const *p;
 
-	char *base = plugin ? 
-		saprintf(DATADIR "/plugins/%s/%s", plugin, name) :
-		saprintf(DATADIR "/%s", name);
+	gchar *base = plugin
+		? g_build_filename(DATADIR, "plugins", plugin, name, NULL)
+		: g_build_filename(DATADIR, name, NULL);
+	GString *fnbuf = g_string_new(base);
+	const gsize baselen = fnbuf->len;
+	g_free(base);
 
-	do {
-		/* if we don't get lang from $LANGUAGE (man 3 gettext) */
-		if ((tmp = getenv("LANGUAGE"))) break;
-		/* fallback on locale enviroments.. (man 5 locale) */
-		if ((tmp = getenv("LC_ALL"))) break;
-		if ((tmp = getenv("LANG"))) break;
-		/* fallback to en language */
-		tmp = "en";
-	} while (0);
+	for (p = g_get_language_names(); *p; p++) {
+		GError *err = NULL;
+		GFile *f;
+		GFileInputStream *ret;
 
-	xstrncpy(&lang[0], tmp, 2);
-	lang[2] = 0;
-	
-help_again:
-	tmp = saprintf("%s-%s.txt", base, lang);
+		if (G_UNLIKELY(!strcmp(*p, "C")))
+			g_string_append(fnbuf, "-en.txt");
+		else
+			g_string_append_printf(fnbuf, "-%s.txt", *p);
+		f = g_file_new_for_path(fnbuf->str);
+		ret = g_file_read(f, NULL, &err);
 
-	if ((fp = fopen(tmp, "r"))) {
-		xfree(base);
-		xfree(tmp);
-		return fp;
+		if (ret) {
+			g_string_free(fnbuf, TRUE);
+			return g_data_input_stream_new(G_INPUT_STREAM(ret));
+		} else if (err->code != G_FILE_ERROR_NOENT)
+			debug_error("help_path() failed to open %s with error: %s\n",
+					fnbuf->str, err->message);
+
+		g_error_free(err);
+		g_string_truncate(fnbuf, baselen);
 	}
 
-	/* Temporary fallback - untill we don't have full en translation */
-	xfree(tmp);
-	if (xstrcasecmp(lang, "pl")) {
-		lang[0] = 'p';
-		lang[1] = 'l';
-		goto help_again;
-	}
-
-	/* last chance, just base without lang. */
-	tmp = saprintf("%s.txt", base);
-	fp = fopen(tmp, "r");
-
-	xfree(tmp);
-	xfree(base);
-	return fp;
+	g_string_free(fnbuf, TRUE);
+	return NULL;
 }
 
 
@@ -1156,6 +1130,7 @@ help_again:
  *
  *  - name - nazwa.
  */
+/*
 int ekg_hash(const char *name)
 {
 	int hash = 0;
@@ -1166,8 +1141,22 @@ int ekg_hash(const char *name)
 	}
 
 	return hash;
+}*/
+/*
+ * new hash, made with queries in mind
+ * but should also nicely behave for formats
+ */
+int ekg_hash(const char *name) {
+	unsigned long long st = 0x4d6947;
+
+	for (; *name; name++) {
+		st = st * 2147483069 + 2147482417;
+		st ^= (*name);
+	}
+	return (int)st;
 }
 
+
 /*
  * mesg_set()
  *
@@ -1261,33 +1250,6 @@ int play_sound(const char *sound_path)
 	return res;
 }
 
-/*
- * child_add()
- *
- * dopisuje do listy uruchomionych dzieci procesw.
- *
- *  - plugin
- *  - pid
- *  - name
- *  - handler
- *  - data
- *
- * 0/-1
- */
-child_t *child_add(plugin_t *plugin, pid_t pid, const char *name, child_handler_t handler, void *priv_data)
-{
-	child_t *c = xmalloc(sizeof(child_t));
-
-	c->plugin	= plugin;
-	c->pid		= pid;
-	c->name		= xstrdup(name);
-	c->handler	= handler;
-	c->priv_data	= priv_data;
-	
-	children_add(c);
-	return c;
-}
-
 /**
  * mkdir_recursive()
  *
@@ -1374,7 +1336,7 @@ const char *prepare_pathf(const char *filename, ...) {
 	size_t len;
 	int fpassed = (filename && *filename);
 
-	len = strlcpy(path, config_dir ? config_dir : "", sizeof(path));
+	len = g_strlcpy(path, config_dir ? config_dir : "", sizeof(path));
 
 	if (len + fpassed >= sizeof(path)) {
 		debug_error("prepare_pathf() LEVEL0 %d + %d >= %d\n", len, fpassed, sizeof(path));
@@ -1504,9 +1466,9 @@ const char *prepare_path_user(const char *path) {
 			xstrcat(out, "/");
 		} else
 			*out = 0;
-		if (homedir && strlcat(out, homedir, sizeof(out)-xstrlen(out)-1) >= sizeof(out)-xstrlen(out)-1)
+		if (homedir && g_strlcat(out, homedir, sizeof(out)-xstrlen(out)-1) >= sizeof(out)-xstrlen(out)-1)
 			return NULL; /* we don't add slash here, 'cause in already has it */
-		if (strlcat(out, in, sizeof(out)-xstrlen(out)) >= sizeof(out)-xstrlen(out))
+		if (g_strlcat(out, in, sizeof(out)-xstrlen(out)) >= sizeof(out)-xstrlen(out))
 			return NULL;
 	}
 
@@ -1596,23 +1558,8 @@ static char *random_line(const char *path) {
 	return NULL;
 }
 
-/**
- * read_file()
- *
- * Read next line from file @a f, if needed alloc memory for it.<br>
- * Remove \\r and \\n chars from end of line if needed.
- *
- * @param f	- opened FILE *
- * @param alloc 
- *		- If  0 than it return internal read_file() either xrealloc()'ed or static char with sizeof()==1024,
- *			which you <b>MUST NOT</b> xfree()<br>
- *		- If  1 than it return strdup()'ed string this <b>MUST</b> xfree()<br>
- *		- If -1 than it free <i>internal</i> pointer which were used by xrealloc()
- *
- * @return Line without \\r and \\n which must or mustn't be xfree()'d. It depends on @a alloc param
- */
-
-char *read_file(FILE *f, int alloc) {
+/* XXX: ekg_fix_utf8() here */
+char *read_file_utf(FILE *f, int alloc) {
 	static char buf[1024];
 	static char *reres = NULL;
 
@@ -1664,25 +1611,65 @@ char *read_file(FILE *f, int alloc) {
 	return (alloc) ? xstrdup(res) : res;
 }
 
-char *read_file_iso(FILE *f, int alloc) {
+/**
+ * read_file()
+ *
+ * Read next line from file @a f, if needed alloc memory for it.<br>
+ * Remove \\r and \\n chars from end of line if needed.
+ *
+ * @param f	- opened FILE *
+ * @param alloc 
+ *		- If  0 than it return internal read_file() either xrealloc()'ed or static char with sizeof()==1024,
+ *			which you <b>MUST NOT</b> xfree()<br>
+ *		- If  1 than it return strdup()'ed string this <b>MUST</b> xfree()<br>
+ *		- If -1 than it free <i>internal</i> pointer which were used by xrealloc()
+ *
+ * @return Line without \\r and \\n which must or mustn't be xfree()'d. It depends on @a alloc param
+ */
+
+char *read_file(FILE *f, int alloc) {
 	static char *tmp = NULL;
-	char *buf = read_file(f, 0);
+	char *buf = read_file_utf(f, 0);
 	char *res;
 
-	xfree(tmp);
+	g_free(tmp);
 	tmp = NULL;
 	if (alloc == -1)
 		return NULL;
 
-	ekg_recode_iso2_inc();
-	res = ekg_iso2_to_locale_dup(buf);
+	res = ekg_recode_from_locale(buf);
 	if (!alloc)
 		tmp = res;
 
-	ekg_recode_iso2_dec();
 	return res;
 }
 
+/**
+ * read_line()
+ *
+ * Read a single line from GDataInputStream.
+ *
+ * @param f - GDataInputStream to read from.
+ *
+ * @return Pointer to a static line which will be overwritten by next
+ * call to read_line() or NULL on EOF or error.
+ */
+gchar *read_line(GDataInputStream *f) {
+	static gchar *buf = NULL;
+	GError *err = NULL;
+
+	g_free(buf);
+	buf = g_data_input_stream_read_line(f, NULL, NULL, &err);
+
+	if (!buf && err) {
+		debug_error("read_line() failed: %s\n", err->message);
+		g_error_free(err);
+	} else if (buf)
+		ekg_fix_utf8(buf);
+
+	return buf;
+}
+
 /**
  * timestamp()
  *
@@ -1714,7 +1701,7 @@ const char *timestamp_time(const char *format, time_t t) {
 	static char buf[100];
 
 	if (!format || format[0] == '\0')
-		return itoa(t);
+		return ekg_itoa(t);
 
 	tm = localtime(&t);
 
@@ -1723,178 +1710,6 @@ const char *timestamp_time(const char *format, time_t t) {
 	return buf;
 }
 
-struct timer *timer_add_ms(plugin_t *plugin, const char *name, unsigned int period, int persist, int (*function)(int, void *), void *data) {
-	struct timer *t;
-	struct timeval tv;
-
-	/* wylosuj now nazw, jeli nie mamy */
-	if (!name) {
-		int i;
-
-		for (i = 1; !name; i++) {
-			int gotit = 0;
-
-			for (t = timers; t; t = t->next) {
-				if (!xstrcmp(t->name, itoa(i))) {
-					gotit = 1;
-					break;
-				}
-			}
-
-			if (!gotit)
-				name = itoa(i);
-		}
-	}
-
-	t = xmalloc(sizeof(struct timer));
-	gettimeofday(&tv, NULL);
-	tv.tv_sec += (period / 1000);
-	tv.tv_usec += ((period % 1000) * 1000);
-	if (tv.tv_usec >= 1000000) {
-		tv.tv_usec -= 1000000;
-		tv.tv_sec++;
-	}
-	memcpy(&(t->ends), &tv, sizeof(tv));
-	t->name = xstrdup(name);
-	t->period = period;
-	t->persist = persist;
-	t->function = function;
-	t->data = data;
-	t->plugin = plugin;
-
-	timers_add(t);
-	return t;
-}
-
-/*
- * timer_add()
- *
- * dodaje timera.
- *
- *  - plugin - plugin obsugujcy timer,
- *  - name - nazwa timera w celach identyfikacji. jeli jest rwna NULL,
- *	     zostanie przyznany pierwszy numerek z brzegu.
- *  - period - za jaki czas w sekundach ma by uruchomiony,
- *  - persist - czy stay timer,
- *  - function - funkcja do wywoania po upyniciu czasu,
- *  - data - dane przekazywane do funkcji.
- *
- * zwraca zaalokowan struct timer lub NULL w przypadku bdu.
- */
-struct timer *timer_add(plugin_t *plugin, const char *name, unsigned int period, int persist, int (*function)(int, void *), void *data)
-{
-	return timer_add_ms(plugin, name, period * 1000, persist, function, data);
-}
-
-struct timer *timer_add_session(session_t *session, const char *name, unsigned int period, int persist, int (*function)(int, session_t *)) {
-	struct timer *t;
-
-	if (!session || !session->plugin) {
-		debug_error("timer_add_session() s: 0x%x s->plugin: 0x%x\n", session, session ? session->plugin : NULL);
-		return NULL;
-	}
-
-	t = timer_add(session->plugin, name, period, persist, (void *) function, session);
-	t->is_session = 1;
-	return t;
-}
-
-/*
- * timer_remove()
- *
- * usuwa timer.
- *
- *  - plugin - plugin obsugujcy timer,
- *  - name - nazwa timera,
- *
- * 0/-1
- */
-int timer_remove(plugin_t *plugin, const char *name)
-{
-	struct timer *t;
-	int removed = 0;
-
-	for (t = timers; t; t = t->next) {
-		if (t->plugin == plugin && !xstrcasecmp(name, t->name)) {
-			t = timers_removei(t);
-			removed++;
-		}
-	}
-
-	return ((removed) ? 0 : -1);
-}
-
-struct timer *timer_find_session(session_t *session, const char *name) {
-	struct timer *t;
-
-	if (!session)
-		return NULL;
-	
-	for (t = timers; t; t = t->next) {
-		if (t->is_session && t->data == session && !xstrcmp(name, t->name))
-			return t;
-	}
-
-	return NULL;
-}
-
-int timer_remove_session(session_t *session, const char *name)
-{
-	struct timer *t;
-	plugin_t *p;
-	int removed = 0;
-
-	if (!session || (!(p = session->plugin)))
-		return -1;
-
-	for (t = timers; t; t = t->next) {
-		if (t->is_session && t->data == session && !xstrcmp(name, t->name)) {
-			t = timers_removei(t);
-			removed++;
-		}
-	}
-
-	return ((removed) ? 0 : -1);
-}
-
-/*
- * timer_handle_command()
- *
- * obsuga timera wywoujcego komend.
- */
-TIMER(timer_handle_command)
-{
-	if (type) {
-		xfree(data);
-		return 0;
-	}
-	
-	command_exec(NULL, NULL, (char *) data, 0);
-	return 0;
-}
-
-/*
- * timer_remove_user()
- *
- * usuwa wszystkie timery uytkownika.
- *
- * 0/-1
- */
-int timer_remove_user(int at)
-{
-	struct timer *t;
-	int removed = 0;
-
-	for (t = timers; t; t = t->next) {
-		if (t->at == at && t->function == timer_handle_command) { 
-			t = timers_removei(t);
-			removed = 1;
-		}
-	}
-
-	return ((removed) ? 0 : -1);
-}
-
 /* 
  * xstrmid()
  *
@@ -2014,6 +1829,10 @@ int isalpha_pl(unsigned char c)
 	return 0;
 }
 
+void ignore_result_helper(int __attribute__((unused)) dummy, ...)
+{
+}
+
 /*
  * strcasestr()
  *
@@ -2060,6 +1879,22 @@ int msg_all(session_t *s, const char *function, const char *what)
 
 	return 0;
 }
+
+#ifndef NO_POSIX_SYSTEM
+static void speech_child_handler(GPid pid, gint status, gpointer data) {
+	speech_pid = 0;
+
+	if (!config_speech_app)
+		buffer_free(&buffer_speech);
+
+	if (buffer_speech.count && !status) {
+		char *str = buffer_tail(&buffer_speech);
+		say_it(str);
+		g_free(str);
+	}
+}
+#endif
+
 /*
  * say_it()
  *
@@ -2101,7 +1936,7 @@ int say_it(const char *str)
 		exit(status);
 	}
 
-	child_add(NULL, pid, NULL, NULL, NULL);
+	ekg_child_add(NULL, "(speech)", pid, speech_child_handler, NULL, NULL);
 	return 0;
 #else
 	return -1;
@@ -2136,9 +1971,6 @@ void debug(const char *format, ...)
 }
 #endif
 
-static char base64_charset[] =
-	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-
 /*
  * base64_encode()
  *
@@ -2150,50 +1982,10 @@ static char base64_charset[] =
  */
 char *base64_encode(const char *buf, size_t len)
 {
-	char *out, *res;
-	int i = 0, j = 0, k = 0;
+	if (!buf)
+		return NULL;
 
-	if (!buf) return NULL;
-/*	if (!len) return NULL; */
-	
-	res = out = xmalloc((len / 3 + 1) * 4 + 2);
-
-	while (j < len) {
-		switch (i % 4) {
-			case 0:
-				k = (buf[j] & 252) >> 2;
-				break;
-			case 1:
-				if (j+1 < len)
-					k = ((buf[j] & 3) << 4) | ((buf[j + 1] & 240) >> 4);
-				else
-					k = (buf[j] & 3) << 4;
-
-				j++;
-				break;
-			case 2:
-				if (j+1 < len)
-					k = ((buf[j] & 15) << 2) | ((buf[j + 1] & 192) >> 6);
-				else
-					k = (buf[j] & 15) << 2;
-
-				j++;
-				break;
-			case 3:
-				k = buf[j++] & 63;
-				break;
-		}
-		*out++ = base64_charset[k];
-		i++;
-	}
-
-	if (i % 4)
-		for (j = 0; j < 4 - (i % 4); j++, out++)
-			*out = '=';
-	
-	*out = 0;
-	
-	return res;
+	return g_base64_encode((guchar*)buf, len);
 }
 
 /*
@@ -2207,49 +1999,12 @@ char *base64_encode(const char *buf, size_t len)
  */
 char *base64_decode(const char *buf)
 {
-	char *res, *save, *foo, val;
-	const char *end;
-	int index = 0;
 	size_t buflen;
 
-	if (!buf || !(buflen = xstrlen(buf)))
+	if (!buf || !(*buf))
 		return NULL;
 
-	save = res = xcalloc(1, (buflen / 4 + 1) * 3 + 2);
-
-	end = buf + buflen - 1;
-
-	while (*buf && buf < end) {
-		if (*buf == '\r' || *buf == '\n') {
-			buf++;
-			continue;
-		}
-		if (!(foo = xstrchr(base64_charset, *buf)))
-			foo = base64_charset;
-		val = (int)(foo - base64_charset);
-		buf++;
-		switch (index) {
-			case 0:
-				*res |= val << 2;
-				break;
-			case 1:
-				*res++ |= val >> 4;
-				*res |= val << 4;
-				break;
-			case 2:
-				*res++ |= val >> 2;
-				*res |= val << 6;
-				break;
-			case 3:
-				*res++ |= val;
-				break;
-		}
-		index++;
-		index %= 4;
-	}
-	*res = 0;
-	
-	return save;
+	return (char*) g_base64_decode(buf, &buflen);
 }
 
 /*
@@ -2368,7 +2123,7 @@ void ekg_update_status(session_t *session)
 			const char *__session	= session_uid_get(session);
 			const char *__uid		= u->uid;
 
-			query_emit_id(NULL, USERLIST_CHANGED, &__session, &__uid);
+			query_emit(NULL, "userlist-changed", &__session, &__uid);
 		}
 	}
 }
@@ -2490,9 +2245,9 @@ int ekg_status_int(const char *text)
  * tekstu wycina kolorki i zwraca informacje o formatowaniu tekstu bez
  * kolorkw.
  */
-uint32_t *ekg_sent_message_format(const char *text)
+guint32 *ekg_sent_message_format(const char *text)
 {
-	uint32_t *format, attr;
+	guint32 *format, attr;
 	char *newtext, *q;
 	const char *p, *end;
 	int len;
@@ -2574,61 +2329,6 @@ uint32_t *ekg_sent_message_format(const char *text)
 	return format;
 }
 
-#if ! USE_UNICODE
-/*
- * tolower_pl()
- *
- * zamienia podany znak na may jeli to moliwe
- * obsuguje polskie znaki
- */
-static int tolower_pl(const unsigned char c) {
-	switch(c) {
-		case 161: /*  */
-			return 177;
-		case 198: /*  */
-			return 230;
-		case 202: /*  */
-			return 234;
-		case 163: /*  */
-			return 179;
-		case 209: /*  */
-			return 241;
-		case 211: /*  */
-			return 243;
-		case 166: /*  */
-			return 182;
-		case 175: /*  */
-			return 191;
-		case 172: /*  */
-			return 188;
-		default: /* reszta */
-			return tolower(c);
-	}
-}
-#endif
-
-size_t strlen_pl(const char *s) {
-#if USE_UNICODE
-	if (!s) s = "";
-	return mbsrtowcs(NULL, &s, 0, NULL);
-#else
-	return xstrlen(s);
-#endif
-}
-
-char *xstrncat_pl(char *dest, const char *src, size_t n) {
-#if USE_UNICODE
-	int len=xstrlen(src);
-	wchar_t *wc = xmalloc((len+1) * sizeof(wchar_t));
-	const char *p = src;
-	
-	if (mbsrtowcs(wc, &p, n, NULL) < 0) n = 0;
-	else n = p ? p - src : len;
-	xfree(wc);
-#endif
-	return xstrncat(dest, src, n);
-}
-
 /*
  * strncasecmp_pl()
  *
@@ -2636,41 +2336,35 @@ char *xstrncat_pl(char *dest, const char *src, size_t n) {
  * dziaa analogicznie do xstrncasecmp()
  * obsuguje polskie znaki
  */
-int strncasecmp_pl(const char *cs, const char *ct , size_t count)
+/* adjusted to use casefold utf8
+ * (normalized would be better but count would have to be adjusted)
+ * count is in bytes, to make it simpler
+ * XXX: potentially slow, try to get rid of it
+ *	in favour of something with the common string being normalized
+ *	before calling
+ */
+int strncasecmp_pl(const char *cs, const char *ct, size_t count)
 {
-	register signed char __res = 0;
-#if USE_UNICODE
-	wchar_t *wc1 = xmalloc((count+1) * sizeof(wchar_t));
-	wchar_t *wc2 = xmalloc((count+1) * sizeof(wchar_t));
-	wchar_t *p;
-	int n1 = mbsrtowcs(wc1, &cs, count, NULL);
-	int n2 = mbsrtowcs(wc2, &ct, count, NULL);
+	gchar *csc = g_utf8_casefold(cs, -1);
+	gchar *ctc = g_utf8_casefold(ct, -1);
 
-	wc1[n1<0 ? 0 : n1] = 0;
-	wc2[n2<0 ? 0 : n2] = 0;
+	gint ret = strncmp(csc, ctc, count);
 
-	for(p=wc1; *p; p++) *p = towlower(*p);
-	for(p=wc2; *p; p++) *p = towlower(*p);
+	g_free(csc);
+	g_free(ctc);
 
-	__res = wcscoll(wc1, wc2);
-
-	xfree(wc1);
-	xfree(wc2);
-#else
-	while (count) {
-		if ((__res = tolower_pl(*cs) - tolower_pl(*ct++)) != 0 || !*cs++)
-			break;
-		count--;
-	}
-#endif
-	return __res;
+	return ret;
 }
 
+#ifndef EKG_NO_DEPRECATED
+
 /*
  * saprintf()
  *
  * dziaa jak sprintf() tylko, e wyrzuca wskanik
  * do powstaego cigu
+ *
+ * NOTE: deprecated, please use g_strdup_printf() instead.
  */
 char *saprintf(const char *format, ...)
 {
@@ -2684,6 +2378,8 @@ char *saprintf(const char *format, ...)
 	return res;
 }
 
+#endif
+
 /*
  * xstrtr()
  *
@@ -2824,7 +2520,7 @@ int ekg_close(int fd) {
 char *password_input(const char *prompt, const char *rprompt, const bool norepeat) {
 	char *pass = NULL;
 
-	if (query_emit_id(NULL, UI_PASSWORD_INPUT, &pass, &prompt, norepeat ? NULL : &rprompt) == -2) {
+	if (query_emit(NULL, "ui-password-input", &pass, &prompt, norepeat ? NULL : &rprompt) == -2) {
 		print("password_nosupport");
 		return NULL;
 	}
@@ -2861,6 +2557,177 @@ int is_utf8_string(const char *txt) {
 	return 1;
 }
 
+/***************** variable stuff ******************/
+
+/*
+ * get_variable_value()
+ *
+ * Returns: a newly-allocated string holding the variable value.
+ * The returned string should be freed with g_free() when no longer needed.
+ */
+static char *get_variable_value(variable_t *v) {
+	/* We delay variable initialization until the
+	 * type is known to be such that is properly
+	 * aligned for reading an int.
+	 */
+	int number = *(int*)(v->ptr);
+	gchar *value = NULL;
+
+	if (!v->display) {
+		value = xstrdup(("(...)"));
+	} else if (v->type == VAR_STR || v->type == VAR_FILE || v->type == VAR_DIR || v->type == VAR_THEME) {
+		char *string = *(char**)(v->ptr);
+		value = (string) ? saprintf(("\"%s\""), string) : xstrdup(("(none)"));
+	} else if (v->type == VAR_BOOL) {
+		value = xstrdup( (number) ? ("1 (on)") : ("0 (off)") );
+	} else if ((v->type == VAR_INT || v->type == VAR_MAP) && !v->map) {
+		value = xstrdup(ekg_itoa(number));
+	} else if (v->type == VAR_INT && v->map) {
+		int i;
+
+		for (i = 0; v->map[i].label; i++)
+			if (v->map[i].value == number) {
+				value = saprintf(("%d (%s)"), number, v->map[i].label);
+				break;
+			}
+
+		if (!value)
+			value = saprintf(("%d"), number);
+
+	} else if (v->type == VAR_MAP && v->map) {
+		GString *s = g_string_new(ekg_itoa(number));
+		int i, first = 1;
+
+		for (i = 0; v->map[i].label; i++) {
+			if ((number & v->map[i].value) || (!number && !v->map[i].value)) {
+				g_string_append(s, (first) ? (" (") : (","));
+				first = 0;
+				g_string_append(s, v->map[i].label);
+			}
+		}
+
+		if (!first)
+			g_string_append_c(s, (')'));
+
+		value = g_string_free(s, FALSE);
+	} else if (v->type == -1) {
+		/* specjal type for "status" */
+		char *string = *(char**)(v->ptr);
+		value = string ? saprintf(("%s"), string) : xstrdup("");
+	}
+
+	return value;
+}
+
+/*
+ * variable_display()
+ *
+ * Displays variable value.
+ */
+void variable_display(variable_t *v, int quiet) {
+	gchar *value;
+
+	if (quiet || v->display == 2)
+		return;
+	
+	value = get_variable_value(v);
+	printq("variable", v->name, value);
+	g_free(value);
+}
+
+/*
+ * get_fake_sess_variable()
+ *
+ * Creates (variable_t) variable from session variable.
+ * The returned value should be freed with g_free() when no longer needed.
+ */
+static variable_t *get_fake_sess_variable(session_t *s, const char *name) {
+	static int fake_int_value;
+	static const char *val;
+	variable_t *var = g_malloc0(sizeof(variable_t));
+	int id;
+
+	var->name = (char *)name;
+	var->type = VAR_STR;
+	var->display = 1;
+	var->ptr = &val;
+
+	/* emulate session_get() */
+	if (!xstrcasecmp(name, "uid"))		val = session_uid_get(s);
+	else if (!xstrcasecmp(name, "alias"))	val = session_alias_get(s);
+	else if (!xstrcasecmp(name, "descr"))	val = session_descr_get(s);
+	else if (!xstrcasecmp(name, "status"))	{ var->type = -1; val = ekg_status_string(session_status_get(s), 2); }
+	else if (!xstrcasecmp(name, "statusdescr")) var->display = 2;
+	else if (!xstrcasecmp(name, "password")) var->display = 0;
+	else if ((id = plugin_var_find(s->plugin, name))) {
+		plugins_params_t *pa = &(((plugin_t *) s->plugin)->params[id-1]);
+		var->type = pa->type;
+		var->map = pa->map;
+		if ((var->type == VAR_INT) || (var->type == VAR_BOOL) || (var->type == VAR_MAP)) {
+			fake_int_value = s->values[id-1] ? atoi(s->values[id-1]) : 0;
+			var->ptr = &fake_int_value;
+		} else
+			var->ptr = &(s->values[id-1]);
+
+		if (pa->secret)
+			var->display = 0;
+	} else {
+		g_free(var);
+		return NULL;
+	}
+
+	return var;
+}
+
+/*
+ * session_variable_display()
+ *
+ * Displays session variable value (/session --get [session uid] <variable name>)
+ *
+ */
+int session_variable_display(session_t *s, const char *name, int quiet) {
+	gchar *value = NULL;
+	variable_t *var;
+
+	if (!(var = get_fake_sess_variable(s, name)))
+		return 0;
+
+	if (var->display != 2) {
+		value = get_variable_value(var);
+		printq("session_variable", session_name(s), name, value);
+	}
+
+	g_free(value);
+	g_free(var);
+
+	return 1;
+}
+
+/*
+ * session_variable_info()
+ *
+ * Displays variable value (/session <session uid>)
+ */
+void session_variable_info(session_t *s, const char *name, int quiet) {
+	gchar *value = NULL;
+	variable_t *var;
+
+	if ( quiet || !xstrcmp(name, "alias") )
+		return;
+
+	if (!(var = get_fake_sess_variable(s, name)))
+		return;
+
+	if (var->display != 2) {
+		value = get_variable_value(var);
+		printq("session_info_param", name, value);
+	}
+
+	g_free(value);
+	g_free(var);
+
+}
+
 /*
  * Local Variables:
  * mode: c
