@@ -19,13 +19,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "ekg2-config.h"
-#include "win32.h"
-
-#ifndef __FreeBSD__
-#define _XOPEN_SOURCE 600
-#define __EXTENSIONS__
-#endif
+#include "ekg2.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -43,27 +37,6 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "commands.h"
-#include "dynstuff.h"
-#ifndef HAVE_STRLCAT
-#  include "compat/strlcat.h"
-#endif
-#ifndef HAVE_STRLCPY
-#  include "compat/strlcpy.h"
-#endif
-#include "plugins.h"
-#include "stuff.h"
-#include "themes.h"
-#include "userlist.h"
-#include "vars.h"
-#include "windows.h"
-#include "xmalloc.h"
-#include "log.h"
-
-#include "debug.h"
-#include "dynstuff_inline.h"
-#include "queries.h"
-
 static void userlist_private_free(userlist_t *u);
 
 struct ignore_label ignore_labels[IGNORE_LABELS_MAX] = {
@@ -124,8 +97,8 @@ void userlist_add_entry(session_t *session, const char *line) {
 	userlist_t *u;
 	int count, i;
 
-	if ((count = array_count(entry)) < 7) {
-		array_free(entry);
+	if ((count = g_strv_length(entry)) < 7) {
+		g_strfreev(entry);
 		return;
 	}
 
@@ -135,7 +108,7 @@ void userlist_add_entry(session_t *session, const char *line) {
 	{
 		int function = EKG_USERLIST_PRIVHANDLER_READING;
 
-		query_emit_id(NULL, USERLIST_PRIVHANDLE, &u, &function, &entry, &count);
+		query_emit(NULL, "userlist-privhandle", &u, &function, &entry, &count);
 	}
 
 	if (valid_plugin_uid(session->plugin, u->uid) != 1) {
@@ -181,26 +154,22 @@ void userlist_add_entry(session_t *session, const char *line) {
  * @return 0 on success, -1 file not found
  */
 int userlist_read(session_t *session) {
-	const char *filename;
 	char *buf;
-	FILE *f;
+	GDataInputStream *f;
 
-	if (!(filename = prepare_pathf("%s-userlist", session->uid)))
-		return -1;
-	
-	if (!(f = fopen(filename, "r")))
+	if (!(f = G_DATA_INPUT_STREAM(config_open("%s-userlist", "r", session->uid))))
 		return -1;
 			
-	while ((buf = read_file(f, 0))) {
+	while ((buf = read_line(f))) {
 		if (buf[0] == '#' || (buf[0] == '/' && buf[1] == '/'))
 			continue;
 		
 		userlist_add_entry(session, buf);
 	}
 
-	query_emit_id(NULL, USERLIST_REFRESH);	/* XXX, wywolywac tylko kiedy dodalismy przynajmniej 1 */
+	query_emit(NULL, "userlist-refresh");	/* XXX, wywolywac tylko kiedy dodalismy przynajmniej 1 */
 
-	fclose(f);
+	g_object_unref(f);
 		
 	return 0;
 } 
@@ -220,22 +189,12 @@ int userlist_read(session_t *session) {
  *		-2 if we fail to create/open userlist file in rw mode
  */
 
-int userlist_write(session_t *session) {
-	const char *filename;
-	FILE *f;
+void userlist_write(session_t *session) {
+	GOutputStream *f;
 	userlist_t *ul;
 
-	if (!prepare_path(NULL, 1))	/* try to create ~/.ekg2 dir */
-		return -1;
-
-	if (!(filename = prepare_pathf("%s-userlist", session->uid)))
-		return -1;
-	
-	if (!(f = fopen(filename, "w"))) {
-		return -2;
-	}
-	fchmod(fileno(f), 0600);
-	fprintf(f, "# vim:fenc=%s\n", config_console_charset);
+	if (!(f = G_OUTPUT_STREAM(config_open("%s-userlist", "w", session->uid))))
+		return;
 
 	/* userlist_dump() */
 	for (ul = session->userlist; ul; ul = ul->next) {
@@ -254,21 +213,18 @@ int userlist_write(session_t *session) {
 		{
 			int function = EKG_USERLIST_PRIVHANDLER_WRITING;
 
-			query_emit_id(NULL, USERLIST_PRIVHANDLE, &u, &function, &entry);
+			query_emit(NULL, "userlist-privhandle", &u, &function, &entry);
 		}
 
 		line = array_join_count(entry, ";", 7);
 
-		fprintf(f, "%s%s\n", 
+		ekg_fprintf(f, "%s%s\n", 
 			line,					/* look upper */
 			u->foreign ? u->foreign : "");		/* backwards compatibility */
 
 		xfree(line);
 		array_free_count(entry, 7);
 	}
-
-	fclose(f);
-	return 0;
 }
 
 /**
@@ -326,7 +282,7 @@ static void userlist_private_free(userlist_t *u) {
 	if (u->priv) {
 		int func = EKG_USERLIST_PRIVHANDLER_FREE;
 
-		query_emit_id(NULL, USERLIST_PRIVHANDLE, &u, &func);
+		query_emit(NULL, "userlist-privhandle", &u, &func);
 	}
 }
 
@@ -334,7 +290,7 @@ void *userlist_private_get(plugin_t *plugin, userlist_t *u) {
 	int func = EKG_USERLIST_PRIVHANDLER_GET;
 	void *up = NULL;
 
-	query_emit_id(plugin, USERLIST_PRIVHANDLE, &u, &func, &up);
+	query_emit(plugin, "userlist-privhandle", &u, &func, &up);
 
 	return up;
 }
@@ -639,7 +595,7 @@ int valid_uid(const char *uid) {
 	char *tmp;
 	tmp = xstrdup(uid);
 
-	query_emit_id(NULL, PROTOCOL_VALIDATE_UID, &tmp, &valid);
+	query_emit(NULL, "protocol-validate-uid", &tmp, &valid);
 	xfree(tmp);
 
 	return (valid > 0);
@@ -671,7 +627,7 @@ int valid_plugin_uid(plugin_t *plugin, const char *uid) {
 
 	tmp = xstrdup(uid);
 
-	query_emit_id(plugin, PROTOCOL_VALIDATE_UID, &tmp, &valid);
+	query_emit(plugin, "protocol-validate-uid", &tmp, &valid);
 	xfree(tmp);
 
 	return (valid > 0);
@@ -809,7 +765,7 @@ const char *format_user(session_t *session, const char *uid) {
 	else
 		tmp = format_string(format_find("known_user"), u->nickname, uid);
 	
-	strlcpy(buf, tmp, sizeof(buf));
+	g_strlcpy(buf, tmp, sizeof(buf));
 	
 	xfree(tmp);
 
@@ -851,12 +807,12 @@ int ignored_remove(session_t *session, const char *uid) {
 
 	tmps	= xstrdup(session->uid);
 	tmp	= xstrdup(u->uid);
-	query_emit_id(NULL, PROTOCOL_IGNORE, &tmps, &tmp, &level, &tmp2);
+	query_emit(NULL, "protocol-ignore", &tmps, &tmp, &level, &tmp2);
 	xfree(tmps);
 	xfree(tmp);
 
 	if ((level & IGNORE_STATUS || level & IGNORE_STATUS_DESCR)) {
-		query_emit_id(NULL, PROTOCOL_UNIGNORE, &u, &session);
+		query_emit(NULL, "protocol-unignore", &u, &session);
 	}
 
 	return 0;
@@ -895,7 +851,7 @@ int ignored_add(session_t *session, const char *uid, ignore_t level) {
 
 	tmps	= xstrdup(session->uid);
 	tmp	= xstrdup(u->uid);
-	query_emit_id(NULL, PROTOCOL_IGNORE, &tmps, &tmp, &oldlevel, &level);
+	query_emit(NULL, "protocol-ignore", &tmps, &tmp, &oldlevel, &level);
 	xfree(tmps);
 	xfree(tmp);
 	
@@ -964,7 +920,7 @@ int ignore_flags(const char *str) {
 				ret |= ignore_labels[y].level;
 	}
 
-	array_free(arr);
+	g_strfreev(arr);
 
 	return ret;
 }
@@ -994,9 +950,9 @@ const char *ignore_format(int level) {
 	for (i = 0; ignore_labels[i].name; i++) {
 		if (level & ignore_labels[i].level) {
 			if (comma++)
-				strlcat(buf, ",", sizeof(buf));
+				g_strlcat(buf, ",", sizeof(buf));
 
-			strlcat(buf, ignore_labels[i].name, sizeof(buf));
+			g_strlcat(buf, ignore_labels[i].name, sizeof(buf));
 		}
 	}
 
@@ -1114,7 +1070,7 @@ struct ekg_group *group_init(const char *names) {
 		g->name = groups[i];
 		ekg_groups_add(&gl, g);
 	}
-	/* NOTE: we don't call here array_free() cause we use items of this
+	/* NOTE: we don't call here g_strfreev() cause we use items of this
 	 *	array @ initing groups. We don't use strdup()
 	 */
 	xfree(groups);
