@@ -1,6 +1,6 @@
 /* $Id$ */
 
-#include "ekg2-config.h"
+#include "ekg2.h"
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -13,24 +13,11 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <ekg/audio.h>
-#include <ekg/commands.h>
-#include <ekg/debug.h>
-#include <ekg/protocol.h>
-#include <ekg/stuff.h>
-#include <ekg/themes.h>
-#include <ekg/userlist.h>
-#include <ekg/xmalloc.h>
-
 #include <libgadu.h>
 
 #include "dcc.h"
 #include "gg.h"
 
-int audiofds[2] = { -1, -1};
-
-AUDIO_DEFINE(gg_dcc);
-
 struct gg_dcc *gg_dcc_socket = NULL;
 
 static int dcc_limit_time = 0;	/* time from the first connection */
@@ -43,122 +30,6 @@ int gg_config_dcc_port;
 char *gg_config_dcc_ip;
 char *gg_config_dcc_limit;
 
-typedef struct {
-	dcc_t *dcc;
-	int len;
-} gg_audio_private_t;
-
-/* some audio stuff */
-AUDIO_CONTROL(gg_dcc_audio_control) {
-	va_list ap;
-	if (type == AUDIO_CONTROL_INIT) {
-		va_start(ap, aio);
-
-		va_end(ap);
-		return (void *) 1;
-	} else if ((type == AUDIO_CONTROL_SET && !aio) || (type == AUDIO_CONTROL_GET && aio)) {
-		gg_audio_private_t *priv;
-		char *attr;
-		int dccid = -1;
-		int dccfd = -1;
-		int len = GG_DCC_VOICE_FRAME_LENGTH_505;
-
-		if (type == AUDIO_CONTROL_GET)	priv = aio->priv_data;
-		else				priv= xmalloc(sizeof(gg_audio_private_t));
-
-		va_start(ap, aio);
-
-		while ((attr = va_arg(ap, char *))) {
-			if (type == AUDIO_CONTROL_GET) {
-				char **val = va_arg(ap, char **);
-				debug("[gg_dcc_audio_control AUDIO_CONTROL_GET] attr: %s value: 0x%x\n", attr, val);
-
-				if (!xstrcmp(attr, "format"))		*val = xstrdup("gsm");
-				else					*val = NULL;
-			} else {
-				char *val = va_arg(ap, char *);
-				debug("[gg_dcc_audio_control AUDIO_CONTROL_SET] attr: %s value: %s\n", attr, val);
-
-				if (!xstrcmp(attr, "dccid"))		dccid = atoi(val);
-				else if (!xstrcmp(attr, "fd"))		dccfd = atoi(val);
-				else if (!xstrcmp(attr, "len"))		len = atoi(val);
-			}
-		}
-		va_end(ap);
-		{ 
-			dcc_t *d;
-			for (d = dccs; d; d = d->next) {
-				if (d->id == dccid) {
-					priv->dcc = d;
-					break;
-				}
-			}
-		}
-		if (!priv->dcc) { xfree(priv); return NULL; }
-		priv->len = len;
-
-		aio		= xmalloc(sizeof(audio_io_t));
-		aio->a		= &gg_dcc_audio;
-		aio->priv_data	= priv;
-		aio->fd		= dccfd;
-	} else if (type == AUDIO_CONTROL_HELP) {
-		return NULL;
-	}
-	return aio;
-} 
-
-WATCHER_AUDIO(gg_dcc_audio_read) {
-	char sbuf[GG_DCC_VOICE_FRAME_LENGTH_505];
-	int len;
-
-	if (type) return -1;
-	
-	len = read(fd, sbuf, sizeof(buf));
-	sbuf[len] = 0;
-
-	if (len > 0) {
-		if (len == GG_DCC_VOICE_FRAME_LENGTH_505)
-			string_append_raw(buf, sbuf+1, len-1);
-		else	string_append_raw(buf, sbuf, len);
-	}
-
-	debug("gg_dcc_audio_read() %d\n", len);
-	return len;
-}
-
-WATCHER_AUDIO(gg_dcc_audio_write) {
-	gg_audio_private_t *priv = data;
-	int len = -1; 
-	int rlen;
-
-	char output[GG_DCC_VOICE_FRAME_LENGTH_505];
-
-//	debug("gg_dcc_audio_write type: %d\n", type);
-	if (type) return 0;
-
-	if (!dccs || !priv->dcc) {
-		debug("gg_dcc_audio_write DCC NOT FOUND\n");
-		return -1;
-	}
-
-	if (!priv->dcc->active) return buf->len;
-
-	if (priv->len == GG_DCC_VOICE_FRAME_LENGTH_505) rlen = GG_DCC_VOICE_FRAME_LENGTH_505 - 1;
-	else						rlen = priv->len;
-
-	if (buf->len < rlen) return 0;
-
-	if (priv->len == GG_DCC_VOICE_FRAME_LENGTH_505) {
-		output[0] = 0;
-		memcpy(&(output[1]), buf->str, rlen);
-	} else	memcpy(&(output[0]), buf->str, rlen);
-
-	if (!(gg_dcc_voice_send(priv->dcc->priv, output, priv->len)))
-		len = rlen;
-
-	return len;
-}
-
 /* 
  * gg_changed_dcc()
  *
@@ -205,12 +76,8 @@ void gg_changed_dcc(const char *var)
 	
 		}
 	} else if (!xstrcmp(var, ("gg:audio"))) {
-		if (gg_config_audio && (!audio_find("oss") || !codec_find("gsm"))) {
-			gg_config_audio = 0;
-			debug("[gg_config_audio] failed to set gg:audio to 1 cause not found oss audio or gsm codec...\n");
-			return;
-		} else if (gg_config_audio)	gg_dcc_audio_init();
-		else				gg_dcc_audio_close();
+		gg_config_audio = 0;
+		debug("[gg_config_audio] gg:audio not supported.\n");
 	}
 
 	if (!in_autoexec)
@@ -377,114 +244,13 @@ COMMAND(gg_command_dcc)
 	}
 
 	if (params[0] && (params[0][0] == 'v' || !xstrncasecmp(params[0], "rvo", 3))) {			/* voice, rvoice */
-		struct gg_dcc *gd = NULL;
-		dcc_t *d;
-		userlist_t *u;
-		gg_userlist_private_t *up;
-		int __ip, __port;
-
 		if (!params[1]) {
 			printq("not_enough_params", name);
 			return -1;
 		}
 
-		if (!gg_config_audio) {
-			printq("dcc_voice_unsupported");
-			return -1;
-		}
-
-		if (!(u = userlist_find(session, get_uid(session, params[1])))) {
-			printq("user_not_found", params[1]);
-			return -1;
-		}
-		up = gg_userlist_priv_get(u);
-
-		if (!session_connected_get(session)) {
-			printq("not_connected", session_name(session));
-			return -1;
-		}
-
-		if (u->status == EKG_STATUS_NA) {
-			printq("dcc_user_not_avail", format_user(session, u->uid));
-			return -1;
-		}
-
-		__ip = user_private_item_get_int(u, "ip");
-
-		if (!__ip) {
-			printq("dcc_user_aint_dcc", format_user(session, u->uid));
-			return -1;
-		}
-		
-		__port = user_private_item_get_int(u, "port");
-
-#if 0
-		list_t l;
-		struct transfer *t, tt;
-		/* sprawdzamy najpierw przychodzce poczenia */
-		
-		for (t = NULL, l = transfers; l; l = l->next) {
-			struct transfer *f = l->data;
-			struct userlist *u;
-			
-			f = l->data;
-
-			if (!f->dcc || !f->dcc->incoming || f->type != GG_SESSION_DCC_VOICE)
-				continue;
-			
-			if (params[1][0] == '#' && atoi(params[1] + 1) == f->id) {
-				t = f;
-				break;
-			}
-
-			if (t && (u = userlist_find(t->uin, NULL))) {
-				if (!xstrcasecmp(params[1], itoa(u->uin)) || (u->display && !xstrcasecmp(params[1], u->display))) {
-					t = f;
-					break;
-				}
-			}
-		}
-
-		if (t) {
-			if ((u = userlist_find(t->uin, NULL)))
-				t->protocol = u->protocol;
-
-			list_add(&watches, t->dcc, 0);
-			voice_open();
-			return 0;
-		}
-#endif
-		if (audiofds[0] != -1 || audiofds[1] != -1) {
-			printq("dcc_voice_running");
-			return 0;
-		}
-
-		if (__port < 10 || !xstrncasecmp(params[0], "rvo", 3)) {
-			/* nie moemy si z nim poczy, wic on sprbuje */
-			gg_dcc_request(g->sess, atoi(u->uid + 3));
-		} else {
-			if (!(gd = gg_dcc_voice_chat(__ip, __port, uin, atoi(u->uid + 3)))) {
-				printq("dcc_error", strerror(errno));
-				return -1;
-			}
-		}
-		if (!(d = dcc_add(session, u->uid, DCC_VOICE, gd))) return -1;
-
-		if (gd)
-			watch_add(&gg_plugin, gd->fd, gd->check, gg_dcc_handler, gd);
-
-		pipe(audiofds);
-		stream_create("Gygy audio OUTPUT",
-				__AINIT_F("oss", AUDIO_READ, "freq", "8000", "sample", "16", "channels", "1"),
-				__CINIT_F("gsm", "with-ms", "1"),
-				__AINIT_F("gg_dcc", AUDIO_WRITE, "dccuid", u->uid, 
-					"len", (up->protocol >= 0x1b) ? itoa(GG_DCC_VOICE_FRAME_LENGTH_505) : itoa(GG_DCC_VOICE_FRAME_LENGTH),
-					"dccid", itoa(d->id) /*, "fd", itoa(audiofds[1]) */ ));
-		stream_create("Gygy audio INPUT",
-				__AINIT_F("gg_dcc", AUDIO_READ, "dccid", itoa(d->id), "uid", u->uid, "fd", itoa(audiofds[0])), 
-				__CINIT_F("gsm", "with-ms", "1"),
-				__AINIT_F("oss", AUDIO_WRITE, "freq", "8000", "sample", "16", "channels", "1"));
-		return 0;
+		printq("dcc_voice_unsupported");
+		return -1;
 	}
 	
 	/* get, resume */
@@ -605,11 +371,6 @@ static void gg_dcc7_close_handler(dcc_t *d)
 	if (!d || !(g = d->priv))
 		return;
 	
-	if (d->type == DCC_VOICE) {
-		close(audiofds[0]);
-		close(audiofds[1]);
-		audiofds[0] = audiofds[1] = -1;
-	}
 	gg_dcc7_free(g);
 }
 
@@ -807,7 +568,7 @@ WATCHER(gg_dcc_handler)	/* tymczasowy */
 
 			__host = inet_ntoa(*((struct in_addr*) &d->remote_addr));
 			__port = d->remote_port;
-			query_emit(NULL, ("protocol-dcc-validate"), &__host, &__port, &__valid, NULL);
+			query_emit(NULL, "protocol-dcc-validate", &__host, &__port, &__valid, NULL);
 
 			if (__valid)
 				watch_add(&gg_plugin, d->fd, d->check, gg_dcc_handler, d);
@@ -925,7 +686,7 @@ WATCHER(gg_dcc_handler)	/* tymczasowy */
 			dcc_filename_set(D, (const char *) d->file_info.filename);
 			dcc_size_set(D, d->file_info.size);
 
-			print("dcc_get_offer", format_user(session_find(uin), dcc_uid_get(D)), dcc_filename_get(D), itoa(d->file_info.size), itoa(dcc_id_get(D)));
+			print("dcc_get_offer", format_user(session_find(uin), dcc_uid_get(D)), dcc_filename_get(D), ekg_itoa(d->file_info.size), ekg_itoa(dcc_id_get(D)));
 
 			if (config_dcc_dir)
 				path = saprintf("%s/%s", config_dcc_dir, dcc_filename_get(D));
@@ -933,49 +694,13 @@ WATCHER(gg_dcc_handler)	/* tymczasowy */
 				path = xstrdup(dcc_filename_get(D));
 
 			if (!stat(path, &st) && st.st_size < d->file_info.size)
-				print("dcc_get_offer_resume", format_user(session_find(uin), dcc_uid_get(D)), dcc_filename_get(D), itoa(d->file_info.size), itoa(dcc_id_get(D)));
+				print("dcc_get_offer_resume", format_user(session_find(uin), dcc_uid_get(D)), dcc_filename_get(D), ekg_itoa(d->file_info.size), ekg_itoa(dcc_id_get(D)));
 			
 			xfree(path);
 
 			break;
 		}
 			
-		case GG_EVENT_DCC_NEED_VOICE_ACK:
-			debug("[gg] GG_EVENT_DCC_NEED_VOICE_ACK\n");
-#if 0
-			if (gg_config_audio && 0) {
-				/* eby nie sprawdzao, pki luser nie odpowie */
-				list_remove(&watches, d, 0);
-
-				if (!(t = find_transfer(d))) {
-					tt.uin = d->peer_uin;
-					tt.type = GG_SESSION_DCC_VOICE;
-					tt.filename = NULL;
-					tt.dcc = d;
-					tt.id = transfer_id();
-					if (!(t = list_add(&transfers, &tt, sizeof(tt)))) {
-						gg_free_dcc(d);
-						break;
-					}
-				}
-			
-				t->type = GG_SESSION_DCC_VOICE;
-
-				print("dcc_voice_offer", format_user(t->uin), itoa(t->id));
-			} else { 
-				list_remove(&watches, d, 0);
-				remove_transfer(d);
-				gg_free_dcc(d);
-			}
-#endif
-			break;
-
-		case GG_EVENT_DCC_VOICE_DATA:
-			debug("[gg] GG_EVENT_DCC_VOICE_DATA\n");
-			if (gg_config_audio && audiofds[1] != -1) {
-				write(audiofds[1], e->event.dcc_voice_data.data, e->event.dcc_voice_data.length);
-			}
-			break;
 		case GG_EVENT_DCC_DONE:
 		{
 			char uin[16];
@@ -1038,20 +763,16 @@ WATCHER(gg_dcc_handler)	/* tymczasowy */
 			}
 
 			xfree(tmp);
-			if (d->type == GG_SESSION_DCC_VOICE) {
-				close(audiofds[1]);
-				close(audiofds[0]);
-
-				audiofds[0] = -1;
-				audiofds[1] = -1;
-
-			}
 			dcc_close(gg_dcc_find(d));
 			gg_free_dcc(d);
 			d = NULL;
 
 			break;
 		}
+
+		case GG_EVENT_DCC_NEED_VOICE_ACK:
+		case GG_EVENT_DCC_VOICE_DATA:
+			break;
 	}
 
 	/* uaktualnij statystyki */
@@ -1118,26 +839,6 @@ void gg_dcc_socket_close()
 	gg_dcc_socket = NULL;
 }
 
-void gg_dcc_audio_init() {
-	if (gg_config_audio) {
-		if (audiofds[0] != -1) close(audiofds[0]);
-		if (audiofds[1] != -1) close(audiofds[1]);
-		audiofds[0] = -1;
-		audiofds[1] = -1;
-		audio_register(&gg_dcc_audio);
-	}
-}
-
-void gg_dcc_audio_close() {
-	if (!gg_config_audio) {
-		if (audiofds[0] != -1) close(audiofds[0]);
-		if (audiofds[1] != -1) close(audiofds[1]);
-		audiofds[0] = -1;
-		audiofds[1] = -1;
-		audio_unregister(&gg_dcc_audio);
-	}
-}
-
 /*
  * Local Variables:
  * mode: c
