@@ -19,13 +19,11 @@
 #ifndef __EKG_GG_IMAGES_H
 #define __EKG_GG_IMAGES_H
 
-#include <ekg/dynstuff.h>
-
 typedef struct {
 	char *filename;
 	char *data;
-	uint32_t size;
-	uint32_t crc32;
+	guint32 size;
+	guint32 crc32;
 } image_t;
 
 extern list_t images;
