@@ -15,7 +15,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#define _FILE_OFFSET_BITS 64	/* allow selection of large files */
+
+#include "ekg2.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
