@@ -20,6 +20,8 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "ekg2.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -33,21 +35,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include <ekg/debug.h>
 #include <ekg/net.h>
-#include <ekg/plugins.h>
-#include <ekg/recode.h>
-#include <ekg/queries.h>
-#include <ekg/protocol.h>
-#include <ekg/themes.h>
-#include <ekg/windows.h>	// XXX
-#include <ekg/stuff.h>
-#include <ekg/vars.h>
-#include <ekg/xmalloc.h>
-
-#include <ekg/log.h>
-#include <ekg/recode.h>
-#include <ekg/msgqueue.h>
 
 #include "icq.h"
 #include "misc.h"
@@ -60,33 +48,32 @@
 #define ICQ_HUB_SERVER "login.icq.com"
 #define ICQ_HUB_PORT	5190
 
+int icq_config_disable_chatstates = 0;
+
 static int icq_theme_init();
 PLUGIN_DEFINE(icq, PLUGIN_PROTOCOL, icq_theme_init);
 
-int icq_send_pkt(session_t *s, string_t buf) {
+int icq_send_pkt(session_t *s, GString *buf) {
 	icq_private_t *j;
-	int fd;
 
 	if (!s || !(j = s->priv) || !buf) {
-		string_free(buf, 1);
+		g_string_free(buf, TRUE);
 		return -1;
 	}
 
-	fd = j->fd;
-
-	debug_io("icq_send_pkt(%s) fd: %d len: %d\n", s->uid, fd, buf->len);
+	debug_io("icq_send_pkt(%s) len: %d\n", s->uid, buf->len);
 	icq_hexdump(DEBUG_IO, (unsigned char *) buf->str, buf->len);
 	if (j->migrate)
 		debug_warn("Client migrate! Packet will not be send\n");
-	else	
-		ekg_write(fd, buf->str, buf->len);
-	string_free(buf, 1);
+	else
+		ekg_connection_write_buf(j->send_stream, buf->str, buf->len);
+	g_string_free(buf, TRUE);
 	return 0;
 }
 
 static TIMER_SESSION(icq_ping) {
 	icq_private_t *j;
-	string_t pkt;
+	GString *pkt;
 
 	if (type)
 		return 0;
@@ -94,7 +81,7 @@ static TIMER_SESSION(icq_ping) {
 	if (!s || !(j = s->priv) || !s->connected)
 		return -1;
 
-	pkt = string_init(NULL);
+	pkt = g_string_new(NULL);
 	icq_makeflap(s, pkt, 0x05);
 	icq_send_pkt(s, pkt);
 
@@ -130,7 +117,7 @@ int icq_write_status_msg(session_t *s) {
 
 int icq_write_status(session_t *s) {
 	icq_private_t *j = s->priv;
-	uint32_t status;
+	guint32 status;
 
 	if (!s || !s->connected)
 		return 0;
@@ -153,9 +140,9 @@ int icq_write_info(session_t *s) {
 #define m_bAvatarsEnabled 0
 #define m_bUtfEnabled 1
 
-	string_t pkt, tlv_5;
+	GString *pkt, *tlv_5;
 
-	tlv_5 = string_init(NULL);
+	tlv_5 = g_string_new(NULL);
 
 #ifdef DBG_CAPMTN
 	icq_pack_append_cap(tlv_5, CAP_TYPING);
@@ -208,14 +195,14 @@ int icq_write_info(session_t *s) {
 	icq_makesnac(s, pkt, 0x02, 0x04, NULL, 0);
 	icq_send_pkt(s, pkt);
 
-	string_free(tlv_5, 1);
+	g_string_free(tlv_5, TRUE);
 	return 0;
 }
 
 void icq_set_security(session_t *s) {
 	icq_private_t *j;
-	string_t pkt;
-	uint8_t webaware;
+	GString *pkt;
+	guint8 webaware;
 
 	if (!s || !(j = s->priv))
 		return;
@@ -246,7 +233,7 @@ void icq_set_security(session_t *s) {
 
 void icq_session_connected(session_t *s) {
 	icq_private_t *j = s->priv;
-	string_t pkt;
+	GString *pkt;
 
 	icq_write_info(s);
 
@@ -280,35 +267,34 @@ void icq_session_connected(session_t *s) {
 
 	/* SNAC 1,1E: Set status */
 	{
-		string_t pkt;
-		string_t tlv_c;
-		uint32_t cookie, status;
+		GString *pkt, *tlv_c;
+		guint32 cookie, status;
 
 		cookie = rand() <<16 | rand();
 		status = (j->status_flags << 16) | icq_status(s->status);
 
-		pkt = string_init(NULL);
+		pkt = g_string_new(NULL);
 
 		icq_pack_append(pkt, "tI", icq_pack_tlv_dword(0x06, status));	/* TLV 6: Status mode and security flags */
 		icq_pack_append(pkt, "tW", icq_pack_tlv_word(0x08, 0x00));	/* TLV 8: Error code */
 
 		/* TLV C: Direct connection info */
-		tlv_c = string_init(NULL);
-		icq_pack_append(tlv_c, "I", (uint32_t) 0x00000000);	/* XXX, getSettingDword(NULL, "RealIP", 0) */
-		icq_pack_append(tlv_c, "I", (uint32_t) 0x00000000);	/* XXX, nPort */
+		tlv_c = g_string_new(NULL);
+		icq_pack_append(tlv_c, "I", (guint32) 0x00000000);	/* XXX, getSettingDword(NULL, "RealIP", 0) */
+		icq_pack_append(tlv_c, "I", (guint32) 0x00000000);	/* XXX, nPort */
 		icq_pack_append(tlv_c, "C", DC_NORMAL);			/* Normal direct connection (without proxy/firewall) */
 		icq_pack_append(tlv_c, "W", ICQ_VERSION);		/* Protocol version */
 		icq_pack_append(tlv_c, "I", cookie);			/* DC Cookie */
 		icq_pack_append(tlv_c, "I", WEBFRONTPORT);		/* Web front port */
 		icq_pack_append(tlv_c, "I", CLIENTFEATURES);		/* Client features*/
-		icq_pack_append(tlv_c, "I", (uint32_t) 0xffffffff);	/* gbUnicodeCore ? 0x7fffffff : 0xffffffff */ /* Abused timestamp */
-		icq_pack_append(tlv_c, "I", (uint32_t) 0x80050003);	/* Abused timestamp */
-		icq_pack_append(tlv_c, "I", (uint32_t) 0x00000000);	/* XXX, Timestamp */
-		icq_pack_append(tlv_c, "W", (uint32_t) 0x0000);		/* Unknown */
+		icq_pack_append(tlv_c, "I", (guint32) 0xffffffff);	/* gbUnicodeCore ? 0x7fffffff : 0xffffffff */ /* Abused timestamp */
+		icq_pack_append(tlv_c, "I", (guint32) 0x80050003);	/* Abused timestamp */
+		icq_pack_append(tlv_c, "I", (guint32) 0x00000000);	/* XXX, Timestamp */
+		icq_pack_append(tlv_c, "W", (guint32) 0x0000);		/* Unknown */
 
 		icq_pack_append(pkt, "T", icq_pack_tlv(0x0C, tlv_c->str, tlv_c->len));
 
-		string_free(tlv_c, 1);
+		g_string_free(tlv_c, TRUE);
 
 		/* TLV(0x1F) - unknown? */
 		icq_pack_append(pkt, "tW", icq_pack_tlv_word(0x1F, 0x00));
@@ -317,15 +303,15 @@ void icq_session_connected(session_t *s) {
 		if (j->xstatus && (j->xstatus - 1 <= MAX_ICQMOOD)) {
 			char *mood = saprintf("icqmood%d", j->xstatus - 1);
 
-			string_t tlv_1d = icq_pack("WCC",
-						(uint32_t) 0x0e,	// item type
-						(uint32_t) 0,		// item flags
-						xstrlen(mood));
-			string_append(tlv_1d, mood);
+			GString *tlv_1d = icq_pack("WCC",
+						(guint32) 0x0e,		// item type
+						(guint32) 0,		// item flags
+						(guint32) xstrlen(mood));
+			g_string_append(tlv_1d, mood);
 
 			icq_pack_append(pkt, "T", icq_pack_tlv(0x1d, tlv_1d->str, tlv_1d->len));
 
-			string_free(tlv_1d, 1);
+			g_string_free(tlv_1d, TRUE);
 			xfree(mood);
 		}
 
@@ -334,7 +320,7 @@ void icq_session_connected(session_t *s) {
 	}
 
 	/* SNAC(1,0x11) - Set idle time */
-	icq_send_snac(s, 0x01, 0x11, NULL, NULL, "I", (uint32_t) 0);
+	icq_send_snac(s, 0x01, 0x11, NULL, NULL, "I", (guint32) 0);
 
 	/* j->idleAllow = 0; */
 
@@ -345,17 +331,17 @@ void icq_session_connected(session_t *s) {
 	icq_send_snac(s, 0x01, 0x02, NULL, NULL,
 			"WWWW WWWW WWWW WWWW WWWW WWWW WWWW WWWW WWWW WWWW WWWW",
 			/* family number, family version, family tool id, family tool version */
-			(uint32_t) 0x01, (uint32_t) 0x04, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x02, (uint32_t) 0x01, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x03, (uint32_t) 0x01, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x04, (uint32_t) 0x01, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x06, (uint32_t) 0x01, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x09, (uint32_t) 0x01, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x0a, (uint32_t) 0x01, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x0b, (uint32_t) 0x01, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x13, (uint32_t) 0x04, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x15, (uint32_t) 0x01, (uint32_t) 0x0110, (uint32_t) 0x161b,
-			(uint32_t) 0x22, (uint32_t) 0x01, (uint32_t) 0x0110, (uint32_t) 0x161b
+			(guint32) 0x01, (guint32) 0x04, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x02, (guint32) 0x01, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x03, (guint32) 0x01, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x04, (guint32) 0x01, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x06, (guint32) 0x01, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x09, (guint32) 0x01, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x0a, (guint32) 0x01, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x0b, (guint32) 0x01, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x13, (guint32) 0x04, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x15, (guint32) 0x01, (guint32) 0x0110, (guint32) 0x161b,
+			(guint32) 0x22, (guint32) 0x01, (guint32) 0x0110, (guint32) 0x161b
 			);
 
 	debug_ok(" *** Yeehah, login sequence complete\n");
@@ -375,7 +361,7 @@ void icq_session_connected(session_t *s) {
 		 * again. You should ask it to delete them by SNAC(15,02)/003E
 		 */
 		/* XXX, cookie */
-		pkt = string_init(NULL);
+		pkt = g_string_new(NULL);
 		icq_makemetasnac(s, pkt, CLI_OFFLINE_MESSAGE_REQ, 0, NULL, NULL);
 		icq_send_pkt(s, pkt);
 
@@ -430,18 +416,18 @@ void icq_session_connected(session_t *s) {
 			icq_send_snac(s, 0x13, 0x08, 0, 0,		// SSI edit: add item(s)
 					"U WW W W WWW",
 					j->default_group_name,		// default group name
-					(uint16_t) j->default_group_id, // Group#
-					(uint16_t) 0,			// Item#
-					(uint16_t) 1,			// Group record
-					(uint16_t) 6,			// Length of the additional data
-					(uint16_t) 0xc8, (uint16_t) 2, (uint16_t) 0	// tlv(0xc8) (len=2, val=0)
+					(guint16) j->default_group_id, // Group#
+					(guint16) 0,			// Item#
+					(guint16) 1,			// Group record
+					(guint16) 6,			// Length of the additional data
+					(guint16) 0xc8, (guint16) 2, (guint16) 0	// tlv(0xc8) (len=2, val=0)
 					);
 			icq_send_snac(s, 0x13, 0x12, 0, 0, "");	// Contacts edit end (finish transaction)
 		}
 	}
 }
 
-static uint32_t icq_get_uid(session_t *s, const char *target) {
+static guint32 icq_get_uid(session_t *s, const char *target) {
 	const char *uid;
 	char *first_invalid = NULL;
 	long int uin;
@@ -483,9 +469,7 @@ static QUERY(icq_session_init) {
 #endif
 
 	j = xmalloc(sizeof(icq_private_t));
-	j->fd = -1;
-	j->fd2= -1;
-	j->stream_buf = string_init(NULL);
+	j->stream_buf = g_string_new(NULL);
 
 	s->priv = j;
 	return 0;
@@ -500,13 +484,14 @@ static QUERY(icq_session_deinit) {
 		return 1;
 #if OLD_ICQ
 	userlist_write(s);
+	config_commit();
 #endif
 	s->priv = NULL;
 
 	private_items_destroy(&j->whoami);
 	xfree(j->default_group_name);
-	string_free(j->cookie, 1);
-	string_free(j->stream_buf, 1);
+	g_string_free(j->cookie, TRUE);
+	g_string_free(j->stream_buf, TRUE);
 	icq_snac_references_list_destroy(&j->snac_ref_list);
 	icq_rates_destroy(s);
 
@@ -541,28 +526,30 @@ static QUERY(icq_print_version) {
 static QUERY(icq_typing_out) {
 	const char *session	= *va_arg(ap, const char **);
 	const char *uid		= *va_arg(ap, const char **);
-	const int len		= *va_arg(ap, const int *);
-	int first		= *va_arg(ap, const int *);
-	uint32_t q1 = rand(), q2 = rand();
-	uint16_t typing = 0;
+	const int chstate	= *va_arg(ap, const int *);
+
+	guint32 q1 = rand(), q2 = rand();
+	guint16 typing;
 
 	session_t *s = session_find(session);
 
+	if (icq_config_disable_chatstates)
+		return -1;
+
 	if (!s || s->plugin != &icq_plugin)
 		return 0;
 
-	if (len>0)
-		typing = (first == 1) ? 2 : 1;
+	typing = (chstate==EKG_CHATSTATE_COMPOSING);
 
 	icq_send_snac(s, 0x04, 0x14, NULL, NULL,
-			"iiWsW", q1, q2, (uint16_t) 1, uid + 4, typing);
+			"iiWsW", q1, q2, (guint16) 1, uid + 4, typing);
 
 	return 0;
 }
 
 void icq_handle_disconnect(session_t *s, const char *reason, int type) {
 	icq_private_t *j;
-	string_t str;
+	GString *str;
 	const char *__reason = reason ? reason : "";
 
 	if (!s || !(j = s->priv))
@@ -574,90 +561,50 @@ void icq_handle_disconnect(session_t *s, const char *reason, int type) {
 	if (s->connected) {
 		/* XXX ?wo? recode & length check */
 		str = icq_pack("WC C U",
-			    (uint32_t) 2, (uint32_t) 4,		/* avatar type & flags -- offline message */
-			    (uint32_t) xstrlen(__reason) + 2,	/* length of message part */
+			    (guint32) 2, (guint32) 4,		/* avatar type & flags -- offline message */
+			    (guint32) xstrlen(__reason) + 2,	/* length of message part */
 			    __reason				/* message */
 			    );
 		icq_send_snac(s, 0x01, 0x1e, 0, 0,		/* Set status (set location info) */
 			"T", icq_pack_tlv(0x1d, str->str, str->len));
-		string_free(str, 1);
+		g_string_free(str, TRUE);
 	}
 
 	timer_remove_session(s, "ping");
 	timer_remove_session(s, "snac_timeout");
 	protocol_disconnected_emit(s, reason, type);
 
-	if (j->fd != -1) {
-		ekg_close(j->fd);
-		j->fd = -1;
-	}
-
-	if (j->fd2 != -1) {
-		ekg_close(j->fd2);
-		j->fd2 = -1;
-	}
-	string_clear(j->stream_buf);
+	g_string_set_size(j->stream_buf, 0);
 	j->migrate = 0;
 }
 
-static WATCHER_SESSION(icq_handle_stream);
-
-static WATCHER_SESSION(icq_handle_connect) {
+static void icq_handle_stream(GDataInputStream *input, gpointer data) {
+	session_t *s = data;
 	icq_private_t *j = NULL;
-	int res = 0;
-	socklen_t res_size = sizeof(res);
-
-	if (type)
-		return 0;
-
-	if (type == 1) {
-		debug ("[icq] handle_connect(): type %d\n", type);
-		return 0;
-	}
-
-	if (!s || !(j = s->priv)) {
-		debug_error("icq_handle_connect() s: 0x%x j: 0x%x\n", s, j);
-		return -1;
-	}
-
-	debug("[icq] handle_connect(%d)\n", s->connecting);
-
-	string_clear(j->stream_buf);
-
-	if (type || getsockopt(fd, SOL_SOCKET, SO_ERROR, &res, &res_size) || res) {
-		if (type)
-			debug_error("[icq] handle_connect(): SO_ERROR %s\n", strerror(res));
-
-		icq_handle_disconnect(s, (type == 2) ? "Connection timed out" : strerror(res), EKG_DISCONNECT_FAILURE);
-	}
-
-	watch_add_session(s, fd, WATCH_READ, icq_handle_stream);
-
-	return -1;
-}
-
-static WATCHER_SESSION(icq_handle_stream) {
-	icq_private_t *j = NULL;
-	char buf[8192];
-	int len, ret, start_len, left;
+	gsize count;
+	int left, ret, start_len;
 
 	if (!s || !(j = s->priv)) {
 		debug_error("icq_handle_stream() s: 0x%x j: 0x%x\n", s, j);
-		return -1;
+		return;
 	}
 
-	if (type)
-		return 0;
+	count = g_buffered_input_stream_get_available(G_BUFFERED_INPUT_STREAM(input));
 
-	len = read(fd, buf, sizeof(buf));
+	if (count>0) {
+		char *tmp = g_malloc(count);
+		gssize result = g_input_stream_read(G_INPUT_STREAM(input), tmp, count, NULL, NULL);
 
-	string_append_raw(j->stream_buf, buf, len);
+		g_string_append_len(j->stream_buf, tmp, result);
 
-	debug_iorecv("icq_handle_stream(%d) fd: %d; rcv: %d, %d in buffer.\n", s->connecting, fd, len, j->stream_buf->len);
+		g_free(tmp);
+	}
 
-	if (len < 1) {
+	debug_iorecv("icq_handle_stream(%d) rcv: %d, %d in buffer.\n", s->connecting, count, j->stream_buf->len);
+
+	if (count < 1) {
 		icq_handle_disconnect(s, strerror(errno), EKG_DISCONNECT_NETWORK);
-		return -1;
+		return;
 	}
 
 	icq_hexdump(DEBUG_IORECV, (unsigned char *) j->stream_buf->str, j->stream_buf->len);
@@ -668,7 +615,7 @@ static WATCHER_SESSION(icq_handle_stream) {
 
 	if ( (left = j->stream_buf->len) > 0 ) {
 		j->stream_buf->len = start_len;
-		string_remove(j->stream_buf, start_len - left);
+		g_string_erase(j->stream_buf, 0, start_len - left);
 	}
 
 	switch (ret) {		/* XXX, magic values */
@@ -683,110 +630,71 @@ static WATCHER_SESSION(icq_handle_stream) {
 		}
 		case -2:
 			debug_error("icq_handle_stream() DISCONNECT\n");
-			return -1;
+			return;
 
 		default:
 			debug_error("icq_handle_stream() icq_flap_loop() returns %d ???\n", ret);
 			break;
 	}
 
-	if (j->fd2 != -1) {	/* fd changed */
-		ekg_close(j->fd);
-		j->fd = j->fd2;
-		j->fd2 = -1;
-
-		if (s->connecting == 2) {
-			watch_add_session(s, j->fd, WATCH_WRITE, icq_handle_connect);
-		} else {
-			debug_error("unknown s->connecting: %d\n", s->connecting);
-		}
-		return -1;
-	}
-
-	return 0;
 }
 
-static WATCHER(icq_handle_hubresolver) {
-	session_t *s = session_find((char *) data);
-	icq_private_t *j;
+static void icq_handle_failure(GDataInputStream *f, GError *err, gpointer data) {
+	session_t *s = data;
 
-	struct sockaddr_in sin;
-	struct in_addr a;
-	int one = 1;
-	int hubport;
-	int res;
-
-	if (type) {
-		xfree(data);
-		close(fd);
-		return 0;
-	}
-
-	if (!s || !(j = s->priv))
-		return -1;
-
-	if (!s->connecting)	/* user makes /disconnect before resolver finished */
-		return -1;
-
-	res = read(fd, &a, sizeof(a));
-
-	if ((res != sizeof(a)) || (res && a.s_addr == INADDR_NONE /* INADDR_NONE kiedy NXDOMAIN */)) {
-		if (res == -1)
-			debug_error("[icq] unable to read data from resolver: %s\n", strerror(errno));
-		else
-			debug_error("[icq] read %d bytes from resolver. not good\n", res);
-
-		/* no point in reconnecting by icq_handle_disconnect() XXX? */
-
-		print("conn_failed", format_find("conn_failed_resolving"), session_name(s));
-		s->connecting = 0;
-		return -1;
-	}
-
-	debug_function("[icq] resolved to %s\n", inet_ntoa(a));
-
-/* port */
-	hubport = session_int_get(s, "hubport");
-	if (hubport < 1 || hubport > 65535)
-		hubport = ICQ_HUB_PORT;
-
-	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
-		debug("[icq] socket() failed: %s\n", strerror(errno));
-		icq_handle_disconnect(s, strerror(errno), EKG_DISCONNECT_FAILURE);
-		return -1;
-	}
-
-	sin.sin_family = AF_INET;
-	sin.sin_port = ntohs(hubport);
-	sin.sin_addr.s_addr = a.s_addr;
-
-	if (ioctl(fd, FIONBIO, &one) == -1)
-		debug_error("[icq] ioctl() FIONBIO failed: %s\n", strerror(errno));
-	if (setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, &one, sizeof(one)) == -1)
-		debug_error("[icq] setsockopt() SO_KEEPALIVE failed: %s\n", strerror(errno));
-
-	res = connect(fd, (struct sockaddr *) &sin, sizeof(struct sockaddr_in));
-
-	if (res == -1 && errno != EINPROGRESS) {
-		int err = errno;
-
-		debug_error("[icq] connect() failed: %s (errno=%d)\n", strerror(err), err);
-		icq_handle_disconnect(s, strerror(err), EKG_DISCONNECT_FAILURE);
-		return -1;
-	}
-
-	j->fd = fd;
-
-	watch_add_session(s, fd, WATCH_WRITE, icq_handle_connect);
-	return -1;
+	icq_handle_disconnect(s, err->message, EKG_DISCONNECT_NETWORK);
 }
 
+static void icq_handle_connect(
+		GSocketConnection *conn,
+		GInputStream *instream,
+		GOutputStream *outstream,
+		gpointer data)
+{
+	session_t *s = data;
+	icq_private_t *j = NULL;
+
+	if (!s || !(j = s->priv)) {
+		debug_error("icq_handle_connect() s: 0x%x j: 0x%x\n", s, j);
+		return;
+	}
+
+	debug_function("[icq] handle_connect(%d)\n", s->connecting);
+
+	g_string_set_size(j->stream_buf, 0);
+
+	j->send_stream = ekg_connection_add(
+			conn,
+			instream,
+			outstream,
+			icq_handle_stream,
+			icq_handle_failure,
+			s);
+}
+
+static void icq_handle_connect_failure(GError *err, gpointer data) {
+	session_t *s = data;
+
+	icq_handle_disconnect(s, err->message, EKG_DISCONNECT_FAILURE);
+}
+
+void icq_connect(session_t *session, const char *server, int port) {
+
+	GSocketClient *sock		= g_socket_client_new();
+	ekg_connection_starter_t cs	= ekg_connection_starter_new(port);
+	
+	ekg_connection_starter_set_servers(cs, server);
+
+	ekg_connection_starter_run(cs, sock, icq_handle_connect, icq_handle_connect_failure, session);
+}
+
+
 static COMMAND(icq_command_addssi) {
 	userlist_t *u;
 	icq_private_t *j;
 	private_data_t *refdata = NULL;
-	uint32_t uid;
-	uint16_t group, iid;
+	guint32 uid;
+	guint16 group, iid;
 	int i, ok=0;
 	char *nickname = NULL, *phone = NULL, *email = NULL, *comment = NULL;
 
@@ -858,7 +766,7 @@ static COMMAND(icq_command_addssi) {
 				continue;
 			}
 		}
-		printq("invalid_params", name);
+		printq("invalid_params", name, argv[i]);
 	}
 
 	if (nickname && (u = userlist_find(session, nickname))) {
@@ -869,10 +777,10 @@ static COMMAND(icq_command_addssi) {
 
 	if ((cmd_add && nickname) || (!cmd_add && ok)) {
 		/* send packets */
-		string_t buddies, data;
-		uint16_t min = 0xffff, max = 0, count = 0;
+		GString *buddies, *data;
+		guint16 min = 0xffff, max = 0, count = 0;
 
-		buddies = string_init(NULL);
+		buddies = g_string_new(NULL);
 		for (u = session->userlist; u; u = u->next) {
 			i = user_private_item_get_int(u, "iid");
 			icq_pack_append(buddies, "W", i);
@@ -906,7 +814,7 @@ static COMMAND(icq_command_addssi) {
 		 */
 		icq_send_snac(session, 0x13, 0x11, NULL, NULL, "");
 
-		data = string_init(NULL);
+		data = g_string_new(NULL);
 
 		/* TLV(0x0066) - Signifies that you are awaiting authorization for this buddy. The client is in
 		 * charge of putting this TLV, but you will not receiving status updates for the contact until
@@ -943,9 +851,9 @@ static COMMAND(icq_command_addssi) {
 				"U WWW WA",
 				target+4,				// item name (uin)
 				group,					// Group#
-				(uint16_t) iid,				// Item#
-				(uint16_t) 0,				// Type of item: 0 -- Buddy record
-				(uint16_t) data->len,			// Length of the additional data
+				(guint16) iid,				// Item#
+				(guint16) 0,				// Type of item: 0 -- Buddy record
+				(guint16) data->len,			// Length of the additional data
 				data
 				);
 		}
@@ -963,30 +871,30 @@ static COMMAND(icq_command_addssi) {
 				"U WWWW T",
 				j->default_group_name,			// default group name
 				group,					// Group#
-				(uint16_t) 0,				// Item#
-				(uint16_t) 1,				// Group record
-				(uint16_t) buddies->len + 4,		// Length of the additional data
+				(guint16) 0,				// Item#
+				(guint16) 1,				// Group record
+				(guint16) buddies->len + 4,		// Length of the additional data
 				icq_pack_tlv(0xc8, buddies->str, buddies->len)	// TLV(0xC8) contains the buddy ID#s of all buddies in the group
 				);
 		} else {
 			icq_send_snac(session, 0x13, 0x09, refdata, icq_cmd_addssi_ack,
 				"U WWW WA",
-/*XXX*/				itoa(uid),				// item name (uin)
+/*XXX*/				ekg_itoa(uid),				// item name (uin)
 				group,					// Group#
-				(uint16_t) iid,				// Item#
-				(uint16_t) 0,				// Type of item: 0 -- Buddy record
-				(uint16_t) data->len,			// Length of the additional data
+				(guint16) iid,				// Item#
+				(guint16) 0,				// Type of item: 0 -- Buddy record
+				(guint16) data->len,			// Length of the additional data
 				data
 				);
 		}
 
 		icq_send_snac(session, 0x13, 0x12, NULL, NULL, "");	// Contacts edit end (finish transaction)
 
-		string_free(data, 1);
-		string_free(buddies, 1);
+		g_string_free(data, TRUE);
+		g_string_free(buddies, TRUE);
 	}
 
-	array_free(argv);
+	g_strfreev(argv);
 
 	return 0;
 }
@@ -995,10 +903,10 @@ static COMMAND(icq_command_delssi) {
 	userlist_t *u;
 	icq_private_t *j;
 	private_data_t *refdata = NULL;
-	uint32_t u_id;
-	uint16_t iid = 0;
-	uint16_t group;
-	string_t buddies;
+	guint32 u_id;
+	guint16 iid = 0;
+	guint16 group;
+	GString *buddies;
 	int i;
 
 	if (params[0])
@@ -1033,14 +941,14 @@ static COMMAND(icq_command_delssi) {
 	 */
 	icq_send_snac(session, 0x13, 0x0A, refdata, icq_cmd_addssi_ack,
 			"U WWW W",
-/*XXX*/			itoa(u_id),				// item name (uin)
+/*XXX*/			ekg_itoa(u_id),				// item name (uin)
 			group,					// Group#
-			(uint16_t) iid,				// Item#
-			(uint16_t) 0,				// Type of item: 0 -- Buddy record
-			(uint16_t) 0				// Length of the additional data
+			(guint16) iid,				// Item#
+			(guint16) 0,				// Type of item: 0 -- Buddy record
+			(guint16) 0				// Length of the additional data
 			);
 
-	buddies = string_init(NULL);
+	buddies = g_string_new(NULL);
 	for (u = session->userlist; u; u = u->next) {
 		if (group == user_private_item_get_int(u, "gid")) {
 			i = user_private_item_get_int(u, "iid");
@@ -1053,13 +961,13 @@ static COMMAND(icq_command_delssi) {
 			"U WWWW T",
 			j->default_group_name,			// default group name
 			group,					// Group#
-			(uint16_t) 0,				// Item#
-			(uint16_t) 1,				// Group record
-			(uint16_t) buddies->len + 4,		// Length of the additional data
+			(guint16) 0,				// Item#
+			(guint16) 1,				// Group record
+			(guint16) buddies->len + 4,		// Length of the additional data
 			icq_pack_tlv(0xc8, buddies->str, buddies->len)	// TLV(0xC8) contains the buddy ID#s of all buddies in the group
 			);
 
-	string_free(buddies, 1);
+	g_string_free(buddies, TRUE);
 
 	icq_send_snac(session, 0x13, 0x12, NULL, NULL, "");	// Contacts edit end (finish transaction)
 
@@ -1068,10 +976,9 @@ static COMMAND(icq_command_delssi) {
 
 
 static void icq_send_msg_ch1(session_t *session, const char *uid, const char *message) {
-	string_t pkt;
-	string_t tlv_2, tlv_101;
+	GString *pkt, *tlv_2, *tlv_101;
 	userlist_t *u = userlist_find(session, uid);
-	uint16_t enc = 0;	/* ASCII */
+	guint16 enc = 0;	/* ASCII */
 	const char *tmp = message;
 
 	while (*tmp) {
@@ -1087,12 +994,12 @@ static void icq_send_msg_ch1(session_t *session, const char *uid, const char *me
 	tlv_101 = icq_pack("WW", enc, 0x00);		// encoding, codepage
 	if (enc == 2) {
 		/* send unicode message */
-		string_t recode = icq_convert_to_ucs2be((char *) message);
-		string_append_raw(tlv_101, recode->str, recode->len);
-		string_free(recode, 1);
+		GString *recode = icq_convert_to_ucs2be((char *) message);
+		g_string_append_len(tlv_101, recode->str, recode->len);
+		g_string_free(recode, TRUE);
 	} else {
 		/* send ASCII/ANSII message */
-		string_append(tlv_101, message);
+		g_string_append(tlv_101, message);
 	}
 
 	/* TLV(2) */
@@ -1100,17 +1007,17 @@ static void icq_send_msg_ch1(session_t *session, const char *uid, const char *me
 				icq_pack_tlv_char(0x501, 0x1),			/* TLV(501) features, meaning unknown, duplicated from ICQ Lite */
 				icq_pack_tlv(0x0101, tlv_101->str, tlv_101->len)/* TLV(101) text TLV. */
 			);
-	string_free(tlv_101, 1);
+	g_string_free(tlv_101, TRUE);
 
 	/* main packet */
-	pkt = icq_pack("iiWs", (uint32_t) rand(), (uint32_t) rand(), 1, uid+4);	// msgid1, msgid2, channel, recipient
+	pkt = icq_pack("iiWs", (guint32) rand(), (guint32) rand(), 1, uid+4);	// msgid1, msgid2, channel, recipient
 	icq_pack_append(pkt, "TTT",
 				icq_pack_tlv(0x02, tlv_2->str, tlv_2->len),	/* TLV(2) message-block */
 				icq_pack_tlv(0x03, NULL, 0),			/* TLV(3) server-ack */
 				icq_pack_tlv(0x06, NULL, 0)			/* TLV(6) received-offline */
 				);
 
-	string_free(tlv_2, 1);
+	g_string_free(tlv_2, TRUE);
 
 	/* message-header */
 	icq_makesnac(session, pkt, 0x04, 0x06, NULL, NULL);	// CLI_SEND_ICBM -- Send message thru server
@@ -1118,19 +1025,19 @@ static void icq_send_msg_ch1(session_t *session, const char *uid, const char *me
 }
 
 static void icq_send_msg_ch2(session_t *session, const char *uid, const char *message) {
-	string_t pkt, t5, t2711;
-	uint32_t msgid1 = rand(), msgid2 = rand();
+	GString *pkt, *t5, *t2711;
+	guint32 msgid1 = rand(), msgid2 = rand();
 	int prio = 1;
 	icq_private_t *j = session->priv;
 	int cookie = j->snac_seq++;
 
-	t5 = string_init(NULL);
+	t5 = g_string_new(NULL);
 	icq_pack_append(t5, "WII", 0, msgid1, msgid2);	//
 	icq_pack_append_cap(t5, CAP_SRV_RELAY);
 	icq_pack_append(t5, "tW", icq_pack_tlv_word(0x0a, 1));
 	icq_pack_append(t5, "T", icq_pack_tlv(0x0f, NULL, 0));		// empty TLV(0x0f)
 
-	t2711 = string_init(NULL);
+	t2711 = g_string_new(NULL);
 {
 		icq_pack_append_rendezvous(t2711, ICQ_VERSION, cookie, MTYPE_PLAIN, 0, 1, prio);
 		char *recode = ekg_locale_to_utf8_dup(message);
@@ -1138,10 +1045,10 @@ static void icq_send_msg_ch2(session_t *session, const char *uid, const char *me
 		xfree(recode);
 		icq_pack_append(t2711, "II", 0, 0xffffffff);	// XXX text & background colors
 		icq_pack_append(t2711, "i", xstrlen(CAP_UTF8_str));
-		string_append(t2711, CAP_UTF8_str);
+		g_string_append(t2711, CAP_UTF8_str);
 }
 	icq_pack_append(t5, "T", icq_pack_tlv(0x2711, t2711->str, t2711->len));
-	string_free(t2711, 1);
+	g_string_free(t2711, TRUE);
 
 
 	/* main packet */
@@ -1154,7 +1061,7 @@ static void icq_send_msg_ch2(session_t *session, const char *uid, const char *me
 }
 
 static COMMAND(icq_command_msg) {
-	uint32_t uin;
+	guint32 uin;
 	char *uid;
 	userlist_t *u;
 
@@ -1181,7 +1088,7 @@ static COMMAND(icq_command_msg) {
 		// send "typing finished" snac
 		const char *sid	= session_uid_get(session);
 		int first = 0, len = 0;
-		query_emit_id(NULL, PROTOCOL_TYPING_OUT, &sid, &uid, &len, &first);
+		query_emit(NULL, "protocol-typing-out", &sid, &uid, &len, &first);
 	}
 
 	u = userlist_find(session, uid);
@@ -1203,7 +1110,7 @@ msgdisplay:
 
 		protocol_message_emit(session, session->uid, rcpts, params[1], NULL, time(NULL), class, NULL, EKG_NO_BEEP, 0);
 
-		array_free(rcpts);
+		g_strfreev(rcpts);
 
 		/* XXX, it's copied from jabber-plugin, however i think we should _always_ add message to queue (if we're offline), even if we want do it quiet */
 		if (!session->connected)
@@ -1298,6 +1205,7 @@ static COMMAND(icq_command_away) {
 	return 0;
 }
 
+
 static COMMAND(icq_command_connect) {
 	icq_private_t *j = session->priv;
 	const char *hubserver;
@@ -1327,13 +1235,10 @@ static COMMAND(icq_command_connect) {
 	j->ssi = 1;	/* XXX */
 	j->aim = 1;	/* XXX */
 
-	if (ekg_resolver2(&icq_plugin, hubserver, icq_handle_hubresolver, xstrdup(session->uid)) == NULL) {
-		print("generic_error", strerror(errno));
-		session->connecting = 0;
-		return -1;
-	}
-
 	printq("connecting", session_name(session));
+
+	icq_connect(session, hubserver, ICQ_HUB_PORT);
+
 	if ((session_status_get(session) == EKG_STATUS_NA))
 		session_status_set(session, EKG_STATUS_AVAIL);
 
@@ -1367,8 +1272,8 @@ static COMMAND(icq_command_reconnect) {
 }
 
 static COMMAND(icq_command_userinfo) {
-	string_t pkt;
-	uint32_t number;
+	GString *pkt;
+	guint32 number;
 	int minimal_req = 0;	/* XXX */
 	private_data_t *ref_data = NULL;
 
@@ -1401,8 +1306,8 @@ static COMMAND(icq_command_userinfo) {
 }
 
 static COMMAND(icq_command_searchuin) {
-	string_t pkt;
-	uint32_t uin;
+	GString *pkt;
+	guint32 uin;
 
 	debug_function("icq_command_searchuin() %s\n", params[0]);
 
@@ -1436,7 +1341,7 @@ static COMMAND(icq_command_search) {
 	char **argv;
 	int i;
 
-	string_t pkt;
+	GString *pkt;
 
 	argv = array_make(params[0], " \t", 0, 1, 1);
 
@@ -1482,21 +1387,21 @@ static COMMAND(icq_command_search) {
 		}
 
 		/* XXX, madrzej? zgadywanie? */
-		printq("invalid_params", name);
-		array_free(argv);
+		printq("invalid_params", name, argv[i]);
+		g_strfreev(argv);
 		return -1;
 	}
 
 	/* XXX, cookie */
 
 	/* Pack the search details */
-	pkt = string_init(NULL);
+	pkt = g_string_new(NULL);
 
 #define wo_idnhtni(type, str) \
 	{ \
-		uint32_t len = xstrlen(str); \
-		icq_pack_append(pkt, "www", (uint32_t) type, len+3, len+1); \
-		string_append_raw(pkt, (char *) str, len+1); \
+		guint32 len = xstrlen(str); \
+		icq_pack_append(pkt, "www", (guint32) type, len+3, len+1); \
+		g_string_append_len(pkt, (char *) str, len+1); \
 	}
 
 	if (first_name) wo_idnhtni(0x0140, first_name);	/* TLV_FIRSTNAME */
@@ -1542,13 +1447,13 @@ static COMMAND(icq_command_search) {
 	icq_makemetasnac(session, pkt, CLI_META_INFO_REQ, CLI_WHITE_PAGES_SEARCH2, NULL, 0);
 	icq_send_pkt(session, pkt);
 
-	array_free(argv);
+	g_strfreev(argv);
 
 	return 0;
 }
 
 static COMMAND(icq_command_auth) {
-	uint32_t number;
+	guint32 number;
 	const char *reason = NULL;
 
 	if (match_arg(params[0], 'l', "list", 2)) {
@@ -1565,7 +1470,7 @@ static COMMAND(icq_command_auth) {
 		target = params[1];
 		reason = params[2];
 	} else if (!target) {
-		printq("invalid_params", name);
+		printq("not_enough_params", name);
 		return -1;
 	}
 
@@ -1584,7 +1489,7 @@ static COMMAND(icq_command_auth) {
 			reason = "Please add me.";
 
 		icq_send_snac(session, 0x13, 0x18, 0, 0,
-				"uUW", number, reason, (uint32_t) 0x00);
+				"uUW", number, reason, (guint32) 0x00);
 
 		return 0;
 	}
@@ -1600,11 +1505,11 @@ static COMMAND(icq_command_auth) {
 		int auth = (match_arg(params[0], 'a', "accept", 2) != 0);
 
 		icq_send_snac(session, 0x13, 0x1a, 0, 0,
-				"ucUW", number, (uint32_t) auth, reason ? reason : "", (uint32_t) 0x00);
+				"ucUW", number, (guint32) auth, reason ? reason : "", (guint32) 0x00);
 		return 0;
 	}
 
-	printq("invalid_params", name);
+	printq("invalid_params", name, params[0]);
 	return -1;
 }
 
@@ -1616,14 +1521,14 @@ static COMMAND(icq_command_rates) {
 		if (!i)
 			print("icq_rates_header");
 		printq("icq_rates",
-			itoa(i+1),
-			itoa(j->rates[i]->win_size),
-			itoa(j->rates[i]->clear_lvl),
-			itoa(j->rates[i]->alert_lvl),
-			itoa(j->rates[i]->limit_lvl),
-			itoa(j->rates[i]->discn_lvl),
-			itoa(j->rates[i]->curr_lvl),
-			itoa(j->rates[i]->max_lvl));
+			ekg_itoa(i+1),
+			ekg_itoa(j->rates[i]->win_size),
+			ekg_itoa(j->rates[i]->clear_lvl),
+			ekg_itoa(j->rates[i]->alert_lvl),
+			ekg_itoa(j->rates[i]->limit_lvl),
+			ekg_itoa(j->rates[i]->discn_lvl),
+			ekg_itoa(j->rates[i]->curr_lvl),
+			ekg_itoa(j->rates[i]->max_lvl));
 	}
 
 	return 0;
@@ -1774,12 +1679,14 @@ EXPORT int icq_plugin_init(int prio) {
 
 	plugin_register(&icq_plugin, prio);
 
-	query_connect_id(&icq_plugin, PROTOCOL_VALIDATE_UID, icq_validate_uid, NULL);
-	query_connect_id(&icq_plugin, PLUGIN_PRINT_VERSION, icq_print_version, NULL);
-	query_connect_id(&icq_plugin, SESSION_ADDED, icq_session_init, NULL);
-	query_connect_id(&icq_plugin, SESSION_REMOVED, icq_session_deinit, NULL);
-	query_connect_id(&icq_plugin, USERLIST_INFO, icq_userlist_info_handle, NULL);
-	query_connect_id(&icq_plugin, PROTOCOL_TYPING_OUT, icq_typing_out, NULL);
+	query_connect(&icq_plugin, "protocol-validate-uid", icq_validate_uid, NULL);
+	query_connect(&icq_plugin, "plugin-print-version", icq_print_version, NULL);
+	query_connect(&icq_plugin, "session-added", icq_session_init, NULL);
+	query_connect(&icq_plugin, "session-removed", icq_session_deinit, NULL);
+	query_connect(&icq_plugin, "userlist-info", icq_userlist_info_handle, NULL);
+	query_connect(&icq_plugin, "protocol-typing-out", icq_typing_out, NULL);
+
+	variable_add(&icq_plugin, ("disable_chatstates"), VAR_BOOL, 1, &icq_config_disable_chatstates, NULL, NULL, NULL);
 
 	command_add(&icq_plugin, "icq:", "?", icq_command_inline_msg, ICQ_ONLY | COMMAND_PASS_UNCHANGED, NULL);
 	command_add(&icq_plugin, "icq:msg", "!uU !", icq_command_msg, ICQ_FLAGS_MSG, NULL);
