@@ -15,20 +15,12 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include "ekg2.h"
+
 #include <stdio.h>
 #include <time.h>
 #include <sys/time.h>
 
-#define __EKG_STUFF_H
-
-#include <ekg/debug.h>
-#include <ekg/dynstuff.h>
-#include <ekg/sessions.h>
-#include <ekg/userlist.h>
-#include <ekg/xmalloc.h>
-
-#include <ekg/queries.h>
-
 #include "people.h"
 #include "irc.h"
 
@@ -67,7 +59,7 @@ int irc_xstrcasecmp_default(char *str1, char *str2)
  *     priv_data->channels 
  *     priv_data->people->channels->onchan
  */
-people_t *irc_find_person(list_t p, char *nick)
+people_t *irc_find_person(irc_private_t *j, list_t p, char *nick)
 {
 	int (*comp_func)(char *,char*);
 	people_t *person;
@@ -78,7 +70,7 @@ people_t *irc_find_person(list_t p, char *nick)
 	if (!xstrncmp(nick, IRC4, 4))
 		debug_error("programmer's mistake in call to irc_find_person!: %s\n", nick);
 
-	if (*nick == '+' || *nick == '%' || *nick == '@') nick++;
+	if (xstrchr(j->nick_signs, *nick)) nick++;
 
 	comp_func = irc_xstrcasecmp_default;
 
@@ -143,8 +135,9 @@ static void update_longest_nick(channel_t *chan)
 	for (p=chan->onchan; p; p=p->next)
 	{
 		people_t *person = (people_t *)p->data;
-		if (person->nick && xstrlen(person->nick+4) > chan->longest_nick)
-			chan->longest_nick = xstrlen(person->nick+4);
+		const gsize nicklen = g_utf8_strlen(person->nick+4, -1);
+		if (person->nick && nicklen > chan->longest_nick)
+			chan->longest_nick = nicklen;
 	}
 	nickpad_string_create(chan);
 }
@@ -172,13 +165,11 @@ static people_t *irc_add_person_int(session_t *s, irc_private_t *j,
 	people_chan_t *pch_tmp;
 	userlist_t *ulist;
 	window_t *w;
-	int k, mode = 0, irccol = 0;
-	char *ircnick, *modes, *t;
+	int mode = 0, irccol = 0;
+	char *ircnick, *t;
 
-	k = (xstrlen(SOP(_005_PREFIX))>>1);
-	modes = SOP(_005_PREFIX) + k + 1;
-	if ((t = xstrchr(modes, *nick)))
-		mode = 1<<(k-(t-modes)-2);
+	if ((t = xstrchr(j->nick_signs, *nick)))
+		mode = 1 << (t - j->nick_signs);
 
 	/* debug("irc_add_person_int: %s %d %d\n", modes, mode, k); */
 	if (mode) nick++;
@@ -195,7 +186,7 @@ static people_t *irc_add_person_int(session_t *s, irc_private_t *j,
 
 	/* add entry in priv_data->people if nick's not yet there */
 	/* ok new irc-find-person checked */
-	if (!(person = irc_find_person(j->people, nick))) {
+	if (!(person = irc_find_person(j, j->people, nick))) {
 	/*	debug("+%s lista ludzi, ", nick); */
 		person = xmalloc(sizeof(people_t));
 		person->nick = xstrdup(ircnick);
@@ -203,7 +194,7 @@ static people_t *irc_add_person_int(session_t *s, irc_private_t *j,
 		list_add(&(j->people), person);
 	}
 	/* add entry in priv_data->channels->onchan if nick's not yet there */
-	if (!(peronchan = irc_find_person(chan->onchan, nick)))  {
+	if (!(peronchan = irc_find_person(j, chan->onchan, nick)))  {
 	/*	debug("+do kanau, "); */
 		list_add(&(chan->onchan), person);
 	}
@@ -248,7 +239,7 @@ people_t *irc_add_person(session_t *s, irc_private_t *j,
 		chan->longest_nick = xstrlen(nick);
 		nickpad_string_create(chan);
 	}
-	query_emit_id(NULL, USERLIST_REFRESH);
+	query_emit(NULL, "userlist-refresh");
 	return ret;
 }
 
@@ -277,7 +268,7 @@ int irc_add_people(session_t *s, irc_private_t *j, char *names, char *channame)
 
 		return 0;
 	}
-	debug("[irc] add_people() %08X\n", j);
+	debug_function("[irc] add_people() %08X\n", j);
 	save = nick = array_make(names, " ", 0, 1, 0);
 	while (*nick) {
 		irc_add_person_int(s, j, *nick, chan);
@@ -292,9 +283,9 @@ int irc_add_people(session_t *s, irc_private_t *j, char *names, char *channame)
 	}
 	nickpad_string_create(chan);
 
-	query_emit_id(NULL, USERLIST_REFRESH);
+	query_emit(NULL, "userlist-refresh");
 
-	array_free(save);
+	g_strfreev(save);
 	return 0;	
 }
 
@@ -361,7 +352,7 @@ int irc_del_person_channel(session_t *s, irc_private_t *j, char *nick, char *cha
 
 	if (!(chan = irc_find_channel(j->channels, channame)))
 		return -1;
-	if (!(person = irc_find_person(j->people, nick)))
+	if (!(person = irc_find_person(j, j->people, nick)))
 		return -1;
 
 	ret = irc_del_person_channel_int(s, j, person, chan);
@@ -369,7 +360,7 @@ int irc_del_person_channel(session_t *s, irc_private_t *j, char *nick, char *cha
 	if (xstrlen(nick) == chan->longest_nick)
 		update_longest_nick(chan);
 
-	query_emit_id(NULL, USERLIST_REFRESH);
+	query_emit(NULL, "userlist-refresh");
 	return ret;
 }
 
@@ -398,7 +389,7 @@ int irc_del_person(session_t *s, irc_private_t *j, char *nick,
 	int ret;
 	char *longnick;
 
-	if (!(person = irc_find_person(j->people, nick))) 
+	if (!(person = irc_find_person(j, j->people, nick))) 
 		return -1;
 
 	/* if person doesn't have any channels, we shouldn't get here
@@ -453,7 +444,7 @@ int irc_del_person(session_t *s, irc_private_t *j, char *nick,
 	}
 	xfree(longnick);
 
-	query_emit_id(NULL, USERLIST_REFRESH);
+	query_emit(NULL, "userlist-refresh");
 	return 0;
 }
 
@@ -467,7 +458,7 @@ int irc_del_channel(session_t *s, irc_private_t *j, char *name)
 	if (!(chan = irc_find_channel((j->channels), name)))
 		return -1;
 
-	debug("[irc]_del_channel() %s\n", name);
+	debug_function("[irc]_del_channel() %s\n", name);
 	while ((p = (chan->onchan)))
 		if (!(p->data)) break;
 		else irc_del_person_channel_int(s, j, (people_t *)p->data, chan);
@@ -489,7 +480,7 @@ int irc_del_channel(session_t *s, irc_private_t *j, char *name)
 	}
 	xfree(tmp);
 
-	query_emit_id(NULL, USERLIST_REFRESH);
+	query_emit(NULL, "userlist-refresh");
 	return 0;
 }
 
@@ -499,9 +490,10 @@ static int irc_sync_channel(session_t *s, irc_private_t *j, channel_t *p)
 	p->syncmode = 2;
 	/* to ma sie rownac ile ma byc roznych syncow narazie tylko WHO
 	 * ale moze bedziemy syncowac /mode +b, +e, +I) */
-	gettimeofday(&(p->syncstart), NULL);
-	watch_write(j->send_watch, "WHO %s\r\n", p->name+4);
-	watch_write(j->send_watch, "MODE %s +b\r\n", p->name+4);
+	g_get_current_time(&(p->syncstart));
+	ekg_connection_write(j->send_stream, "WHO %s\r\n", p->name+4);
+	ekg_connection_write(j->send_stream, "MODE %s +b\r\n", p->name+4);
+	ekg_connection_write(j->send_stream, "MODE %s\r\n", p->name+4);
 	return 0;
 }
 
@@ -530,9 +522,9 @@ int irc_color_in_contacts(irc_private_t *j, int mode, userlist_t *ul)
 
 	/* GiM: this could be done much easier on intel ;/ */
 	for (i=0; i<len; i++)
-		if (mode & (1<<(len-1-i))) break;
+		if (mode & (1<<i)) break;
 	
-	switch (SOP(_005_PREFIX)[i+1]) {
+	switch (j->nick_modes[i]) {
 		case 'o':	ul->status = EKG_STATUS_AVAIL;		break;	/* op */
 		case 'h':	ul->status = EKG_STATUS_AWAY;		break;	/* half-op */
 		case 'v':	ul->status = EKG_STATUS_XA;		break;	/* voice */
@@ -545,15 +537,11 @@ int irc_color_in_contacts(irc_private_t *j, int mode, userlist_t *ul)
 
 int irc_nick_prefix(irc_private_t *j, people_chan_t *ch, int irc_color)
 {
-	char *t = SOP(_005_PREFIX);
-	char *p = xstrchr(t, ')');
-	*(ch->sign)=' ';
-	(ch->sign)[1] = '\0';
-	if (p) {
-		p++;
-		if (irc_color < xstrlen(p))
-			*(ch->sign) = p[irc_color];
-	} 
+	ch->sign[0] = ' ';
+	ch->sign[1] = '\0';
+	if (irc_color < xstrlen(j->nick_signs))
+		*(ch->sign) = j->nick_signs[irc_color];
+
 	return 0;
 }
 		
@@ -580,7 +568,7 @@ int irc_nick_change(session_t *s, irc_private_t *j, char *old_nick, char *new_ni
 	window_t *w;
 	char *t1, *t2 = irc_uid(new_nick);
 
-	if (!(per = irc_find_person(j->people, old_nick))) {
+	if (!(per = irc_find_person(j, j->people, old_nick))) {
 		debug_error("irc_nick_change() person not found?\n");
 		xfree(t2);
 		return 0;
@@ -619,7 +607,7 @@ int irc_nick_change(session_t *s, irc_private_t *j, char *old_nick, char *new_ni
 			 * this way */
 		}
 	}
-	query_emit_id(NULL, USERLIST_REFRESH);
+	query_emit(NULL, "userlist-refresh");
 
 	/* update nickname in internal structures */
 	t1 = per->nick;
@@ -651,7 +639,7 @@ int irc_free_people(session_t *s, irc_private_t *j)
 	channel_t *chan;
 	window_t *w;
 
-	debug("[irc] free_people() %08X %s\n", s, s->uid);
+	debug_function("[irc] free_people() %08X %s\n", s, s->uid);
 	for (t1=j->people; t1; t1=t1->next) {
 		per = (people_t *)t1->data;
 		list_destroy(per->channels, 1);
