@@ -18,9 +18,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#define _XOPEN_SOURCE 600
-#include "ekg2-config.h"
-#include <ekg/win32.h>
+#include "ekg2.h"
 
 #include <sys/types.h>
 
@@ -52,20 +50,6 @@
 #endif
 #include <time.h>
 
-#include <ekg/debug.h>
-#include <ekg/dynstuff.h>
-#include <ekg/plugins.h>
-#include <ekg/protocol.h>
-#include <ekg/sessions.h>
-#include <ekg/stuff.h>
-#include <ekg/userlist.h>
-#include <ekg/themes.h>
-#include <ekg/vars.h>
-#include <ekg/xmalloc.h>
-#include <ekg/log.h>
-
-#include <ekg/queries.h>
-
 #include "jabber.h"
 #include "jabber_dcc.h"
 
@@ -314,7 +298,7 @@ JABBER_HANDLER(jabber_handle_stream_features) {
 			for (method = n->children; method; method = method->next) {
 				if (!xstrcmp(method->name, "method")) {
 					if (!xstrcmp(method->data, "zlib")) {
-#ifdef HAVE_ZLIB
+#ifdef HAVE_LIBZ
 						if ((tmp2 = xstrstr(tmp, "zlib")) && ((tmp2 < method_comp) || (!method_comp)) && 
 								(tmp2[4] == ',' || tmp2[4] == '\0')) {
 							method_comp = tmp2;	 /* found more preferable method */
@@ -498,7 +482,7 @@ JABBER_HANDLER(jabber_handle_challenge) {
 
 	/* decode && parse challenge data */
 	data = base64_decode(n->data);
-	debug_error("[jabber] PARSING challange (%s): \n", data);
+	debug_error("[jabber] PARSING challenge (%s): \n", data);
 	arr = array_make(data, "=,", 0, 1, 1);	/* maybe we need to change/create another one parser... i'm not sure. please notify me, 
 						   I'm lazy, sorry */
 	/* for chrome.pl and jabber.autocom.pl it works */
@@ -510,7 +494,7 @@ JABBER_HANDLER(jabber_handle_challenge) {
 		debug_error("[%d] %s: %s\n", i / 2, arr[i], __(arr[i+1]));
 		if (!arr[i+1]) {
 			debug_error("Parsing var<=>value failed, NULL....\n");
-			array_free(arr);
+			g_strfreev(arr);
 			j->parser = NULL; 
 			jabber_handle_disconnect(s, "IE, Current SASL support for ekg2 cannot handle with this data, sorry.", EKG_DISCONNECT_FAILURE);
 			return;
@@ -559,12 +543,12 @@ JABBER_HANDLER(jabber_handle_challenge) {
 		cnonce = base64_encode(tmp_cnonce, sizeof(tmp_cnonce));
 
 		xmpp_temp	= saprintf(":xmpp/%s", realm);
-		auth_resp	= jabber_challange_digest(username, password, nonce, cnonce, xmpp_temp, realm);
+		auth_resp	= jabber_challenge_digest(username, password, nonce, cnonce, xmpp_temp, realm);
 		session_set(s, "__sasl_excepted", auth_resp);
 		xfree(xmpp_temp);
 
 		xmpp_temp	= saprintf("AUTHENTICATE:xmpp/%s", realm);
-		auth_resp	= jabber_challange_digest(username, password, nonce, cnonce, xmpp_temp, realm);
+		auth_resp	= jabber_challenge_digest(username, password, nonce, cnonce, xmpp_temp, realm);
 		xfree(xmpp_temp);
 
 		string_append(str, "username=\"");	string_append(str, username);	string_append_c(str, '\"');
@@ -585,7 +569,7 @@ JABBER_HANDLER(jabber_handle_challenge) {
 		xfree(username);
 		xfree(cnonce);
 	}
-	array_free(arr);
+	g_strfreev(arr);
 }
 
 JABBER_HANDLER(jabber_handle_proceed) {
@@ -849,7 +833,7 @@ JABBER_HANDLER(jabber_handle_message) {
 					char buf[100];
 					string_append(body, "Sent: ");
 					if (!strftime(buf, sizeof(buf), nazwa_zmiennej_do_formatowania_czasu, tm) 
-						string_append(body, itoa(bsent);	/* if too long display seconds since the Epoch */
+						string_append(body, ekg_itoa(bsent);	/* if too long display seconds since the Epoch */
 					else	string_append(body, buf);	/* otherwise display formatted time */
 					new_line = 1;
 				}
@@ -975,7 +959,7 @@ JABBER_HANDLER(jabber_handle_message) {
 		int secure	= (x_encrypted != NULL);
 		time_t sent	= bsent;
 		char *text	= tlenjabber_unescape(body->str);
-		uint32_t *format= jabber_msg_format(text, nhtml);
+		guint32 *format= jabber_msg_format(text, nhtml);
 
 		if (!sent) sent = time(NULL);
 
@@ -1219,7 +1203,7 @@ static void jabber_handle_xmldata_result(session_t *s, xmlnode_t *form, const ch
 		} else debug_error("jabber_handle_xmldata_result() name: %s\n", form->name);
 	}
 	if (print_end) print("jabber_form_end", session_name(s), uid, "");
-	array_free(labels);
+	g_strfreev(labels);
 }
 
 struct jabber_iq_generic_handler {
@@ -1399,6 +1383,17 @@ static inline int jabber_status_int(int tlen, const char *text) {
 	return ekg_status_int(text);
 }
 
+static status_t role_and_affiliation_to_ekg2_status(char *role, char * affiliation) {
+	if(!xstrcmp(role, "moderator"))
+		return xstrcmp(affiliation, "owner") ? EKG_STATUS_AVAIL : EKG_STATUS_FFC;
+	else if (!xstrcmp(role,"participant"))
+		return EKG_STATUS_AWAY;
+	else if (!xstrcmp(role, "visitor"))
+		return EKG_STATUS_XA;
+	else /* none */
+		return EKG_STATUS_NA;
+}
+									    
 JABBER_HANDLER(jabber_handle_presence) {
 	jabber_private_t *j = s->priv;
 
@@ -1532,14 +1527,14 @@ JABBER_HANDLER(jabber_handle_presence) {
 
 						if (ulist) {
 							jabber_userlist_private_t *up = jabber_userlist_priv_get(ulist);
-							ulist->status = EKG_STATUS_AVAIL;
+							ulist->status = role_and_affiliation_to_ekg2_status(role, affiliation);
 							
 							if (up) {
 								up->role	= xstrdup(role);
 								up->aff		= xstrdup(affiliation);
 							}
 						}
-						query_emit_id(NULL, USERLIST_REFRESH);
+						query_emit(NULL, "userlist-refresh");
 						debug("[MUC, PRESENCE] NEWITEM: %s (%s) ROLE:%s AFF:%s\n", nickjid, __(jid), role, affiliation);
 						xfree(nickjid);
 						xfree(jid); xfree(role); xfree(affiliation);
@@ -1697,7 +1692,7 @@ static void newmail_common(session_t *s) { /* maybe inline? */
 	if (config_sound_mail_file) 
 		play_sound(config_sound_mail_file);
 	else if (config_jabber_beep_mail)
-		query_emit_id(NULL, UI_BEEP, NULL);
+		query_emit(NULL, "ui-beep", NULL);
 	/* XXX, we NEED to connect to MAIL_COUNT && display info about mail like mail plugin do. */
 	/* XXX, emit events */
 }
