@@ -8,6 +8,7 @@ extern int config_contacts_descr;
 extern int config_contacts_edge;
 extern int config_contacts_frame;
 extern int config_contacts_margin;
+extern int config_contacts_vertical_margin;
 extern int config_contacts_orderbystate;
 extern int config_contacts_wrap;
 extern char *config_contacts_order;
@@ -18,7 +19,7 @@ extern int contacts_group_index;
 
 int ncurses_contacts_update(window_t *w, int save_pos);
 void ncurses_contacts_changed(const char *name);
-void ncurses_contacts_new(window_t *w);
+void ncurses_contacts_set(window_t *w);
 
 #endif /* __EKG_NCURSES_CONTACTS_H */
 
