@@ -3,6 +3,7 @@
 /*
  *  (C) Copyright 2003 Wojtek Kaniewski <wojtekka@irc.pl>
  *		  2004 Piotr Kupisiewicz <deletek@ekg2.org>
+ *		  2010 Sawomir Nizio <poczta-sn@gazeta.pl>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -18,59 +19,56 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "ekg2-config.h"
+#include "ekg2.h"
 
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
 #include <unistd.h>
 
-#include <ekg/debug.h>
-#include <ekg/plugins.h>
-#include <ekg/windows.h>
-#include <ekg/vars.h>
-#include <ekg/stuff.h>
-#include <ekg/themes.h>
-#include <ekg/xmalloc.h>
-
-#include <ekg/queries.h>
-
-#include "ecurses.h"
+#include "backlog.h"
 #include "bindings.h"
-#include "old.h"
 #include "contacts.h"
+#include "input.h"
+#include "lastlog.h"
 #include "mouse.h"
 #include "notify.h"
+#include "nc-stuff.h"
+#include "spell.h"
+#include "statusbar.h"
 
 static int ncurses_theme_init();
 PLUGIN_DEFINE(ncurses, PLUGIN_UI, ncurses_theme_init);
 /* vars */
-int config_aspell;
-char *config_aspell_lang;
 int config_backlog_size;
+int config_backlog_scroll_half_page;
 int config_display_transparent;
 int config_enter_scrolls;
-int config_header_size;
 int config_margin_size;
-int config_mark_on_window_change = 0;
-int config_kill_irc_window = 1;
-int config_statusbar_size;
+int config_mark_on_window_change	= 0;
+int config_kill_irc_window		= 1;
 int config_lastlog_size;
 int config_lastlog_lock;
-int config_text_bottomalign	= 0;
-int config_typing_interval	= 1;
-int config_typing_timeout	= 10;
-int config_typing_timeout_empty = 5;
+int config_text_bottomalign		= 0;
 
 static int config_traditional_clear	= 1;
 
+static int redraw_timer_id = 0;
+
 int ncurses_initialized;
 int ncurses_plugin_destroyed;
 
 const char *ncurses_settitle_formats[3] = { NULL, "\e]0;%s%s%s\a", "\e_%s%s%s\e\\" };
 static int ncurses_settitle = 0;
 
-QUERY(ncurses_password_input); /* old.c */
+QUERY(ncurses_password_input); /* nc-stuff.c */
+
+	/* XXX: any need for random arguments? */
+static void ncurses_set_title(const gchar* a, const gchar* b, const gchar* c) {
+		/* XXX: recode? */
+	if (ncurses_settitle)
+		printf(ncurses_settitle_formats[ncurses_settitle], a, b, c);
+}
 
 /**
  * ncurses_beep()
@@ -158,7 +156,7 @@ static QUERY(ncurses_ui_window_switch) {
 	if (config_mark_on_window_change)
 		command_exec(NULL, NULL, "/mark -1", 1);
 
-	if ((wc = window_find_sa(NULL, "__contacts", 1))) {
+	if ((wc = window_exist(WINDOW_CONTACTS_ID))) {
 		/* XXX, na pewno nie chcemy zapisywac polozenia userlisty podczas zmiany okna? */
 		ncurses_contacts_update(wc, 0);
 	}
@@ -172,32 +170,40 @@ static QUERY(ncurses_ui_window_switch) {
 	ncurses_redraw_input(0);	/* redraw prompt... */
 	ncurses_commit();
 
-	if (w->act & 2) { /* enable <composing/> on incoming chat message receival */
-		w->in_active = 1;
-		if (!w->out_active) /* send <active/>, as we showed interest in chat */
-			ncurses_window_gone(w);
-	}
+	ncurses_typingsend(w, EKG_CHATSTATE_ACTIVE);
 
 	p = w->alias ? w->alias : (w->target ? w->target : NULL);
-	if (ncurses_settitle)
-		printf(ncurses_settitle_formats[ncurses_settitle], p ? p : "", p ? " - " : "", "EKG2");
+	ncurses_set_title(p ? p : "", p ? " - " : "", "EKG2");
 
 	return 0;
 }
 
+static int redraw_timer(gpointer data) {
+	window_t *w = (window_t *)data;
+
+	if (window_find_ptr(w)) {
+		ncurses_redraw(w);
+		if (w->lock == 0)
+			ncurses_commit();
+	}
+
+	redraw_timer_id = 0;
+	return FALSE;	// done. destroy timer
+}
+
 static QUERY(ncurses_ui_window_print)
 {
 	window_t *w	= *(va_arg(ap, window_t **));
-	fstring_t *line = *(va_arg(ap, fstring_t **));
+	const fstring_t *line = *(va_arg(ap, const fstring_t **));
 
 	ncurses_window_t *n;
 	int bottom = 0, prev_count, count = 0;
 
-	if (!(n = w->priv_data)) { 
+	if (!(n = w->priv_data)) {
 		/* BUGFIX, cause @ ui-window-print handler (not ncurses plugin one, ncurses plugin one is called last cause of 0 prio)
-		 *	plugin may call print_window() 
+		 *	plugin may call print_window()
 		 */
-		ncurses_window_new(w);	
+		ncurses_window_new(w);
 		if (!(n = w->priv_data)) {
 			debug("ncurses_ui_window_print() IInd CC still not w->priv_data, quitting...\n");
 			return -1;
@@ -208,7 +214,7 @@ static QUERY(ncurses_ui_window_print)
 
 	if (n->start == n->lines_count - w->height || (n->start == 0 && n->lines_count <= w->height))
 		bottom = 1;
-	
+
 	count = ncurses_backlog_add(w, line);
 
 	if (n->overflow) {
@@ -234,11 +240,17 @@ static QUERY(ncurses_ui_window_print)
 		w->more = 1;
 
 	if (!w->floating) {
-		ncurses_redraw(w);
-		if (w->lock == 0) // && w == window_current) it should be tested
-			ncurses_commit();
+		n->redraw = 1;
+		if (window_current && window_current->id == w->id && !w->more) {
+			/* redraw only visible window and lines,
+			 * but wait 5ms for next lines
+			 */
+			if (redraw_timer_id>0)
+				g_source_remove(redraw_timer_id);
+			redraw_timer_id = g_timeout_add(5/*ms*/, redraw_timer, w);
+		}
 	}
-	
+
 	return 0;
 }
 
@@ -270,18 +282,8 @@ static QUERY(ncurses_ui_window_act_changed)
 static QUERY(ncurses_ui_window_target_changed)
 {
 	window_t *w = *(va_arg(ap, window_t **));
-	ncurses_window_t *n = w->priv_data;
-	char *tmp, *p;
-
-	xfree(n->prompt);
-
-	p = w->alias ? w->alias : (w->target ? w->target : NULL);
-	tmp = format_string(format_find((p) ? "ncurses_prompt_query" : "ncurses_prompt_none"), p);
-	n->prompt = tmp; 
-	n->prompt_len = xstrlen(tmp);
-
-	ncurses_update_real_prompt(n);
 
+	ncurses_prompt_set(w, w->alias ? w->alias : w->target);
 	update_statusbar(1);
 
 	return 0;
@@ -322,7 +324,7 @@ static QUERY(ncurses_ui_window_clear)
  * ncurses_all_contacts_changed()
  *
  * wywoywane przy zmianach userlisty powodujcych konieczno
- * podkasowania sorted_all_cache (zmiany w metakontaktach 
+ * podkasowania sorted_all_cache (zmiany w metakontaktach
  * i ncurses:contacts_metacontacts_swallow)
  */
 
@@ -333,7 +335,7 @@ static QUERY(ncurses_all_contacts_changed)
 
 /*	ncurses_contacts_changed(data); */
 
-	if ((w = window_find_sa(NULL, "__contacts", 1))) {
+	if ((w = window_exist(WINDOW_CONTACTS_ID))) {
 		ncurses_contacts_update(w, !data);
 		ncurses_commit();
 	}
@@ -378,15 +380,10 @@ static QUERY(ncurses_conference_renamed)
 	window_t *w;
 
 	for (w = windows; w; w = w->next) {
-		ncurses_window_t *n = w->priv_data;
-
 		if (w->target && !xstrcasecmp(w->target, oldname)) {
 			xfree(w->target);
-			xfree(n->prompt);
 			w->target = xstrdup(newname);
-			n->prompt = format_string(format_find("ncurses_prompt_query"), newname);
-			n->prompt_len = xstrlen(n->prompt);
-			ncurses_update_real_prompt(n);
+			ncurses_prompt_set(w, newname);
 		}
 	}
 
@@ -403,7 +400,7 @@ static QUERY(ncurses_conference_renamed)
  */
 static void ncurses_changed_aspell(const char *var)
 {
-#ifdef WITH_ASPELL
+#ifdef HAVE_LIBASPELL
 	/* probujemy zainicjowac jeszcze raz aspell'a */
 	if (!in_autoexec)
 		ncurses_spellcheck_init();
@@ -412,7 +409,7 @@ static void ncurses_changed_aspell(const char *var)
 
 static QUERY(ncurses_postinit)
 {
-#ifdef WITH_ASPELL
+#ifdef HAVE_LIBASPELL
 	ncurses_spellcheck_init();
 #endif
 	ncurses_contacts_changed(NULL);
@@ -426,7 +423,7 @@ static QUERY(ncurses_binding_set_query)
 	int quiet = va_arg(ap, int);
 
 	ncurses_binding_set(quiet, p1, p2);
-	
+
 	return 0;
 }
 
@@ -449,10 +446,10 @@ static QUERY(ncurses_binding_adddelete_query)
 static QUERY(ncurses_lastlog_changed) {
 	window_t *w;
 
-	if (config_lastlog_size < 0) 
+	if (config_lastlog_size < 0)
 		config_lastlog_size = 0;
 
-	if (!(w = window_find_sa(NULL, "__lastlog", 1)))
+	if (!(w = window_exist(WINDOW_LASTLOG_ID)))
 		return 0;
 
 	ncurses_lastlog_new(w);
@@ -463,41 +460,6 @@ static QUERY(ncurses_lastlog_changed) {
 	return 0;
 }
 
-static QUERY(ncurses_ui_window_lastlog) {
-	window_t *w;
-	ncurses_window_t *n;
-
-	int lock_old = config_lastlog_lock;
-	int retval;
-
-	if (!(w = window_find_sa(NULL, "__lastlog", 1)))
-		w = window_new("__lastlog", NULL, 1001);
-
-	n = w->priv_data;
-
-	if (!n || !n->handle_redraw) {
-		debug_error("ncurses_ui_window_lastlog() BAD __lastlog wnd?\n");
-		return -1;
-	}
-
-	config_lastlog_lock = 0;
-	if (!(retval = n->handle_redraw(w)) && !config_lastlog_noitems) {	/* if we don't want __backlog wnd when no items founded.. */
-		/* destroy __backlog */
-		window_kill(w);
-		config_lastlog_lock = lock_old;
-/* XXX bugnotes, when killing visible w->floating window we should do: implement in window_kill() */
-		ncurses_resize();
-		ncurses_commit();
-		return 0;
-	}
-
-	n->start = n->lines_count - w->height + n->overflow;
-	config_lastlog_lock = 1;
-	ncurses_redraw(w);
-	config_lastlog_lock = lock_old;
-	return retval;
-}
-
 static QUERY(ncurses_setvar_default)
 {
 	config_contacts_size = 9;	  /* szeroko okna kontaktw */
@@ -521,6 +483,7 @@ static QUERY(ncurses_setvar_default)
 	config_contacts_metacontacts_swallow = 1;
 
 	config_backlog_size = 1000;	    /* maksymalny rozmiar backloga */
+	config_backlog_scroll_half_page = 1;	    /* tryb przewijania: p ekranu lub ekran bez jednej linii */
 	config_display_transparent = 1;     /* czy chcemy przezroczyste to? */
 	config_kill_irc_window = 1;	    /* czy zamyka kanay ircowe przez alt-k? */
 	config_statusbar_size = 1;
@@ -528,7 +491,7 @@ static QUERY(ncurses_setvar_default)
 	config_enter_scrolls = 0;
 	config_margin_size = 15;
 	config_mark_on_window_change = 0;
-#ifdef WITH_ASPELL
+#ifdef HAVE_LIBASPELL
 	xfree(config_aspell_lang);
 
 	config_aspell_lang = xstrdup("pl");
@@ -539,7 +502,7 @@ static QUERY(ncurses_setvar_default)
 /*
  * ncurses_display_transparent_changed ()
  *
- * called when var display_transparent is changed 
+ * called when var display_transparent is changed
  */
 static void ncurses_display_transparent_changed(const char *var)
 {
@@ -553,7 +516,7 @@ static void ncurses_display_transparent_changed(const char *var)
 		background = COLOR_BLACK;
 		assume_default_colors(COLOR_WHITE, COLOR_BLACK);
 	}
-	init_pair(7, COLOR_BLACK, background); 
+	init_pair(7, COLOR_BLACK, background);
 	init_pair(1, COLOR_RED, background);
 	init_pair(2, COLOR_GREEN, background);
 	init_pair(3, COLOR_YELLOW, background);
@@ -616,7 +579,7 @@ static COMMAND(ncurses_cmd_dump) {
 		} else if (!fname)
 			fname = params[i];
 		else {	// fname again?
-			printq("invalid_params", name);
+			printq("invalid_params", name, params[i]);
 			return -1;
 		}
 	}
@@ -638,11 +601,7 @@ static COMMAND(ncurses_cmd_dump) {
 		fstring_t *backlog = n->backlog[i-1];
 		/* XXX, kolorki gdy user chce */
 
-#ifdef USE_UNICODE
-		fprintf(f, "%ld %ls\n", backlog->ts, backlog->str.w);
-#else
-		fprintf(f, "%ld %s\n", backlog->ts, backlog->str.b);
-#endif
+		fprintf(f, "%ld %s\n", backlog->ts, backlog->str);
 	}
 
 	fclose(f);
@@ -654,15 +613,9 @@ static int ncurses_theme_init() {
 #ifndef NO_DEFAULT_THEME
 	/* prompty i statusy dla ui-ncurses */
 	format_add("ncurses_prompt_none", "", 1);
-	format_add("ncurses_prompt_query", "[%1] ", 1);
-	format_add("statusbar", " %c(%w%{time}%c)%w %c(%w%{?session %{?away %G}%{?avail %Y}%{?chat %W}%{?dnd %K}%{?xa %g}%{?gone %R}"
-			"%{?invisible %C}%{?notavail %r}%{session}}%{?!session ---}%c) %{?window (%wwin%c/%w%{?typing %C}%{window}}"
-			"%{?query %c:%W%{query}}%{?debug %c(%Cdebug}%c)%w%{?activity  %c(%wact%c/%W}%{activity}%{?activity %c)%w}"
-			"%{?mail  %c(%wmail%c/%w}%{mail}%{?mail %c)}%{?more  %c(%Gmore%c)}", 1);
-	format_add("header", " %{?query %c(%{?query_away %w}%{?query_avail %W}%{?query_invisible %K}%{?query_notavail %k}"
-			"%{?query_chat %W}%{?query_dnd %K}%{query_xa %g}%{?query_gone %R}%{?query_unknown %M}%{?query_error %m}%{?query_blocking %m}"
-			"%{query}%{?query_descr %c/%w%{query_descr}}%c) %{?query_ip (%wip%c/%w%{query_ip}%c)} %{irctopic}}"
-			"%{?!query %c(%wekg2%c/%w%{version}%c) (%w%{url}%c)}", 1);
+	format_add("ncurses_prompt_query", "[%Y%1%n] ", 1);
+	format_add("statusbar", " %c(%w%{time}%c)%w %c(%w%{?session %{?away %G}%{?avail %Y}%{?chat %W}%{?dnd %K}%{?xa %g}%{?gone %R}%{?invisible %C}%{?notavail %r}%{session}}%{?!session ---}%c) %{?window (%wwin%c/%w%{?typing %C}%{window}}%{?query %c:%W%{query}}%{?debug %c(%Cdebug}%c)%w%{?activity  %c(%wact%c/%W}%{activity}%{?activity %c)%w}%{?mail  %c(%wmail%c/%w}%{mail}%{?mail %c)}%{?more  %c(%Gmore%c)}", 1);
+	format_add("header", " %{?query %c(%{?query_away %w}%{?query_avail %W}%{?query_invisible %K}%{?query_notavail %k}%{?query_chat %W}%{?query_dnd %K}%{query_xa %g}%{?query_gone %R}%{?query_unknown %M}%{?query_error %m}%{?query_blocking %m}%{query}%{?query_descr %c/%w%{query_descr}}%c) %{?query_ip (%wip%c/%w%{query_ip}%c)} %{irctopic}}%{?!query %c(%wekg2%c/%w%{version}%c) (%w%{url}%c)}", 1);
 	format_add("statusbar_act_important", "%Y", 1);
 	format_add("statusbar_act_important2us", "%W", 1);
 	format_add("statusbar_act", "%K", 1);
@@ -671,12 +624,16 @@ static int ncurses_theme_init() {
 	format_add("statusbar_act_important2us_typing", "%C", 1);
 	format_add("statusbar_timestamp", "%H:%M", 1);
 
-#ifdef WITH_ASPELL
+	/* lastlog */
+	format_add("lastlog_title",	_("%) %gLastlog [%B%2%n%g] from window: %W%T%1%n"), 1);
+	format_add("lastlog_title_cur", _("%) %gLastlog [%B%2%n%g] from window: %W%T%1 (*)%n"), 1);
+
+#ifdef HAVE_LIBASPELL
 	/* aspell */
 	format_add("aspell_init", "%> Please wait while initiating spellcheck...", 1);
 	format_add("aspell_init_success", "%> Spellcheck initiated.", 1);
 	format_add("aspell_init_error", "%! Spellcheck error: %T%1%", 1);
-#endif 
+#endif
 #endif
 	return 0;
 }
@@ -690,68 +647,74 @@ EXPORT int ncurses_plugin_init(int prio)
 
 	PLUGIN_CHECK_VER("ncurses");
 
-	query_emit_id(NULL, UI_IS_INITIALIZED, &is_UI);
+	query_emit(NULL, "ui-is-initialized", &is_UI);
 
-	if (is_UI) 
+	if (is_UI)
 		return -1;
 	plugin_register(&ncurses_plugin, prio);
 
+	query_register("ui-window-update-lastlog", QUERY_ARG_END);
+
 	ncurses_setvar_default(NULL, dummy);
 
-	query_connect_id(&ncurses_plugin, SET_VARS_DEFAULT, ncurses_setvar_default, NULL);
-	query_connect_id(&ncurses_plugin, UI_BEEP, ncurses_beep, NULL);
-	query_connect_id(&ncurses_plugin, UI_IS_INITIALIZED, ncurses_ui_is_initialized, NULL);
-	query_connect_id(&ncurses_plugin, UI_WINDOW_SWITCH, ncurses_ui_window_switch, NULL);
-	query_connect_id(&ncurses_plugin, UI_WINDOW_PRINT, ncurses_ui_window_print, NULL);
-	query_connect_id(&ncurses_plugin, UI_WINDOW_NEW, ncurses_ui_window_new, NULL);
-	query_connect_id(&ncurses_plugin, UI_WINDOW_KILL, ncurses_ui_window_kill, NULL);
-	query_connect_id(&ncurses_plugin, UI_WINDOW_TARGET_CHANGED, ncurses_ui_window_target_changed, NULL);
-	query_connect_id(&ncurses_plugin, UI_WINDOW_ACT_CHANGED, ncurses_ui_window_act_changed, NULL);
-	query_connect_id(&ncurses_plugin, UI_WINDOW_REFRESH, ncurses_ui_window_refresh, NULL);
-	query_connect_id(&ncurses_plugin, UI_WINDOW_CLEAR, ncurses_ui_window_clear, NULL);
-	query_connect_id(&ncurses_plugin, UI_WINDOW_UPDATE_LASTLOG, ncurses_ui_window_lastlog, NULL);
-	query_connect_id(&ncurses_plugin, UI_REFRESH, ncurses_ui_refresh, NULL);
-	query_connect_id(&ncurses_plugin, UI_PASSWORD_INPUT, ncurses_password_input, NULL);
-	query_connect_id(&ncurses_plugin, SESSION_ADDED, ncurses_statusbar_query, NULL);
-	query_connect_id(&ncurses_plugin, SESSION_REMOVED, ncurses_statusbar_query, NULL);
-	query_connect_id(&ncurses_plugin, SESSION_EVENT, ncurses_statusbar_query, NULL);
-	query_connect_id(&ncurses_plugin, SESSION_RENAMED, ncurses_statusbar_query, NULL);
-	query_connect_id(&ncurses_plugin, BINDING_SET, ncurses_binding_set_query, NULL);
-	query_connect_id(&ncurses_plugin, BINDING_COMMAND, ncurses_binding_adddelete_query, NULL);
-	query_connect_id(&ncurses_plugin, BINDING_DEFAULT, ncurses_binding_default, NULL);
-	query_connect_id(&ncurses_plugin, VARIABLE_CHANGED, ncurses_variable_changed, NULL);
-	query_connect_id(&ncurses_plugin, CONFERENCE_RENAMED, ncurses_conference_renamed, NULL);
+	query_connect(&ncurses_plugin, "set-vars-default", ncurses_setvar_default, NULL);
+	query_connect(&ncurses_plugin, "ui-beep", ncurses_beep, NULL);
+	query_connect(&ncurses_plugin, "ui-is-initialized", ncurses_ui_is_initialized, NULL);
+	query_connect(&ncurses_plugin, "ui-window-switch", ncurses_ui_window_switch, NULL);
+	query_connect(&ncurses_plugin, "ui-window-print", ncurses_ui_window_print, NULL);
+	query_connect(&ncurses_plugin, "ui-window-new", ncurses_ui_window_new, NULL);
+	query_connect(&ncurses_plugin, "ui-window-kill", ncurses_ui_window_kill, NULL);
+	query_connect(&ncurses_plugin, "ui-window-target-changed", ncurses_ui_window_target_changed, NULL);
+	query_connect(&ncurses_plugin, "ui-window-act-changed", ncurses_ui_window_act_changed, NULL);
+	query_connect(&ncurses_plugin, "ui-window-refresh", ncurses_ui_window_refresh, NULL);
+	query_connect(&ncurses_plugin, "ui-window-clear", ncurses_ui_window_clear, NULL);
+	query_connect(&ncurses_plugin, "ui-refresh", ncurses_ui_refresh, NULL);
+#if 0
+	query_connect(&ncurses_plugin, "ui-password-input", ncurses_password_input, NULL);
+#endif
+	query_connect(&ncurses_plugin, "session-added", ncurses_statusbar_query, NULL);
+	query_connect(&ncurses_plugin, "session-removed", ncurses_statusbar_query, NULL);
+	query_connect(&ncurses_plugin, "session-event", ncurses_statusbar_query, NULL);
+	query_connect(&ncurses_plugin, "session-renamed", ncurses_statusbar_query, NULL);
+	query_connect(&ncurses_plugin, "binding-set", ncurses_binding_set_query, NULL);
+	query_connect(&ncurses_plugin, "binding-command", ncurses_binding_adddelete_query, NULL);
+	query_connect(&ncurses_plugin, "binding-default", ncurses_binding_default, NULL);
+	query_connect(&ncurses_plugin, "variable-changed", ncurses_variable_changed, NULL);
+	query_connect(&ncurses_plugin, "conference-renamed", ncurses_conference_renamed, NULL);
 
-	query_connect_id(&ncurses_plugin, CONFIG_POSTINIT, ncurses_postinit, NULL);
-	query_connect_id(&ncurses_plugin, PROTOCOL_DISCONNECTING, ncurses_session_disconnect_handler, NULL);
+	query_connect(&ncurses_plugin, "config-postinit", ncurses_postinit, NULL);
+	query_connect(&ncurses_plugin, "protocol-disconnecting", ncurses_session_disconnect_handler, NULL);
 
 /* redraw userlisty: */
 	/* podanie czegokolwiek jako data do ncurses_all_contacts_changed() powoduje wyzerowanie n->start */
 
-	query_connect_id(&ncurses_plugin, UI_REFRESH, ncurses_all_contacts_changed, (void *) 1);
-	query_connect_id(&ncurses_plugin, USERLIST_REFRESH, ncurses_all_contacts_changed, NULL /* ? */);
+	query_connect(&ncurses_plugin, "ui-refresh", ncurses_all_contacts_changed, (void *) 1);
+	query_connect(&ncurses_plugin, "userlist-refresh", ncurses_all_contacts_changed, NULL /* ? */);
 
-	query_connect_id(&ncurses_plugin, SESSION_CHANGED, ncurses_all_contacts_changed, (void *) 1);
-	query_connect_id(&ncurses_plugin, SESSION_EVENT, ncurses_all_contacts_changed, NULL);
+	query_connect(&ncurses_plugin, "session-changed", ncurses_all_contacts_changed, (void *) 1);
+	query_connect(&ncurses_plugin, "session-event", ncurses_all_contacts_changed, NULL);
 
-	query_connect_id(&ncurses_plugin, METACONTACT_ADDED, ncurses_all_contacts_changed, NULL);
-	query_connect_id(&ncurses_plugin, METACONTACT_REMOVED, ncurses_all_contacts_changed, NULL);
-	query_connect_id(&ncurses_plugin, METACONTACT_ITEM_ADDED, ncurses_all_contacts_changed, NULL);
-	query_connect_id(&ncurses_plugin, METACONTACT_ITEM_REMOVED, ncurses_all_contacts_changed, NULL);
+	query_connect(&ncurses_plugin, "metacontact-added", ncurses_all_contacts_changed, NULL);
+	query_connect(&ncurses_plugin, "metacontact-removed", ncurses_all_contacts_changed, NULL);
+	query_connect(&ncurses_plugin, "metacontact-item-added", ncurses_all_contacts_changed, NULL);
+	query_connect(&ncurses_plugin, "metacontact-item-removed", ncurses_all_contacts_changed, NULL);
 
-	query_connect_id(&ncurses_plugin, USERLIST_CHANGED, ncurses_all_contacts_changed, NULL);
-	query_connect_id(&ncurses_plugin, USERLIST_ADDED, ncurses_all_contacts_changed, NULL);
-	query_connect_id(&ncurses_plugin, USERLIST_REMOVED, ncurses_all_contacts_changed, NULL);
-	query_connect_id(&ncurses_plugin, USERLIST_RENAMED, ncurses_all_contacts_changed, NULL);
+	query_connect(&ncurses_plugin, "userlist-changed", ncurses_all_contacts_changed, NULL);
+	query_connect(&ncurses_plugin, "userlist-added", ncurses_all_contacts_changed, NULL);
+	query_connect(&ncurses_plugin, "userlist-removed", ncurses_all_contacts_changed, NULL);
+	query_connect(&ncurses_plugin, "userlist-renamed", ncurses_all_contacts_changed, NULL);
 
-	command_add(&ncurses_plugin, ("mark"), NULL, cmd_mark, 0, "-a --all");
-	command_add(&ncurses_plugin, ("dump"), NULL, ncurses_cmd_dump, 0, "-a --append -w --window");
+	command_add(&ncurses_plugin, ("mark"), "p", cmd_mark, 0, "-a --all");
+	command_add(&ncurses_plugin, ("dump"), "pf pf pf", ncurses_cmd_dump, 0, "-a --append -w --window");
+	command_add(&ncurses_plugin, ("lastlog"), "p? p? p? p? p?", ncurses_cmd_lastlog, 0, 
+		"-c --caseinsensitive -C --CaseSensitive -s --substring -r --regex -R --extended-regex -w --window");
 
-#ifdef WITH_ASPELL
+#ifdef HAVE_LIBASPELL
 	variable_add(&ncurses_plugin, ("aspell"), VAR_BOOL, 1, &config_aspell, ncurses_changed_aspell, NULL, NULL);
 	variable_add(&ncurses_plugin, ("aspell_lang"), VAR_STR, 1, &config_aspell_lang, ncurses_changed_aspell, NULL, NULL);
 #endif
 	variable_add(&ncurses_plugin, ("backlog_size"), VAR_INT, 1, &config_backlog_size, changed_backlog_size, NULL, NULL);
+	variable_add(&ncurses_plugin, ("backlog_scroll_half_page"), VAR_BOOL, 1, &config_backlog_scroll_half_page, NULL, NULL, NULL);
 	/* this isn't very nice solution, but other solutions would require _more_
 	 * changes...
 	 */
@@ -763,13 +726,22 @@ EXPORT int ncurses_plugin_init(int prio)
 	variable_add(&ncurses_plugin, ("contacts_groups"), VAR_STR, 1, &config_contacts_groups, ncurses_contacts_changed, NULL, dd_contacts);
 	variable_add(&ncurses_plugin, ("contacts_groups_all_sessons"), VAR_BOOL, 1, &config_contacts_groups_all_sessions, ncurses_contacts_changed, NULL, dd_contacts);
 	variable_add(&ncurses_plugin, ("contacts_margin"), VAR_INT, 1, &config_contacts_margin, ncurses_contacts_changed, NULL, dd_contacts);
+	variable_add(&ncurses_plugin, ("contacts_vertical_margin"), VAR_INT, 1, &config_contacts_vertical_margin, ncurses_contacts_changed, NULL, dd_contacts);
 	variable_add(&ncurses_plugin, ("contacts_metacontacts_swallow"), VAR_BOOL, 1, &config_contacts_metacontacts_swallow, (void (*)(const char *))ncurses_all_contacts_changed, NULL, dd_contacts);
 	variable_add(&ncurses_plugin, ("contacts_order"), VAR_STR, 1, &config_contacts_order, ncurses_contacts_changed, NULL, dd_contacts);
 	variable_add(&ncurses_plugin, ("contacts_orderbystate"), VAR_BOOL, 1, &config_contacts_orderbystate, ncurses_contacts_changed, NULL, dd_contacts);
 	variable_add(&ncurses_plugin, ("contacts_size"), VAR_INT, 1, &config_contacts_size, ncurses_contacts_changed, NULL, dd_contacts);
 	variable_add(&ncurses_plugin, ("contacts_wrap"), VAR_BOOL, 1, &config_contacts_wrap, ncurses_contacts_changed, NULL, dd_contacts);
-	variable_add(&ncurses_plugin, ("lastlog_size"), VAR_INT, 1, &config_lastlog_size, (void (*)(const char *))ncurses_lastlog_changed, NULL, NULL);
+
+	variable_add(&ncurses_plugin, ("lastlog_display_all"), VAR_INT, 1, &config_lastlog_display_all, NULL, variable_map(3, 
+			0, 0, "current window",
+			1, 2, "current window + configured",
+			2, 1, "all windows + configured"), NULL);
 	variable_add(&ncurses_plugin, ("lastlog_lock"), VAR_BOOL, 1, &config_lastlog_lock, NULL, NULL, NULL);
+	variable_add(&ncurses_plugin, ("lastlog_matchcase"), VAR_BOOL, 1, &config_lastlog_case, NULL, NULL, NULL);
+	variable_add(&ncurses_plugin, ("lastlog_noitems"), VAR_BOOL, 1, &config_lastlog_noitems, NULL, NULL, NULL);
+	variable_add(&ncurses_plugin, ("lastlog_size"), VAR_INT, 1, &config_lastlog_size, (void (*)(const char *))ncurses_lastlog_changed, NULL, NULL);
+
 	variable_add(&ncurses_plugin, ("display_transparent"), VAR_BOOL, 1, &config_display_transparent, ncurses_display_transparent_changed, NULL, NULL);
 	variable_add(&ncurses_plugin, ("enter_scrolls"), VAR_BOOL, 1, &config_enter_scrolls, NULL, NULL, NULL);
 	variable_add(&ncurses_plugin, ("header_size"), VAR_INT, 1, &config_header_size, header_statusbar_resize, NULL, NULL);
@@ -783,8 +755,25 @@ EXPORT int ncurses_plugin_init(int prio)
 
 	variable_add(&ncurses_plugin, ("typing_interval"), VAR_INT, 1, &config_typing_interval, ncurses_typing_retimer, NULL, NULL);
 	variable_add(&ncurses_plugin, ("typing_timeout"), VAR_INT, 1, &config_typing_timeout, NULL, NULL, NULL);
-	variable_add(&ncurses_plugin, ("typing_timeout_empty"), VAR_INT, 1, &config_typing_timeout_empty, NULL, NULL, NULL);
-	
+	variable_add(&ncurses_plugin, ("typing_timeout_inactive"), VAR_INT, 1, &config_typing_timeout_inactive, NULL, NULL, NULL);
+
+	{ /* initialize locale-related vars */
+		const gchar utf8hellip[] = { 0xe2, 0x80, 0xa6 };
+		const gchar asciihellip[] = { '.', '.', '.' };
+
+		ncurses_hellip = g_locale_from_utf8(utf8hellip, sizeof(utf8hellip),
+				NULL, NULL, NULL);
+		if (!ncurses_hellip) {
+				/* fallback to asciihellip */
+			ncurses_hellip = g_locale_from_utf8(asciihellip, sizeof(asciihellip),
+					NULL, NULL, NULL);
+
+				/* failure here? you're joking, right?
+				 * well, better die early. */
+			g_assert(ncurses_hellip);
+		}
+	}
+
 	have_winch_pipe = 0;
 #ifdef SIGWINCH
 	if (pipe(winch_pipe) == 0) {
@@ -801,7 +790,7 @@ EXPORT int ncurses_plugin_init(int prio)
 	timer_add(&ncurses_plugin, "ncurses:clock", 1, 1, ncurses_statusbar_timer, NULL);
 
 	ncurses_init();
-	
+
 	header_statusbar_resize(NULL);
 	ncurses_typing_retimer(NULL);
 
@@ -822,8 +811,7 @@ EXPORT int ncurses_plugin_init(int prio)
 			ncurses_settitle = 1;
 	}
 
-	if (ncurses_settitle)
-		printf(ncurses_settitle_formats[ncurses_settitle], "", "", "EKG2");
+	ncurses_set_title("", "", "EKG2");
 
 	return 0;
 }
@@ -833,15 +821,18 @@ static int ncurses_plugin_destroy()
 	ncurses_plugin_destroyed = 1;
 	ncurses_initialized = 0;
 
-	ncurses_disable_mouse(); 
+	ncurses_disable_mouse();
 
 	watch_remove(&ncurses_plugin, 0, WATCH_READ);
 	if (have_winch_pipe)
 		watch_remove(&ncurses_plugin, winch_pipe[0], WATCH_READ);
 
 	timer_remove(&ncurses_plugin, "ncurses:clock");
+	if (redraw_timer_id>0)
+		g_source_remove(redraw_timer_id);
 
 	ncurses_deinit();
+	g_free(ncurses_hellip);
 
 	plugin_unregister(&ncurses_plugin);
 
