@@ -3,8 +3,11 @@
 
 extern int ncurses_typing_mod;			/* whether buffer was modified */
 extern window_t *ncurses_typing_win;		/* last window for which typing was sent */
+extern int config_typing_timeout;
+extern int config_typing_timeout_inactive;
+extern int config_typing_interval;
 
-void ncurses_window_gone(window_t *w);
+int ncurses_typingsend(window_t *w, int chatstate);
 TIMER(ncurses_typing);
 
 #endif
\ No newline at end of file
