@@ -17,21 +17,16 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "ekg2-config.h"
+#include "ekg2.h"
 
-#include <stdint.h>
-
-#include <ekg/stuff.h>
-#include <ekg/xmalloc.h>
-
-uint32_t rivchat_fix32(uint32_t x) {
+guint32 rivchat_fix32(guint32 x) {
 	return x;
 /*
-	return (uint32_t)
-		(((x & (uint32_t) 0x000000ffU) << 24) |
-		((x & (uint32_t) 0x0000ff00U) << 8) |
-		((x & (uint32_t) 0x00ff0000U) >> 8) |
-		((x & (uint32_t) 0xff000000U) >> 24));
+	return (guint32)
+		(((x & (guint32) 0x000000ffU) << 24) |
+		((x & (guint32) 0x0000ff00U) << 8) |
+		((x & (guint32) 0x00ff0000U) >> 8) |
+		((x & (guint32) 0xff000000U) >> 24));
 */
 }
 
